/*
 * Scheme for system demostration. 
 * This data base shows a simple stock system,
 * which generates customer's invoices.
 */

create schema demo descr "Demostration Schema"
                   language "C";

table items             descr "Stock Items" {

    itemno  num(4)      descr "Item Code"
                        primary key,
    dsc     char(45)    descr "Description",
    cost    num(14,2)   descr "Wholesale price",
    prvta   num(14,2)   descr "List price",
    cantid  num(4)      descr "Quantity in Stock"
                        not null
                        >=0
};

table clientes          descr "Customers" {

    clieno  num(4)      descr "Customer Code"
                        primary key,
    nombre  char(30)    descr "Surname and first name"
                        not null,
    direc   char(30)    descr "Address",
    prov    char        descr "Province or state"
                        in provin:nombre,
    local   num(2)      descr "City"
                        in local(prov):nombre,
    cp      char(4)     descr "Postal code" mask "4N",
    fealta  date        descr "Date of Addition"
                        <= today  default today,
    iva     num(4,2)    descr "Value Added Tax"
                        not null
                        default 0
}
index nombre (nombre(0,8));

table ordenes           descr "Invoice heading" {

    orderno num(4)      descr "Invoice number"
                        primary key,
    fecha   date        descr "Date"
                        <= today
                        default today,
    clieno  num(4)      descr "Customer"
                        in clientes:(nombre),
    valor   num(14,2)   descr "Subtotal",
    miva    num(14,2)   descr "Value Added Tax",
    mentr   num(1)      descr "Method of Shipping"
                        in (0:"Client Pick-Up",
                            1:"Delivery to Client",
                            2:"Third Party Pick-Up")
                        not null
                        default 1,
    direc   char(50)    descr "1-Delivery Address. 2-Information of Third Party."
};

table movim             descr "Invoice Lines" {

    orderno num(4)      descr "Invoice number"
						in ordenes,
    itemno  num(4)      descr "Item Code"
                        in items:dsc,
    cantid  num(4)      descr "Quantity"
                        not null
                        > 0,
    prvta   num(14,2)   descr "List Price"
}
primary key (orderno, itemno);

table provin[24]        descr "Provinces or States" {

    codpro     char     descr "Provincial or state code"
                        not null
                        primary key,
    nombre  char(20)    descr "Name of State or province"
                        not null
};

table local             descr "City" {

    codpro     char     descr "State Code"
                        in provin:nombre,
    codloc    num(2)    descr "City Code",
    nombre    char(20)  descr "City name"
                        not null
}
primary key (codpro, codloc);

//
// These lines define Data Base User Privileges
//
grant manip on schema demo to public;
grant all on clientes, items, ordenes, movim, provin, local
      to public;

