/*
 * Schema para Base de Datos del Sistema demostracin. 
 * La base de datos describe un sistema muy sencillo de stock,
 * que permite generar facturas a los clientes.
 */

create schema demo descr "Esquema demostracin"
                   language "c";

table items             descr "Codificacin de items en stock" {

    itemno  num(4)      descr "Nmero de item"
                        primary key,
    dsc     char(45)    descr "Descripcin",
    costo   num(6,2)   descr "Costo de adquisicin",
    prvta   num(6,2)   descr "Precio final de Venta"
                        default (costo * 1.20),
    cantid  num(4)      descr "Cantidad en stock"
                        not null
                        >=0,
};

table clientes          descr "Clientes de la Compaa" {

    clieno  num(4)      descr "Nmero de cliente"
    					primary key,
    nombre  char(30)    descr "Nombre del Cliente"
                        not null,
    direc   char(30)    descr "Direccin"
						default ($empresa),
    prov    char        descr "Provincia"
    					mask ">A"
						default "X"
                        in provin,
    loc     num(2)      descr "Localidad"
                        in local(prov):nombre,
    cp      char(4)     descr "Cdigo postal" mask "4N",
    fealta  date        descr "Fecha de alta"
                        default today
                        check (this between today - 90 and today),
    iva     num(4,2)    descr "I.V.A."
                        not null
                        default 18,
	limite	num(6,2)	descr "Lmite de crdito",
}
index nombre (nombre(0,8));

table ordenes           descr "Cabeceras de las facturas" {
    orderno num(4)      descr "Nmero de factura"
                        primary key,
    fecha   date        descr "Fecha"
                        <= today
                        default today,
    clieno  num(4)      descr "Cliente"
    					not null
                        in clientes:(nombre),
    valor   num(14,2)   descr "Valor neto",
    miva    num(14,2)   descr "Monto de IVA",
    mentr   num(1)      descr "Modo de entrega"
                        in (0:"Retira directamente el Cliente",
                            1:"Envo a domicilio",
                            2:"Retira un tercero")
                        not null
                        default 1,
    direc   char(50)    descr "1-Direccin entrega. 2-Datos de quien retira.",
};

table movim             descr "Renglones de las facturas." {

    orderno num(4)      descr "Nmero de factura"
						in ordenes,
    itemno  num(4)      descr "item"
                        in items:dsc,
    cantid  num(4)      descr "Cantidad comprada"
                        not null
                        > 0,
    prvta   num(6,2)   descr "Precio de venta unitario",
    total	num(14,2)  descr "Total item",
}
primary key (orderno, itemno);

table provin[24]        descr "Provincias" {

    codpro     char     descr "Letra (segn patente automotores)"
                        not null
                        mask ">A"
                        primary key,
    nombre  char(20)    descr "Nombre de la provincia"
                        not null
};

table local             descr "Localidades por provincia" {

    codpro     char     descr "Provincia"
                        in provin:nombre,
    codloc    num(2)    descr "Cdigo de la localidad",

    nombre    char(20)  descr "Nombre"
                        not null
}
primary key (codpro, codloc);


//
// Estas sentencias definen los permisos de los usuarios respecto
// de esta base de datos
//
grant manip on schema demo to public;
grant all on clientes, items, ordenes, movim, provin, local
      to public;

