/********************************************************************
* Copyright (c) 1987 InterSoft Co.  All Rights Reserved
* THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF InterSoft Co.
* The copyright notice above does not evidence any
* actual or intended publication of such source code.
*
* $Id: reqconn.h,v 1.2 1996/07/03 13:39:28 hernan Exp $
*
* DESCRIPTION:
*********************************************************************/

#ifndef REQCONN_H
#define REQCONN_H

#ifdef __GNUC__
#pragma interface
#endif

#include <ifound/stack.h>
#include <ifound/priv/pconfig.h>
#include <essentia/netconn.h>    

class FramedObject;
class gtype(PtrArray, FramedObject);

struct RequestInfo {
	Int object;
	Short command;
};

class gtype(Stack, RequestInfo);
declare(Stack, RequestInfo); 

class RequestConnection : public NetConnection
{
public:

	explicit RequestConnection(const String &, const String &, bool execSv = true);
	explicit RequestConnection(int handle = 0, ConvType ct = NO_CONVERT);
	explicit RequestConnection(const String &service, bool local = true);
	~RequestConnection();

	enum MsgType {
		Info,
		Request,
		Reply
	};
	
	Int currObject() const;
	Short currCommand() const;
	MsgType messageType() const;
	bool needReply() const;
	bool isReply() const;
	
	operator bool() const;

	// to send
	RequestConnection &operator()(Short cmd, FramedObject *obj = NULL);
	RequestConnection &request(Short cmd, FramedObject *obj = NULL);
	RequestConnection &reply();

	void endMessage();
	void endReply();
	void flush();
    
    RequestConnection &waitForReply();
    
	// to read
	bool nextRequest();		// return false if is reply (should exit loop)
	void dispatch();

	Int verboseLevel() const;
	void verboseLevel(Int vl);
	
	FramedObject *findObject(Int id);

	void setDefaultObject(FramedObject *obj);
	Int addObject(FramedObject *obj);
	void delObject(FramedObject *obj);
	
	bool canWrite();
	bool canRead();
	bool empty();

protected:
	void _end();

	void dumpState(const String &mess, Int level = 30) const;
	
	bool isWriting() const;
	bool isReading() const;
	bool notLast() const;

	virtual void _flush();
	virtual Int _fill();
	virtual void flushBuff();
	virtual void fillBuff();
	
	void start(FramedObject *obj, Short cmd, MsgType type);
	void start(Int obj, Short cmd, MsgType type);
	
private:
	typedef NetConnection inherited;

	void initValues();

	bool connected(FramedObject *) const;

    // objects related to this connection
    enum {
    	DefaultId = 0,
    	NotLast = 0x80000000,
    	IdMask = ~NotLast
    };
	Int lastId_d;
	gtype(PtrArray, FramedObject) &objs_d;
	
	Int verboseLevel_d;

	// current request header data
	Int currObj_d;
	Short currCmd_d;
	MsgType msgType_d;
	
	Stack(RequestInfo) requests_d;

	// current state of buffer
	enum {
		Empty,
		Reading,
		Writing
	} state_d;

	MsgBuff currBuff_d;
	bool appendHeader_d;
	bool wasRemoted_d;

	inline MsgBuff currBuff() const;
	inline bool appendHeader() const;
	inline bool wasRemoted() const;

	// to hide these names
	void flushResult(Int);
	short getOp();
	void putC(char c);
	char getC();
	bool moreData();
	
	void SetBuffer(MsgBuff);
	bool canAppendHeader();
	void Transmit();
	void RmtBuff();
	void RestartBuffer(MsgBuff);
	void Flush();
	void RewindBuffer();
};

#include <cracker/reqconn.icc>

#endif 











