/********************************************************************
* Copyright (c) 1992 InterSoft Co.  All Rights Reserved
* THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF InterSoft Co.
* The copyright notice above does not evidence any
* actual or intended publication of such source code.
*
* $Id: cnffile.h,v 1.16 1996/08/06 21:29:32 gustavof Exp $
*
* DESCRIPTION
* Class definitions for general Config File.
*********************************************************************/

#ifndef CNFFILE_H
#define CNFFILE_H

#include <ifound.h>

#ifdef __GNUC__
#pragma interface
#endif

#include <ifound/str.h>

// configuracion de los directorios
#define CONFIG_DIR		"ESSENTIA"
#define SERVERS			"SERVERS"
#define SV_SUBDIR		PATH_SLASH"servers"PATH_SLASH

// connection types
#define ICON			'i'
#define RCON			'r'

// server configuration file extensions
#define IEXT			".sv"
#define REXT			".rsv"

#define SCH_SEPAR		","
#define DBASE_SEPAR		":"
#define SERVER_SEPAR	";"

#include <stdio.h>

enum Token {
	CACHESIZE,
	CKPOINTFREQ,
	CKPOINTLOWBOUND,
	CKPOINTUPBOUND,
	CORRUPTIONTOLERANT,
	HANDLEVERSIONS,
	HKEEPINGMSGBOUND,
	HOST,
	JOURNALBUFFER,
	JOURNALDIR,
	LOCKS,
	LOCKSPERCLIENT,
	LOCKUNIFYBOUND,
	NAMESPACES,
	NEXTSCHEMAID,
	NOTIFIES,
	RUNLOGFILE,
	SCHEMAS,
	SERVERIDENT,
	STATEFILE,
	DBASE,
	MIRRORING,
	BACKUP,
	JOURNALFLUSHFREQ,
	TRANSTIMEOUT,
	LOCKWAIT,
	PENDLOCKNOTIFTIME,
	MATURETIME,
	OBJECTREPOSITORY,
	MAXATTACHSPCLIENT,
	LOCKUNIFYPERCENT,
	DISKCONFIGURATION,
	CACHEPAGESIZE,
	PROFILEFILE,
	PROFILEFREQ
};

class ConfigFile {

// private:

	void clear();

protected:

	String filename;	// configuration file name

	Int serverId;		// server's id
	char ty;			// connection type
	String host;		// server host (for remote servers)

	Int nschs;			// number of schemas attended by server
	String schemas;		// schemas attended by server

	bool dirty_;		// whether it needs to be saved
	bool is_namesp;		// whether it is a namespace or a schema

	String objectRepository_;		// directory for storing objects
	
	static Token findToken(const String &lexeme);

	bool readFile(const String &file);

	virtual void resolveOthers(Token t, const String &tname,
			const String &val, Int);
	virtual void storeOthers(FILE *f);
	
	void checkNullValue(const String &tname, const String &aux, Int line) ;
	
public:

	ConfigFile();
    ConfigFile(const ConfigFile &);
	ConfigFile(char t, const String &hst);
	virtual ~ConfigFile();
		
	const String &getFileName();
	String getServerName() const;
		
	char getType() const;
	Int  getServerId() const;
	const String &getHost() const;

	Int    getNSchs();
	String getSchName(Int i);
	bool   isNameSpace() const;
	const  String &getSchemas();
	bool   delSchema(const String &schname);
	bool   findSchema(const String &schname);
	bool   read(const String &svname, char tt = '\0');
	const String &objectRepository() const;
	bool setObjectRepository(const String &d);	
	operator void*();
	bool operator !();

	void store();
};

#endif // CNFFILE_H
