/********************************************************************
* Copyright (c) 1995 InterSoft Co.  All Rights Reserved
* THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF InterSoft Co.
* The copyright notice above does not evidence any
* actual or intended publication of such source code.
*
* $Id: netconn.h,v 1.6 1996/08/06 19:19:01 gab Exp $
*
* DESCRIPTION:
*
*********************************************************************/

#ifndef NETCONN_H
#define NETCONN_H

#ifdef __GNUC__
#pragma interface
#endif

#include <essentia/connect.h>
#include <essentia/sockconn.h>
    
class CompressedConnection;    
class NetConnection : public Connection {

	friend void setPackageNumber(NetConnection *, Int);
	friend Int getPackageNumber(NetConnection *);

//private:

	// Common
	SocketConnection *scon;
	CompressedConnection *ccon;	
	bool  needConvert;
	short sendOp;

	// Buffer and pointer used by getC/putC
	char mBuff[sizeof(Int)];
	char *pMBuff;

	// For stop recusive use of scon on connection error.
	static bool recursOnError;
	
	// For determine who set and how to get the comm. msg. size (buff. size).
	enum ConnType {
		server,
		client,
		clone,
	};

	bool useCompression;
	inline void reset();
	void createBuffer(ConnType);
	Int _receiveBody();

protected:

	inline bool isNeedConvert() const;
	void Send();
	inline char* PMBuffer() const;

	// Common
	virtual void _flush();
	virtual Int _fill();

public:

	// Client.
	// ------------------------------------------------
	NetConnection(const String &, const String &, bool execSv = true);

	// Server.
	// ------------------------------------------------
	explicit NetConnection(const String &s = NULL_SERVICE,
			bool localCli = true);

	// Common.
	// ------------------------------------------------
	explicit NetConnection(int, ConvType ct = NO_CONVERT); // Clone.
	~NetConnection();

	virtual void flush();
	
	inline void flushResult(Int r);

	static void error(const String &msg, ...);
	inline Status status() const;

	inline static bool isRemoteChannel(Int fd);
	
	// Options handling
	short getOp();
	inline NetConnection &operator()(Int opt);

	inline Int result();

 	// Guiltys : RemoteComand and RemoteExecD.
	inline int sockDescr() const;
	inline int fDescr() const;


	// Parameters handling
	virtual Connection &operator<<(Int);
	virtual Connection &operator<<(const String &s);
	virtual Connection &operator<<(const char *s);

	virtual Connection &operator>>(Int &i);
	virtual Connection &operator>>(String &s);
	virtual Connection &operator>>(char *&cp);

	void putC(char c);
	char getC();

	inline bool moreData();
	
	Int receivedBytes;
	Int sentBytes;
	Int roundTrips;

	
};

inline void NetConnection::reset()
{
	sendOp = ERR;
	rcv_pdata = snd_pdata = buffer->data;
  	snd_cnt = size - 1;
  	rcv_cnt = 0;
}

inline bool NetConnection::moreData()
{
	return rcv_cnt > 0;
}

inline Int NetConnection::result() 
{ 
	Int i;
	*this >> i;
	return i; 
}

inline NetConnection& NetConnection::operator()(Int opt)
{ 
	buffer->h.serverOp = sendOp = opt; 
	return *this; 
}

inline void NetConnection::flushResult(Int r)
{ 
	*this << r; 
	flush(); 
}

inline Status NetConnection::status() const
{ 
	assert(scon); 
	return scon->status(); 
}

inline bool NetConnection::isRemoteChannel(Int fd)
{
	return SocketConnection::isSocket(fd);
}

inline int NetConnection::sockDescr() const 
{ 
	assert(scon); 
	return scon->sockDescr(); 
}                    

int NetConnection::fDescr() const
{
	assert(scon);
	return scon->fDescr();
}


inline bool NetConnection::isNeedConvert() const
{
	return needConvert;
}

inline char *NetConnection::PMBuffer() const
{
	return pMBuff;
}                  

#endif // NETCONN_H

