/********************************************************************
* Copyright (c) 1995 InterSoft Co.  All Rights Reserved
* THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF InterSoft Co.
* The copyright notice above does not evidence any
* actual or intended publication of such source code.
*
* $Id: dbm.h,v 1.8 1996/06/06 21:15:42 pablog Exp $
*
* DESCRIPTION
*
* Database manager API
*
*********************************************************************/

#ifndef DBM_H
#define DBM_H


/**
 * <EM>Transaction</EM> class.
 * @version $Revision: 1.8 $
 * @author $Author: pablog $
 */
class Transaction {

public:

	Transaction();
	~Transaction();
	
	void begin();
	void commit();
	void abort();
};

/**
 * <EM>DataBase</EM> class.
 * @version $Revision: 1.8 $
 * @author $Author: pablog $
 */
class DataBase {

	useIdCheck(DataBase_id);

	friend class DbObject;
	
//private:

	DataBaseRep *rep_d;
	
	inline DataBaseRep *rep();

public:

	DataBase(const String &name);
	~DataBase();

	Int lock(Int l);
	void free();
};

/**
 * <EM>DbObject</EM> class.
 * @version $Revision: 1.8 $
 * @author $Author: pablog $
 */
class DbObject {

	useIdCheck(DbObject_id);

// private:

	DbObjectRep *rep_d;
	DbSet *set_d;

	inline DbObjectRep *rep() const;
	inline DbSet *set();

public:

	DbObject(const String &db, const String &extnt, void *buff = NULL);
	~DbObject();
		
	bool accessBy(const String &name);
	bool accessBy(Int nIndex);
	
	void init();
	void del();
	void add();
	void put();
	Int  get(Int mode = this_key|default_lock, Int nparts = 0);
	void free(Int mode = this_key, Int nparts = 0);
	Int  find(Int mode = this_key, Int nparts = 0);
	
	inline DbSet &extent();
};

/**
 * <EM>DbSet </EM>class.
 * @version $Revision: 1.8 $
 * @author $Author: pablog $
 */
class DbSet {

	useIdCheck(DbSet_id);

	friend class DbObject;
	
// private:

	DbSetRep *rep_x;
	
	inline DbSetRep *rep();
	
	DbSet(DbSetRep *r);
	~DbSet();
	
public:

	Int delAll();
	Int lock(Int lde);
	void free();
};

/**
 * <EM>DbCursor</EM> class.
 * @version $Revision: 1.8 $
 * @author $Author: pablog $
 */
class DbCursor {

	useIdCheck(DbCursor_id);

// private:

	DbCursorRep *rep_d;
		
	inline DbCursorRep *rep();

public:

	DbCursor(DbObject &obj, const String &indName = NULL_STRING, 
			  Int mode = io_not_lock);
	DbCursor(DbObject &obj, const String &txt, int mode = io_not_lock);
	~DbCursor();

	Int fetch();
	Int fetchPrev();
	Int fetchThis();
	void moveFirst();
	void moveLast();
	Int count();

	inline Int operator++();
	inline Int operator--();

	DbCursor &operator<<(const String &name);

	DbCursor &operator<(const AnyValue &av);
	DbCursor &operator<=(const AnyValue &av);
	DbCursor &operator>=(const AnyValue &av);
	DbCursor &operator>(const AnyValue &av);
	DbCursor &operator,(const AnyValue &av);

	void setFromFld(int nfield, const AnyValue &av);
	void setToFld(int nfield, const AnyValue &av);
};

#include <ideafix/dbm.icc>

#endif

