/********************************************************************
* Copyright (c) 1995 InterSoft Co.  All Rights Reserved
* THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF InterSoft Co.
* The copyright notice above does not evidence any
* actual or intended publication of such source code.
*
* $Id: cdbrep.h,v 1.1 1996/07/03 13:46:53 hernan Exp $
*
* DESCRIPTION
*********************************************************************/

#ifndef CDBREP_H
#define CDBREP_H

#include <ideafix.h>
#include <cfix.h>

#include <ideafix/priv/dbrep.h>

inline int schToI(schema sch)
{
	return (int)((sch >> 16) & 0xff);
}

inline schema iToSch(int n)
{
	return (schema)(n << 16);
}

inline int tabToI(dbtable tab)
{
	return (int)(((tab >> 8) & 0xff)-1);
}

inline dbtable iToTab(int n)
{
	return (dbtable)((n+1) << 8);
}

inline int fldToI(dbfield fld)
{
	return (int)((fld & 0xff)-1);
}

inline dbfield iToFld(dbtable tab, int slot)
{
	return (dbfield)((tab) | (slot + 1));
}

inline int indToI(dbindex ind)
{
	return (int)(ind & 0xff);
}

inline int curToI(dbcursor cur)
{
	return (int)(cur & 0xff);
}

inline dbcursor iToCur(dbtable tab, int cur)
{
	return (dbcursor)(tab|cur);
}

inline dbindex iToInd(dbtable tab, int ind)
{
	return (dbindex)(tab|ind);
}

class CDataBaseRep;
class CDbObjectRep;
class CDbSetRep;

CDataBaseRep *findDataBase(Int n);
CDbObjectRep *findObject(Int n);
CDbSetRep *findSet(Int n);
DbMemberRep *findMember(Int f);
DbIndexRep *findIndex(Int n);
DbCursorRep *findCursor(Int n);
Int oldToNewMode(Int mode);
DbMemberRep *findIndexMember(Int ind, Int fld);
AnyValue value(DbMemberRep *m, va_list &ap);
CDataBaseRep *getCDataBaseRep(DataBaseRep *);
CDbObjectRep *getCDbObjectRep(DbObjectRep *);
CDbSetRep *getCDbSetRep(DbSetRep *);

class gtype(PtrArray, CDbObjectRep);

class CDbObjectRep : public DbObjectRep {

// private:

	bool alias_;

public:		

	CDbObjectRep(DbSetRep *set, bool alias = false, void *buff = NULL);
	~CDbObjectRep();
	
	bool isAnAlias()					{ idCheck(); return alias_; }
};

class CDataBaseRep : public DataBaseRep {

// private:

	gtype(PtrArray, CDbObjectRep) &objects_;

public:

	CDataBaseRep(const String &name, const String &descr);
	CDataBaseRep(const String &name, Int mode, Int version = CURRENT_VERSION);
	~CDataBaseRep();

	void addSet(DbSetRep *set);
	void freeSetSlot(DbSetRep *s);

	DbSetRep *newDbSetRep(const String &name, const String &descr, 
						  DbTypeRep *typeRep, Int flags, Int size);
	DbSetRep *newDbSetRep(const String &name = NULL_STRING);

	CDbObjectRep *object(Int n);
	Int objectNo(CDbObjectRep *obj);

	CDbObjectRep *addAlias(DbSetRep *set);
	void delAlias(CDbObjectRep *obj);
	void delAllAlias();
	CDbObjectRep *findAliasSource(CDbObjectRep *aObj);

	CDataBaseRep *getCDataBaseRep()		{ idCheck(); return this; }
};

class CDbSetRep : public DbSetRep {

// private:

	Int		relDepth_;
	Int		relErrFlag_;

public:

	CDbSetRep(DataBaseRep *dataBase, const String &name, const String &descr, 
		   DbTypeRep *typeRep, Int flags, Int size);
	CDbSetRep(DataBaseRep *dataBase, const String &name = NULL_STRING);
	~CDbSetRep();

	Int relDepth()				{ idCheck(); return relDepth_;				}
	Int relErrFlag()			{ idCheck(); return relErrFlag_;			}
	void relErrFlag(Int err)	{ idCheck(); relErrFlag_ = err;				}
	void relDepth(Int d)		{ idCheck(); relDepth_ = d;					}

	CDbSetRep *getCDbSetRep()	{ idCheck(); return this;					}
};

#endif
