/********************************************************************
* Copyright (c) 1994 InterSoft Co.  All Rights Reserved
* THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF InterSoft Co.
* The copyright notice above does not evidence any
* actual or intended publication of such source code.
*
* $Id: dbirep.h,v 1.10 1996/06/06 21:17:09 pablog Exp $
*
* DESCRIPTION
*********************************************************************/

#ifndef DBREP_H
#define DBREP_H

#include <ideafix/dbtype.h>
#include <ideafix/priv/dbtype.h>
#include <ifound/anyvalue.h>

#define OID_NAME		"self"
#define OID_DESCR	"Object Id"
#define CTIME 		"ctime"
#define MTIME 		"mtime"
#define CDATE 		"cdate"
#define MDATE 		"mdate"
#define CUID  			"cuid"
#define MUID  		"muid"
#define LENUID			5

class DbObjectRep;
class DbMemberRep;
class DbTypeRep;
class DbSetRep;
class DbIndexRep;
class DataBaseRep;
class DbIndexRepPart;
class DbIndexRepImplem;
class DataBaseRepImplem;
class DbSetRepImplem;
class DbCursorRep;
class DbCursorRepImplem;
class DbField;
class DbExpr;
class InCheck;
class EssDataBase;
class EssObjectPrefix;
class TimeStamp;
class DbObjectRepBuffer;
class DbObjectRepStack;
class DbEssIndex;
class DbObjectRepPrefix;
class MsgTable;
class DbObjectRepCache;
class CDataBaseRep;
class CDbSetRep;

struct UpdateOp;
struct DbPrivgRep;

class gtype(PtrArray, DbIndexRepPart);
class gtype(PtrArray, DbIndexRep);
class gtype(PtrArray, DbSetRep);
class gtype(PtrArray, DbMemberRep);
class gtype(PtrArray, DbCursorRep);
class gtype(PtrArray, DbPrivgRep);
class gtype(PtrArray, DataBaseRep);
class gtype(PtrArray, DbTypeRep);
class gtype(PtrArray, DbObjectRepBuffer);
class gtype(PtrArrayCursor, DbObjectRepBuffer);
class gtype(PtrArray, UpdateOp);

// typedef used to define dbm handlers
typedef void (*DBHANDLER)(DbSetRep *, const String &, const String &);

/**
  * <EM>DbObjectRep</EM> class.
  * @version $Revision: 1.10 $
  * @author $Author: pablog $
  */
class DbObjectRep {

	useIdCheck(DbObjectRep_id);
	
// private:

	DbSetRep *extent_x;
	DbIndexRep *index_x;
	DbObjectRepBuffer *buffer_d;
	DbObjectRepStack *stack_d;
	DbObjectRepCache *cache_d;
	
	inline DbObjectRepBuffer *objectBuffer();
	inline DbIndexRep *index();
	inline DbObjectRepCache *cache();
	
	DbObjectRepStack *stack();

	void create(void *buff, bool attachSet = true);

	inline void index(DbIndexRep *ind);

protected:

	DbObjectRep(DbSetRep *set, void *buff = NULL);

public:

	// creating a DbObjectRep without associated extent
	DbObjectRep(void *buff = NULL);
	// creating a DbObjectRep from a DataBaseRep name and extent name
	DbObjectRep(const String &dbName, const String &extnt, void *buff = NULL);
	// creating a DbObjectRep from an extent name and the current DataBaseRep
	DbObjectRep(const String &extnt, void *buff = NULL);
	// creating a DbmOject from a DataBaseRep and extent name
	DbObjectRep(DataBaseRep *db, const String &extnt, void *buff = NULL);

	virtual ~DbObjectRep();
		
	bool accessBy(const String &name);
	bool accessBy(Int nIndex);
	
	void init();
	void del();
	void add();
	void add(const TimeStamp &crts, const TimeStamp &modts);
	void put();
	Int  get(Int mode = this_key|default_lock, Int nparts = 0);
	void free(Int mode = this_key, Int nparts = 0);
	Int  find(Int mode = this_key, Int nparts = 0);
	                                                  
	void update(gtype(PtrArray,UpdateOp) &ops);

	void copy(DbObjectRep *objRep);
	
	void push();
	Int  pop();
	Int  stackSize();
	void discard();
	void discardAll();

	TimeStamp mTimeStamp();
	TimeStamp cTimeStamp();

	Int setCache(Int n, Int mode);

	const String &name();
	DbSetRep &extent();

	// these routines should be private
	char *buffer();
	Int length();
	DbSetRep *set();	
};

/**
  * <EM>DbSetRep</EM> class
  * @version $Revision: 1.10 $
  * @author $Author: pablog $
  */
class DbSetRep {

	useIdCheck(DbSetRep_id);
	
// private:

	String	name_d;
	String	descr_d;
	Int		flags_d;
	Int		size_d;
	Int		mode_d;
	Int		actPrivg_d;

	gtype(PtrArray, DbPrivgRep) &privgs_d;
	gtype(PtrArray, DbIndexRep) &indices_d;
	gtype(PtrArray, DbCursorRep) &cursors_d;
	DbTypeRep *typeRep_x;
	DbSetRepImplem *implem_d;
	DataBaseRep *dataBase_x;

	void calculateActPrivg(Int nopen = 0);

	inline void descr(const String &d);
	inline void flags(Int f);
	inline void size(Int s);
	inline void mode(Int m);
	inline void typeRep(DbTypeRep *t);
	inline void implem(DbSetRepImplem *i);
	inline void name(const String &nm);
	inline void actPrivg(Int p);
	
	void addIndex(DbIndexRep *index);
	void delAllIndices();
	
	void addPrivg(DbPrivgRep *privg);
	void delAllPrivgs();
	
	Int addCursor(DbCursorRep *cur);
	void delCursor(DbCursorRep *cur);
	void delAllCursors();
		
	Int dropIndex(DbIndexRep *ind);
	
	bool create();
	
	void setPCreatStamp();			

	Int nIndices();
	
public:

	DbSetRep(DataBaseRep *dataBase, const String &name, const String &descr, 
		     DbTypeRep *typeRep, Int flags, Int size);
	DbSetRep(DataBaseRep *dataBase, const String &name = NULL_STRING);
	virtual ~DbSetRep();
	
	inline const String &name();
	inline Int flags();
	inline const String &descr();
	inline Int size();
	inline DbTypeRep *typeRep();
	inline DataBaseRep *dataBase();		
	inline DbSetRepImplem *implem();		

	DbCursorRep *cursor(Int n);

	bool wasAttached();
	
	bool hasVirtuals();
	bool hasPCreatStamp();
	bool hasCreatStamp();
		
	inline bool isTemp();

	Int nMembers();
	Int memberNo(const String &n);
	Int memberNo(DbMemberRep *m);
	DbMemberRep *member(const String &n);
	DbMemberRep *member(Int n);

	DbIndexRep *index(const String &name);
	DbIndexRep *index(Int nIndex);
	DbIndexRep *primaryKey();
	Int indexNo(const String &name);
				
	Int typeLength();
	
	Int get(DbObjectRep *obj, Int mode, Int nparts);
	void free(DbIndexRep *index, DbObjectRepBuffer *buffer, Int mode, Int nparts);
	Int find(DbIndexRep *index, DbObjectRepBuffer *buffer, Int mode, Int nparts);
	void init(DbObjectRepBuffer *buffer);
	void del(DbObjectRepBuffer *buffer);
	void add(DbObjectRepBuffer *buffer);
	void add(DbObjectRepBuffer *buffer, const TimeStamp &crts, 
			 const TimeStamp &modts);
	void put(DbObjectRepBuffer *buffer);
	void push(DbObjectRepBuffer *buffer, DbObjectRepStack *stack);
	Int pop(DbObjectRep *obj, DbObjectRepStack *stack);
	void discard(DbObjectRepStack *stack);
	String checkObject(DbObjectRepBuffer *buffer, String &par, Int nInd = ERR);
	void update(gtype(PtrArray,UpdateOp) &ops);
				
	Int rename(const String &newName);
	Int delAll();
	Int lock(Int lockMode);
	void free();
	void flush();
	Int actualSize();
	Int nObjects();

	Int dropIndex(const String &indname);
	Int dropIndex(Int indnum);

	DbMemberRep *addMember(const String &name, const String &descr, 
							  Int dim, Int ty, Int sqlTy, Int len, Int ndec, 
							  Int flags, const String &def, 
							  const String &mask);

	bool renameMember(DbMemberRep *m, const String &newName);

	TimeStamp mTimeStamp(DbObjectRepBuffer *buffer);
	TimeStamp cTimeStamp(DbObjectRepBuffer *buffer);

	String getIndices();
	String getMembers();

	DbIndexRep *createIndex(const String &name, Int flags, Int separ);

	Int setDeflt(DbMemberRep *m, DbExpr *deflt);
	Int setRelCheck(DbMemberRep *m, DbExpr *rchk);
	Int setCollSet(DbMemberRep *m, const String &cs);
	Int setInCheck(DbMemberRep *m, InCheck *ic);

	Int *getEquiv(DbSetRep *to);
	void copy(DbSetRep *to, Int *members, Int flag);

	void copyPrivgs(DbSetRep *from);

	virtual CDbSetRep *getCDbSetRep();
};

/**
  * DataBaseRep class
  * @version $Revision: 1.10 $
  * @author $Author: pablog $
  */
class DataBaseRep {

	useIdCheck(DataBaseRep_id);

// private:

	String 						name_d;
	String 						descr_d;
	Int 						mode_d;
	Int 						owner_d;
	Int 						actPrivg_d;	
	Int							id_d;
	DataBaseRepImplem 			*implem_d;
	gtype(PtrArray, DbPrivgRep)	&privgs_d;
	gtype(PtrArray, DbSetRep)	&sets_d;
	gtype(PtrArray, DbTypeRep)	&typesDescr_d;
			
	static DBHANDLER dbHandler_sd;
	static MsgTable *dmsg_sd;

	static gtype(PtrArray, DataBaseRep) *dataBases_sd;
	static gtype(PtrArray, DataBaseRep) &dataBases();
	static Int addDataBaseRep(DataBaseRep *db);
	static Int delDataBaseRep(DataBaseRep *db);
	static bool changeId(DataBaseRep *db, Int newId);

	static Int ioErrNo_sd;
	static Int ioErrDetail_sd;

	void calculateActPrivg(Int nopen = 0);

	void addPrivg(DbPrivgRep *p);

	void addTypeRep(DbTypeRep *typeRep);
	Int delTypeRep(DbTypeRep *typeRep);
	
	inline void name(const String &nm);
	inline void descr(const String &d);
	inline void mode(Int m);
	inline void owner(Int o);
	
	static const char *detailMsg(Int nerr);

	Int dropSet(DbSetRep *set);

protected:

	virtual void addSet(DbSetRep *set);
	virtual void freeSetSlot(DbSetRep *s);

	virtual DbSetRep *newDbSetRep(const String &name, const String &descr, 
								  DbTypeRep *typeRep, Int flags, Int size);
	virtual DbSetRep *newDbSetRep(const String &name = NULL_STRING);

public:

	DataBaseRep(const String &name, const String &descr);
	DataBaseRep(const String &name, Int mode, Int version = CURRENT_VERSION);
	virtual ~DataBaseRep();
	
	static String msg(const char *m);
	static String msg(const String &m);
	static String msg(Int m);

	static DBHANDLER setDbHandler(DBHANDLER f);
	static void dbHandler(DbSetRep *set, const String &err, const String &par);
		
	static void ioErrNo(Int err);
	static void ioErrDetail(Int errd);
	static Int ioErrNo();
	static Int ioErrDetail();
	
	static void beginTransaction(Int st = io_eabort);
    static Int rollBack();
	static void optimisticTransaction();
	static bool endTransaction();
	static bool endTransaction(bool syn, bool rttrans);
	static bool endSyncTransaction();
	static bool transOk();
	static void corruptTrans();
	static Int transMode();
	static bool inTransaction();
	static void saneTrans();
		
	static String getDataBases();
	
	static bool checkPoint();
	static Int versionNumber(const String &schName, const Date &d);	
	static DataBaseRep *find(const String &schName, Int vers=CURRENT_VERSION);
	
	void setDescr(const String &descr);
	Int drop();
	Int lock(Int l);
	void free();
	Int version();
	void modify();

	Int dropSet(const String &name);
	Int dropSet(Int nSet);

	String getSets();
	
	static void reOpen(DataBaseRep *db);
	
	static Int convert(DbSetRep *oldSet, DbSetRep *newSet, bool force, 
					   bool drop, IFCPVA err_func, FCPVA msg_func, 
					   unsigned int cache, unsigned int dcache, 
					   bool correctRefs);		

	Int setSeqNumber(DbSetRep *set);

	void attach(DbSetRep *set);
	DbTypeRep *createTypeRep(const String &name, const String &descr, 
							 Int nMembers);
	Int deleteTypeRep(DbTypeRep *typeRep);
	
	DbSetRep *createSet(const String &name, const String &descr, Int flags, 
						Int size, Int nMembers);
	DbSetRep *reCreateSet(const String &name, const String &descr, Int flags,
						  Int size, Int nMembers);
												
	bool chown(Int uid);

	inline const String &name();
	inline const String &descr();
	inline Int mode();
	inline Int owner();
	inline Int id();
	
	inline DataBaseRepImplem *implem();

	DbSetRep *set(const String &nm);
	Int nSets();
	DbSetRep *set(Int n);
	Int setNo(const String &nm);
	Int setNo(DbSetRep *s);
		
	virtual CDataBaseRep *getCDataBaseRep();

	static Int dbmsType();

	static DataBaseRep *dataBase(const String &name);
	static DataBaseRep *current();
	static DataBaseRep *current(DataBaseRep *db);
	static Int nDataBases();
	static DataBaseRep *dataBase(Int i);
	static DataBaseRep *findNext(DataBaseRep *db);
};

/**
  * DbObjectRep class
  * @version $Revision: 1.10 $
  * @author $Author: pablog $
  */
class DbObjectRepBuffer {

// private:

	DbObjectRepPrefix *objectPrefix_;
	Int objectNo_;
	char *buffer_;
	DbSetRep *set_;
	bool wasAlloced_;
		
public:

	DbObjectRepBuffer(DbSetRep *set, void *buffer = NULL);
	~DbObjectRepBuffer();
		
	DbObjectRepBuffer &operator=(DbObjectRepBuffer &ob);
  
	char *buffer();
	Int length();
	Int fullLength();

	inline DbObjectRepPrefix *objectPrefix();
	inline Int objectNo();
	inline void objectNo(Int oNo);
};

/**
  * <EM>DbObjectRepStack</EM> class
  * @version $Revision: 1.10 $
  * @author $Author: pablog $
  */
class DbObjectRepStack {

// private:

	char	*stack_;		// Object stack
	Int	aObjs_;		// Room for this many objects w/out realloc
	Int	nObjs_;		// Current number of objs

public:

	DbObjectRepStack();
	~DbObjectRepStack();

	void push(char *buff, Int len);
	char *pop(Int len);			

	inline Int size();

	void discardAll();	
};

/**
  * <EM>DbMemberRep</EM> class
  * @version $Revision: 1.10 $
  * @author $Author: pablog $
  */
class DbMemberRep {

	useIdCheck(DbMemberRep_id);
	
// private:

	friend class DbSetRepImplem;
	friend class DbEssSet;
		
	String		name_;
	String		descr_;
	Int			dim_;
	Int			offset_;
	Int			mType_;
	Int			mIType_;
	Int			sqlType_;
	Int			flags_;
	Int			len_;
	Int			nDec_;
	String		mask_;
	DbExpr		*relCheck_;
	DbExpr		*deflt_;
	InCheck		*inCheck_;
	String		cMask_;
	String		collset_;
	DbField		*fld_;			// used only by Essentia DBM

	inline void name(const String &n);
	inline void mType(Int t);
	inline void deflt(DbExpr *d);
	inline void mask(const String &m);
	inline void dim(Int d);
	inline void len(Int l);
	inline void mIType(Int it);
	inline void cMask(const String &cm);
	inline void nDec(Int nd);
	inline void inCheck(InCheck *ic);
	inline void relCheck(DbExpr *rc);
	inline void collset(const String &cs);
	inline void descr(const String &d);
	inline void flags(Int f);
	inline void offset(Int o);
	inline void sqlType(Int sqlt);
		
	Int calcIType();
	Int calcStampFlags();

public:

	DbMemberRep(const String &name, const String &descr, Int flags, 
				   Int type, Int sqlTy, const String &mask, Int dim, 
				   Int len, Int ndec, Int offset, DbExpr *deflt = NULL, 
				   DbExpr *relCheck = NULL, InCheck *inCheck = NULL,
				   const String &collset = NULL_STRING);
	~DbMemberRep();
	
	bool isNull();

	inline Int mType();
	inline Int len();
	inline Int flags();
	inline Int dim();
	inline DbExpr *deflt();
	inline bool isString();
	inline bool isNumeric();
	inline bool isDate();
	inline bool isTime();
	inline const String &collset();
	inline const String &mask();
	inline const String &cMask();
	inline DbField *fld();
	inline Int offset();
	inline bool isVirtual();
	inline bool isCreatStamp();
	inline bool isModifStamp();
	inline Int mIType();
	inline const String &name();
	inline Int nDec();
	inline const String &descr();
	inline InCheck *inCheck();
	inline Int sqlType();
		
	void setPCreatStamp();

	Int storeLength(Int l = ERR);
	void store(char *buff, char *&key);		
	void load(char *buff, const char *&key);		
	
	void set(char *buffer, const AnyValue &av, Int row = 0, Int rep = 1,
			 Int offset = 0, Int len = ERR);
	Int cmp(char *buffer, Int dima, Int dimb);
	AnyValue get(char *buffer, Int row = 0, Int offset = 0, Int len = ERR);
	void init(DbSetRep *set, char *buffer);
	AnyValue value(const String &val);
};

/**
  * <EM>DbIndexRep</EM> class.
  * @version $Revision: 1.10 $
  * @author $Author: pablog $
  */
class DbIndexRep {

	useIdCheck(DbIndexRep_id);
	
// private:

	friend class DbEssSet;
	friend class EssObjectCache;
		
	String name_;
	Int flags_;
	Int storeLength_;
	DbIndexRepImplem *implem_;
	gtype(PtrArray, DbIndexRepPart) &parts_;
	DbSetRep *set_;
	
	void implem(DbIndexRepImplem *i);
	
	char *buildKey(DbObjectRepBuffer *buffer);
	void unBuildKey(DbObjectRepBuffer *buffer, const char *key);
			
public:

	DbIndexRep(DbSetRep *dbSet, const String &name, Int flags);
	DbIndexRep(const String &n, Int f);
	~DbIndexRep();
										
	Int build(Int flags, ...);
	Int complete();
	Int addKey(DbObjectRep &obj);
	
	Int addMember(DbMemberRep *member, Int flags, Int off, Int len);
	
	void addPart(DbIndexRepPart *p);
	
	inline const String &name();
	inline Int flags();
	inline Int storeLength();
	inline bool isTemp();
	inline bool isUnique();
	bool isPrimaryKey();

	inline DbIndexRepImplem *implem();
	inline DbSetRep *set();

	Int nParts();
	DbIndexRepPart *part(Int n);
	bool isPart(DbMemberRep *m);
	
	Int cmp(char *buffer1, char *buffer2);
	void setNullValue(DbObjectRepBuffer *buffer);
	void setHighValue(DbObjectRepBuffer *buffer);
	bool checkKey(DbObjectRepBuffer *buffer);
	bool findKey(DbSetRep *set, DbObjectRepBuffer *buffer, Int fmode = this_key, 
				Int nparts = 0, char **key_ptr = NULL, Int lmode = io_lock);
	Int cmpKey(DbObjectRepBuffer *buff1, DbObjectRepBuffer *buff2);

	String getMembers();

	void setCache(Int n);
};

/**
  * <EM>DbTypeRep</EM> class
  * @version $Revision: 1.10 $
  * @author $Author: pablog $
  */
class DbTypeRep {

	useIdCheck(DbTypeRep_id);
	
// private:

	String name_;
	String descr_;
	Int   len_;
	Int	  flags_;
	Int	  maxMembers_;
	gtype(PtrArray, DbMemberRep) &members_;
	DataBaseRep *dataBase_;

	Int sizeOf(Int ty);
	void align(Int ty);

	inline DataBaseRep *dataBase();

public:
	
	DbTypeRep(DataBaseRep *db, const String &name, const String &descr, 
				 Int nMembers);
	~DbTypeRep();

	DbMemberRep *addMember(const String &name, const String &descr, 
							  Int dim, Int ty, Int sqlTy, Int len, Int ndec, 
							  Int flags, const String &deflt, 
							  const String &mask);
	DbMemberRep *addMember(const String &nm, const String &descr, 
							  Int flags, Int ty, Int sqlTy, 
							  const String &mask, Int dim, Int len, Int ndec,
							  DbExpr *deflt = NULL, DbExpr *relCheck = NULL, 
							  InCheck *incheck = NULL, 
							  const String &collset = NULL_STRING);

	Int memberNo(const String &n);
	Int memberNo(DbMemberRep *m);
	DbMemberRep *member(Int mno);
	DbMemberRep *member(const String &n);
	
	inline Int len();
	inline Int flags();

	Int nMembers();
	bool isCompleted();

	String getMembers();

	void setPCreatStamp();			
};

// This structure holds access privileges for users
struct DbPrivgRep {
	Int id_;		// A group or user id.
	Int idType_;	// Is group or user id.
	Int privg_;	// Bits with privileges

	DbPrivgRep(Int id, Int idType, Int privg)
	:	id_(id), idType_(idType), privg_(privg) 
	{ }
};

/**
  * <EM>DbIndexRepPart</EM> class.
  * @version $Revision: 1.10 $
  * @author $Author: pablog $
  */
class DbIndexRepPart {

	useIdCheck(DbIndexRepPart_id);
	
// private:

	friend class DbSetRepImplem;
	friend class DbEssSet;
	
	Int				mIType_;
	Int				flags_;
	Int				len_;
	Int				offset_;
	DbMemberRep	*member_;

	inline void member(DbMemberRep *m);
	inline void flags(Int f);
	inline void mIType(Int it);
	inline void offset(Int o);
	
public:

	DbIndexRepPart(DbMemberRep *member, Int mIType, Int flags, Int len, 
				 Int offset);
	~DbIndexRepPart();

	inline DbMemberRep *member();
	inline Int offset();
	inline Int len();
	inline Int flags();
	Int storeLength();
};

/**
  * <EM>DbCursorRep</EM> class
  * @version $Revision: 1.10 $
  * @author $Author: pablog $
  */
class DbCursorRep {

	useIdCheck(DbCursorRep_id);

	friend class DbCursor;
	friend class DataBaseRepImplem;
	friend class EssDataBase;
	friend class DbCursorRepImplem;
	friend class DbEssCursor;
	friend class DbIxCursor;
			
// private:

	DbObjectRep *obj_;			// DbObjectRep associated with this cursor
	DbIndexRep *ind_;			// index used
	Int lMode_;				// lock mode
	Int flags_;					// range flags
	Int wkFlags_;				// work flags
	Int fNRec_;				// there are fnrecs forwarded records in cache
	Int bNRec_;				// there are bnrecs backwarded records in cache
	bool moreFRecs_;			// are there more forwarded records ?
	bool moreBRecs_;			// are there more backwarded records ?
	DbObjectRepBuffer *rangeFrom_;// from range
	DbObjectRepBuffer *rangeTo_;	// to range
	DbObjectRepBuffer *currRange_;// current range
	Int rangeMember_;			// current member inside the current range
	gtype(PtrArray, DbMemberRep) &members_;
							// fields array which this cursor was created
	gtype(PtrArray, DbObjectRepBuffer) &objBuffers_;
	gtype(PtrArrayCursor, DbObjectRepBuffer) &objBuffCur_;
	DbCursorRepImplem *implem_;
	Int id_;
	
	inline DbObjectRep *object();
	inline void flags(Int f);
	inline void wkFlags(Int f);
	inline void fNRec(Int n);
	inline void bNRec(Int n);
	inline void moreFRecs(Int n);
	inline void moreBRecs(Int n);
		
	inline Int flags();
	inline Int wkFlags();
	inline Int lMode();
	inline Int fNRec();
	inline Int bNRec();

	inline DbObjectRepBuffer *rangeFrom();
	inline DbObjectRepBuffer *rangeTo();
	inline DbCursorRepImplem *implem();
	
	gtype(PtrArray, DbMemberRep) &members();

	void reset();
	void resetValues(bool forw);
	void clearRanges();
	bool subFRecord();
	bool subBRecord();
	void addFRecord();
	void addBRecord();
	void moveFOffset();
	void moveBOffset();
	Int setRecord(bool forw);
	Int setRecord();
	void prepareRange();

	Int nMembers();
	DbMemberRep *member(Int i);
	void reAllocBuffers(Int n);
	DbObjectRepBuffer *objectBuffer(Int i);
	void goCurrObjTo(Int n);
	void goCurrObjToFirst();
	void goCurrObjToLast();
			
	void setRangeValue(DbObjectRepBuffer *buffer, int nfld, const AnyValue &av);
	Int addMember(const String &name);
	void setCond(Int range);
	void addRangeValue(const AnyValue &av);

	bool haveToSendRange();
		
public:

	DbCursorRep(DbObjectRep &obj, const String &indName = NULL_STRING, 
			  	Int mode = io_not_lock);
	~DbCursorRep();
		
	Int fetch();
	Int fetchPrev();
	Int fetchThis();
	void moveFirst();
	void moveLast();
	Int count();

	void setFromFld(int nfield, const AnyValue &av);
	void setToFld(int nfield, const AnyValue &av);
	
	inline Int id();
	inline DbIndexRep *index();
};

#include <ideafix/priv/dbrep.icc>

#endif
