// -----------------------------------------------------------------------
// Copyright (c) 1995 InterSoft Co.  All Rights Reserved
// THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF InterSoft Co.
// The copyright notice above does not evidence any actual or intended 
// publication of such source code.
//
// $Id: fmactn.h,v 1.6 1996/07/15 19:53:27 eugen Exp $
//
// Description:
// -----------------------------------------------------------------------

#ifndef FMACTN_H
#define FMACTN_H

#ifdef __GNUG__
#	pragma interface
#endif

#include <ifound.h>
#include <ideafix/fmtype.h>
#include <ideafix/priv/optnobj.h>

class FormRep;
class FmFieldObj;

class FmAction : public ObjectWithOptions {
public:
	virtual ~FmAction();
	
	inline FormRep *formRep() const;
	
	// return action's id
	inline Fm::Event id() const;
	// alias of 'id()'
	inline operator int() const;

	// action's name
	inline const String &name() const;

	// action's title
	inline const String &title() const;

	// action's description
	inline const String &descr() const;

	// bitmap's name
	inline const String &bitmapName() const;
	inline const String &pushBitmapName() const;

	// try to execute this action
	virtual bool preProcess();

	// process the action's result
	virtual bool processResult() const;

	// call user's code associated with this action
	virtual void callUser();

	// get & set action's result
	inline Fm::Action result() const;
	inline void result(Fm::Action res);

	// whether the action should be ignored
	bool ignored() const;

	// whether the action is confirmed
	bool confirmed() const;

	bool addOption_(const ObjectOption &option);
	bool delOption_(const ObjectOption &option);
	static const OptionTable optTable[];
	const OptionTable *optionTable() const;
	OptionValue *createOptionValue(OptionType type);

	struct Opt {
		enum {
			name,
			action_id,
			title,
			descr,
			bitmap,
			pushed_bitmap,
			ignore_in_key,
			ignore_in_data,
			confirm,
			reset_after,
			validate_before,
		};
	};

protected:

	// default copy constructor should be OK
	FmAction(FormRep *theForm, Fm::Event id, const String &name,
			const String &bitmap);
			
	void desired(FmFieldObj *obj) const;
	void desiredKey() const;
	void desiredData() const;
	
	void setId(Fm::Event id);
	void setName(const String &name);
	void confirm(bool v);
	void ignoreInKeyZone(bool v);
	void ignoreInDataZone(bool v);

private:
	// The form this action belongs to
	FormRep *fm_;

	// This action's id
	Fm::Event id_;

	// This action's internal name
	String name_;
	// Action's title (for menu etc.)
	String title_;
	// This action's short description
	String descr_;

	// Whether this action should be ignored in these zones
	bool ignoreInKey_;
	bool ignoreInData_;
	
	// Another thing that could be here is the action's icon
	String bitmapName_, pushBitmapName_;

	// Whether this action should be confirmed
	bool confirm_;

	// The result of the user's code for this action
	Fm::Action result_;
};


class FmActionNull : public FmAction {
public:
	FmActionNull(FormRep *f);

	bool preProcess();
};


class FmActionRead : public FmAction {
public:
	FmActionRead(FormRep *form);

	bool preProcess();
	bool processResult() const;
	void callUser();

protected:
	FmActionRead(FormRep *form, Fm::Event id, const String &name,
			const String &bitmap);
	
	virtual bool leaveKeyZone();
};

class FmActionReadNextPrev : public FmActionRead {
protected:
	FmActionReadNextPrev(FormRep *form, Fm::Event id, const String &name,
			const String &bitmap);
	bool leaveKeyZone();
};

class FmActionReadFirstLast : public FmActionRead {
protected:
	FmActionReadFirstLast(FormRep *form, Fm::Event id, const String &name,
			const String &bitmap);
	bool leaveKeyZone();
};

class FmActionReadNext : public FmActionReadNextPrev {
public:
	FmActionReadNext(FormRep *form);
};

class FmActionReadPrev : public FmActionReadNextPrev {
public:
	FmActionReadPrev(FormRep *form);
};

class FmActionReadFirst : public FmActionReadFirstLast {
public:
	FmActionReadFirst(FormRep *form);
};

class FmActionReadLast : public FmActionReadFirstLast {
public:
	FmActionReadLast(FormRep *form);
};


class FmActionReset : public FmAction {
public:
	FmActionReset(FormRep *f);

	bool preProcess();
	bool processResult() const;
};


class FmActionUpdate : public FmAction {
public:
	FmActionUpdate(FormRep *f);

	bool preProcess();
	bool processResult() const;
};


class FmActionAdd : public FmAction {
public:
	FmActionAdd(FormRep *f);

	bool preProcess();
	bool processResult() const;
};


class FmActionDelete : public FmAction {
public:
	FmActionDelete(FormRep *f);

	bool preProcess();
	bool processResult() const;
};


class FmActionClose : public FmAction {
	typedef FmAction inherited;
public:
	FmActionClose(FormRep *f);

	bool preProcess();
	bool processResult() const;

	bool delOption_(const ObjectOption &option);
};


class FmActionUndo : public FmAction {
public:
	FmActionUndo(FormRep *f);
	
	bool preProcess();
	bool processResult() const;
};


class FmActionSubform : public FmAction {
public:
	FmActionSubform(FormRep *f);
	
	bool preProcess();
	bool processResult() const;
};

class FmActionClearField : public FmAction {
public:
	FmActionClearField(FormRep *f);
	
	bool preProcess();
	bool processResult() const;
};


class FmActionUserDef : public FmAction {
	typedef FmAction inherited;
public:
	FmActionUserDef(FormRep *form,
		Fm::Event id = Fm::no_action, const String &name = NULL_STRING);
			
	bool preProcess();
	bool processResult() const;
	inline bool validateBeforeExec() const;
	inline bool resetAfterExec() const;

	bool addOption_(const ObjectOption &option);
	bool delOption_(const ObjectOption &option);

private:
	// Whether a reset should be implied after executing this action
	bool reset_;

	// Whether all objects should be validated before executing this action
	bool validate_;
};

#include <ideafix/priv/fmactn.icc>

#endif


