// -----------------------------------------------------------------------
// Copyright (c) 1995 InterSoft Co.  All Rights Reserved
// THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF InterSoft Co.
// The copyright notice above does not evidence any actual or intended 
// publication of such source code.
//
// $Id: fmbuff.h,v 1.5 1996/07/15 19:53:29 eugen Exp $
//
// Description:
// -----------------------------------------------------------------------

#ifndef FMBUFF_H
#define FMBUFF_H

#ifdef __GNUC__
#pragma interface
#endif

#include <ifound.h>

class FmFieldObj;
class FmObjContainer;

class gtype(PtrArray, char);
declare(PtrArray, char);

#define assert2(EX,P1) \
	do { if (EX) break; ::DUMP(P1); \
	_assert(quote(EX), __FILE__, __LINE__); } while(0)
#define assert3(EX,P1,P2) \
	do { if (EX) break; ::DUMP(P1); ::DUMP(P2); \
	_assert(quote(EX), __FILE__, __LINE__); } while(0)

class FmBuffer
{
	friend class FmObjContainer;

public:
	explicit FmBuffer(FmObjContainer *obj);
	~FmBuffer();

	static void create(FmBuffer *buff, FmObjContainer *obj, FmBuffer *par);
	static void destroy(FmBuffer *buff);

	inline FmObjContainer *owner() const;
	inline FmBuffer *parent() const;
	Int numRows() const;

private:
	FmObjContainer	*owner_x;
	FmBuffer *parent_x;
	Int modifiedRows_d;
	gtype(PtrArray, char) buffer_d;

	FmBuffer(FmObjContainer *obj, FmBuffer *par);
	
	void _assert(const char *, const char *, int) const;
	void _assert(const char *, const char *, int, const String &) const;
	
	// ---------- others methods are for hiding ------------
	
	// to disallow copying
	void operator =(FmBuffer &);
	FmBuffer(FmBuffer&);

	// to disallow heap allocation of FmBuffer
	void *operator new(size_t s);
	void *operator new[](size_t s);
	void operator delete(void *);
	void operator delete[](void *);

	void *operator new(size_t s, FmBuffer *p);
};


///////////////////////////////////////////////////////////////

class FmBufferRow
{
public:
	inline FmBufferRow();
	inline FmBufferRow(const FmBufferRow &br);
	inline FmBufferRow(const FmBufferRow &br, Int row);
	inline FmBufferRow(FmBuffer *buff, Int row);
	FmBufferRow(FmFieldObj *obj, Int row);
	
	inline FmObjContainer *owner() const;
	inline FmBuffer *buffer() const;
	inline Int row() const;
	inline Int numRows() const;
	
	inline FmBufferRow &operator=(const FmBufferRow &br);
	inline bool operator==(const FmBufferRow &br) const;
	inline bool operator!=(const FmBufferRow &br) const;

private:
	FmBuffer *buffer_x;
	Int      row_d;
	
	void _assert(const char *, const char *, int) const;
	void _assert(const char *, const char *, int, const String &) const;
};

void DUMP(const FmBuffer *buff);
void DUMP(const FmBufferRow &br);

////////////////////////////////////////////////////////////////////////

#include <ideafix/priv/fmbuff.icc>

#endif
