// -----------------------------------------------------------------------
// Copyright (c) 1995 InterSoft Co.  All Rights Reserved
// THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF InterSoft Co.
// The copyright notice above does not evidence any actual or intended 
// publication of such source code.
//
// $Id: fmcback.h,v 1.3 1996/08/15 16:37:30 eduardoc Exp $
//
// Description:
// -----------------------------------------------------------------------

#ifndef FMCBACK_H
#define FMCBACK_H

#ifdef __GNUG__
#	pragma interface
#endif

#include <ifound.h>

class FormRep;
class FmFieldObj;
class RequestConnection;
class wxKeyEvent;
  
class FmKeyEvent;
class FmSpecialEvent;
class FmWxKeyEvent;

////////////////////////////////////////////////////////////////////////////
  
class FmCallBackEvent {
public:

	virtual ~FmCallBackEvent();

	enum Type {
		evSpecial,
		evKey,
		evWxKey
	};

	Type type() const;

	bool operator==(const FmCallBackEvent &fk) const;

	void send(RequestConnection &conn) const;
	static FmCallBackEvent *receive(RequestConnection &conn);

protected:
	FmCallBackEvent(Type);
	
private:
	Type type_d;
};

////////////////////////////////////////////////////////////////////////////

class FmSpecialEvent : public FmCallBackEvent {
public:
	typedef FormRep::CallBack SubType;
	FmSpecialEvent(SubType subType);
	FmSpecialEvent(RequestConnection &conn);

	SubType subType() const;

	bool operator==(const FmSpecialEvent &fk) const;

	void send(RequestConnection &conn) const;

private:
	SubType subType_d;
};           

////////////////////////////////////////////////////////////////////////////

class FmKeyEvent : public FmCallBackEvent {
public:	
	FmKeyEvent(Int key);
	FmKeyEvent(RequestConnection &conn);

	Int key() const;

	bool operator==(const FmKeyEvent &fk) const;

	void send(RequestConnection &conn) const;

private:
	Int key_d;
};

///////////////////////////////////////////////////////////////////////////                

class FmWxKeyEvent : public FmCallBackEvent {
public:	                  
	FmWxKeyEvent(const wxKeyEvent &key);
	FmWxKeyEvent(RequestConnection &conn);

	Int key() const;
	bool shift() const;
	bool control() const;
	bool alt() const;
	bool meta() const;

	bool operator==(const FmWxKeyEvent &fk) const;

	void send(RequestConnection &conn) const;

private:
	Int key_d;
	bool shift_d, control_d, alt_d, meta_d;
};

//////////////////////////////////////////////////////////////////////////                                             
////////////////       struct FmCallBack        ////////////////////////////
//////////////////////////////////////////////////////////////////////////
   
struct FmCallBack {
	FormRep	 	*form;
	FmCallBackEvent	*event;

	FmFieldObj	*from;
	FmFieldObj	*to;

	FmCallBack	*next;

    FmCallBack(FormRep *fm, RequestConnection &conn);
    FmCallBack(FormRep *fm, FmCallBackEvent *event,
		    		FmFieldObj *from, FmFieldObj *to);
    virtual ~FmCallBack();

   	bool filt(FmFieldObj *obj);
   	bool filt(const FmCallBackEvent &ev, FmFieldObj *obj);
   	virtual bool execute(FmFieldObj *obj, Int row);

   	void send(RequestConnection &conn) const;
}; 

////////////////////////////////////////////////////////////////////////////

class CPPFmCallBack : public FmCallBack {
public:	
	typedef bool (FormRep::*FpCallBack)(FmFieldObj *obj, Int r);
	FpCallBack fpCallBack;

	CPPFmCallBack(FormRep *form, FpCallBack callBack,
		FmCallBackEvent *event, FmFieldObj *from, FmFieldObj *to);
   	
	bool execute(FmFieldObj *obj, Int row);
};                   

////////////////////////////////////////////////////////////////////////////

class CFmCallBack : public FmCallBack {
public:	
	fm_status_fp fpCallBack;
	
	CFmCallBack(FormRep *form, fm_status_fp callBack,
		FmCallBackEvent *event, FmFieldObj *from, FmFieldObj *to);
			
	bool execute(FmFieldObj *obj, Int row);
};                                                                      

////////////////////////////////////////////////////////////////////////
//////////////////     class CallBackIterator    ///////////////////////
////////////////////////////////////////////////////////////////////////

class CallBackIterator {
	FormRep *frep;

	FmFieldObj *from, *to;

	FmCallBack **cb_ptr;
	FmCallBack *cb;
	
public:
	         
    CallBackIterator(FormRep *rep,
    	FmFieldObj *from = NULL, FmFieldObj *to = NULL);

    bool iterate();
    virtual bool Step( FmCallBack *cb ) = NULL;

	bool checkFields();
	void remove();
	void replace(FmCallBack *cb);
	void insert(FmCallBack *cb);
};

RequestConnection &operator<<(RequestConnection &conn, const FmCallBackEvent &ev);
RequestConnection &operator>>(RequestConnection &conn, FmCallBackEvent *&ev);
RequestConnection &operator<<(RequestConnection &conn, const FmCallBack &cb);

#include <ideafix/priv/fmcback.icc>

#endif

