// -----------------------------------------------------------------------
// Copyright (c) 1995 InterSoft Co.  All Rights Reserved
// THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF InterSoft Co.
// The copyright notice above does not evidence any actual or intended 
// publication of such source code.
//
// $Id: fmedit.h,v 1.5 1996/08/21 08:31:46 kostya Exp $
//
// Description:
// -----------------------------------------------------------------------

#ifndef FMEDIT_H
#define FMEDIT_H

#ifdef __GNUG__
#	pragma interface
#endif

#include <ifound.h>

class FmValueObj;
class FmCharField;
class FmNumField;
class FmFloatField;
class FmBoolField;
class FmDateField;
class FmTimeField;

class wxObject;
class wxEvent;
class wxKeyEvent;
class wxItem;
class wxPanel;

class gtype(PtrArray, wxItem);
class gtype(SimpleArray, String);
class gtype(SimpleArray, String);
class gtype(SimpleArray, bool);

class FmValueEditor {
public:
	FmValueEditor(FmValueObj *obj);
	virtual ~FmValueEditor();
	
	FmValueObj *object() const;
	wxItem *item(Int row) const;

	virtual String type() const = 0;

	// creation/deleting wxItem
	FM_DECLARE_WXOBJECTS
	
	void updatePosition();

	// set/get value
	void setValue(Int row, const String &value);
	String getValue(Int row) const;

	// controlling behaviour
	void setSkip(Int row, bool skip);
	void setReadOnly(Int row, bool readonly);

	// optional featires
	virtual bool dontProcessKey(wxKeyEvent &ev) = 0;
	virtual bool canHaveFocus() const = 0;
	virtual bool formattedValue() const = 0;

protected:
	virtual wxItem *createItem(wxPanel *, int x, int y, int w, int h,
			Int style, char *name) = 0;

	virtual void setValue(wxItem *, const String &value) = 0;
	virtual String getValue(wxItem *) const = 0;

	virtual void setSkip(wxItem *, bool skip) = 0;
	virtual void setReadOnly(wxItem *, bool readonly) = 0;

	static void actionFunc(wxObject& object, wxEvent& event);

private:
	// to prevent copying
	FmValueEditor(const FmValueEditor &);
	const FmValueEditor &operator=(const FmValueEditor &);
	
	friend class FmValueObj;
	FmValueObj *object_;
	gtype(PtrArray, wxItem) &items_;
	gtype(SimpleArray, bool) &shown_;
};

#define WX_ITEM_TYPE(wxClass)				\
	typedef wxClass wxItemClass;			\
	static wxItemClass *field(wxItem *itm);	\
	static wxItem *item(wxItemClass *itm);	\

#define WX_ITEM_TYPE_IMPLEMENT(EditorClass)							\
inline EditorClass::wxItemClass *EditorClass::field(wxItem *itm)	\
{																	\
	return (wxItemClass*)itm;										\
}																	\
inline wxItem *EditorClass::item(EditorClass::wxItemClass *itm)		\
{																	\
	return itm;														\
}																	\

class wxField;

class FmTextEditor : public FmValueEditor {
	typedef FmValueEditor inherited;

public:
	FmTextEditor(FmValueObj *obj);
	
	String type() const;

	// set/get value
	void setValue(wxItem *, const String &value);
	String getValue(wxItem *) const;
	void _setMasks(const String tstMask, const String outMask);
	
	// controlling behaviour
	void setSkip(wxItem *, bool skip);
	void setReadOnly(wxItem *, bool readonly);
	
	// optional featires
	bool dontProcessKey(wxKeyEvent &ev);
	bool canHaveFocus() const;
	bool formattedValue() const;

protected:
	wxItem *createItem(wxPanel *, int x, int y, int w, int h,
			Int style, char *name);

	wxField *createField(FmCharField *obj, wxPanel *,
			int x, int y, int w, int h, Int style, char *name);
	wxField *createField(FmNumField *obj, wxPanel *,
			int x, int y, int w, int h, Int style, char *name);
	wxField *createField(FmFloatField *obj, wxPanel *,
			int x, int y, int w, int h, Int style, char *name);
	wxField *createField(FmBoolField *obj, wxPanel *,
			int x, int y, int w, int h, Int style, char *name);
	wxField *createField(FmDateField *obj, wxPanel *,
			int x, int y, int w, int h, Int style, char *name);
	wxField *createField(FmTimeField *obj, wxPanel *,
			int x, int y, int w, int h, Int style, char *name);

private:
	WX_ITEM_TYPE(wxField);
};

class FmSelectionEditor : public FmValueEditor {
	typedef FmValueEditor inherited;

public:
	FmSelectionEditor(FmValueObj *obj);
	
	void setValue(wxItem *, const String &value);
	String getValue(wxItem *) const;

	FM_DECLARE_WXOBJECTS

	bool formattedValue() const;

protected:
	wxItem *createItem(wxPanel *, int x, int y, int w, int h,
			Int style, char *name);

	virtual wxItem *createSelection(wxPanel *, int x, int y, int w, int h,
			Int style, char *name, int num, char **list) = 0;
	virtual int getSelection(wxItem *) const = 0;
	virtual void setSelection(wxItem *, int sel) = 0;

private:
	gtype(SimpleArray, String) &values_;
	gtype(SimpleArray, String) &descrs_;
	char **list_;
};

class wxListBoxField;

class FmListBoxEditor : public FmSelectionEditor {
	typedef FmSelectionEditor inherited;

public:
	FmListBoxEditor(FmValueObj *obj);

	String type() const;

private:
	WX_ITEM_TYPE(wxListBoxField);

	wxItem *createSelection(wxPanel *, int x, int y, int w, int h,
			Int style, char *name, int num, char **list);
	int getSelection(wxItem *) const;
	void setSelection(wxItem *, int sel);
	void setSkip(wxItem *, bool skip);
	void setReadOnly(wxItem *, bool readonly);
	bool dontProcessKey(wxKeyEvent &ev);
	bool canHaveFocus() const;
};

class wxRadioBoxField;

class FmRadioBoxEditor : public FmSelectionEditor {
	typedef FmSelectionEditor inherited;

public:
	FmRadioBoxEditor(FmValueObj *obj);
	
	String type() const;

private:

	WX_ITEM_TYPE(wxRadioBoxField);
	
	wxItem *createSelection(wxPanel *, int x, int y, int w, int h,
			Int style, char *name, int num, char **list);
	int getSelection(wxItem *) const;
	void setSelection(wxItem *, int sel);
	void setSkip(wxItem *, bool skip);
	void setReadOnly(wxItem *, bool readonly);
	bool dontProcessKey(wxKeyEvent &ev);
	bool canHaveFocus() const;
};

class wxCheckBoxField;

class FmCheckBoxEditor : public FmValueEditor {
	typedef FmValueEditor inherited;

public:
	FmCheckBoxEditor(FmValueObj *obj);
	
	String type() const;

	// set/get value
	void setValue(wxItem *, const String &value);
	String getValue(wxItem *) const;
	
	// controlling behaviour
	void setSkip(wxItem *, bool skip);
	void setReadOnly(wxItem *, bool readonly);
	
	// optional featires
	bool dontProcessKey(wxKeyEvent &ev);
	bool canHaveFocus() const;
	bool formattedValue() const;

protected:
	wxItem *createItem(wxPanel *, int x, int y, int w, int h,
			Int style, char *name);

private:
	WX_ITEM_TYPE(wxCheckBoxField);
};

class wxSliderField;

class FmSliderEditor : public FmValueEditor {
	typedef FmValueEditor inherited;

public:
	FmSliderEditor(FmValueObj *obj);
	
	String type() const;

	// set/get value
	void setValue(wxItem *, const String &value);
	String getValue(wxItem *) const;
	
	// controlling behaviour
	void setSkip(wxItem *, bool skip);
	void setReadOnly(wxItem *, bool readonly);
	
	// optional featires
	bool dontProcessKey(wxKeyEvent &ev);
	bool canHaveFocus() const;
	bool formattedValue() const;

protected:
	wxItem *createItem(wxPanel *, int x, int y, int w, int h,
			Int style, char *name);

private:
	WX_ITEM_TYPE(wxSliderField);
	
	Int minValue, maxValue;
};

#include <ideafix/priv/fmedit.icc>

#endif

