/********************************************************************
* Copyright (c) 1994 InterSoft Co.  All Rights Reserved
* THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF InterSoft Co.
* The copyright notice above does not evidence any
* actual or intended publication of such source code.
*
* $Id: fmexpr.h,v 1.7 1996/08/21 11:58:31 lohol Exp $
*
* DESCRIPTION
* Class definitions for FmExpr Type.
*********************************************************************/

#ifndef FMEXPR_H
#define FMEXPR_H

#include <ifound/expr.h>
#include <ifound/functype.h>

#if HAVE_PRAGMA_PACK
#	pragma pack(4)
#endif

class FmFieldObj;
class DbExpr;

enum FormOper {
	fm_op_avg,
	fm_op_sum,
	fm_op_min,
	fm_op_max,
	fm_op_count,
};

class FmExpr : public Expr {
public:	
	FmExpr &operator=(const FmExpr &expr);

	String getFormFields();

	bool resolveObjects();

	bool load(FILE *in);
	void store(FILE *out);

	static void FmExpr::execFunction(ExprStack &s, void (FmExpr::*exec)(FmValueObj *));
	static void FmExpr::execFunction(ExprStack &s, void (FmExpr::*exec)(FmValueObj *, Int row));

	void execDescr(FmValueObj *fld, Int row);
	void execChanged(FmValueObj *fld);
	void execSum(FmValueObj *fld);
	void execAvg(FmValueObj *fld);
	void execMin(FmValueObj *fld);
	void execMax(FmValueObj *fld);
	void execCount(FmValueObj *fld);
	void execMulti(FormOper op, FmValueObj *obj);

	static bool resolveKeyHelp(ExprStack &s);
	static bool resolveAccum(ExprStack &s);

private:
	// base form
	FormRep *formRep;

	// field to treat as 'this'
	FmFieldObj *field;
	Int row;

	// target field, to be used to format expression result
	FmValueObj *tfield;

	AnyValue thisval;
	String fldList;
	
	enum {
		THIS = INT_NULL
	};
	void resolveOthers(Instruction::Id op);
	
	void execFmFieldSet();
	void execFmFieldGet();
	FmValueObj *popFmField(Int &row);
	void execDbFieldSet();
	void execDbFieldGet();

protected :
    bool resolveForeignObj( const String &fName, Int &objId, Int &classId );

private :
	Int resolveFmField(const String &fName);

	void addFunctions();

public:
	FmExpr(FormRep *form);
	FmExpr(const Expr &expr, FormRep *form);
	FmExpr(const String &expr, FormRep *form);
	// TODO:
	//FmExpr(const DbExpr &e, Int table, FormRep *form);
	~FmExpr();

	AnyValue eval();
	String formatValue(const AnyValue &v);
	void setInfo(Int r);
	void setInfo(FmFieldObj *f = NULL, Int r = 0, FmValueObj *tf = NULL);
	void setInfo(FmFieldObj *f, Int r, 
			const String &val, FmValueObj *tf = NULL);
	AnyValue execute();

	FmExpr *getFmExpr();
	String getMsg(const String &tab, const String &sec,
			const String &msg);

	Int resolveGlobalObj(const String &fName);

	void init( FormRep *form = 0 );

	static Int compileForeignObjName( const String &name,
			String &sName, String &tName, String &fName );
};

#if HAVE_PRAGMA_PACK
#	pragma pack()
#endif

#endif // FMEXPR_H
