// -----------------------------------------------------------------------
// Copyright (c) 1995 InterSoft Co.  All Rights Reserved
// THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF InterSoft Co.
// The copyright notice above does not evidence any actual or intended 
// publication of such source code.
//
// $Id: fminchk.h,v 1.6 1996/07/15 19:53:34 eugen Exp $
//
// Description:
// -----------------------------------------------------------------------

#ifndef FMINCHK_H
#define FMINCHK_H

#ifdef __GNUC__
#pragma interface
#endif

#include <ideafix/priv/fmrep.h>
#include <ideafix/priv/fmexpr.h>

class EnumObject;
class InCheck;
class IxIntegrity;
class BaseOptionValueArray;
class gtype(OptionValueArray, ValueVal);
class FmInListCheck;
class FmInTableCheck;

class FmInCheck {
public:
	FmInCheck(FmValueObj *field, bool negated = false, bool ignore = false);
	virtual ~FmInCheck();

    FmValueObj *field() const;

	Int fieldRow() const;
	void setFieldRow(Int row);

	bool negated() const;
	void setNegated(bool n);
	bool ignore() const;
	void setIgnore(bool i);

	virtual FmInListCheck *inList();
	virtual FmInTableCheck *inTable();

	// check for existance (ERR - non existant)
	bool goTo(const String &value);
	bool goTo(Int index);

	virtual bool openSequence();
	virtual void closeSequence();

	Int currIndex() const;
	const String &currValue() const;
	const String &currDescr() const;

	Int valueLength() const;
	virtual Int descrLength() const = 0;

private:
	// links
	FmValueObj *field_x;
	Int			fieldRow_d;
	
	// configuration
	bool negated_d;
	bool ignore_d;

	// current state
	Int currIndex_d;
	String currValue_d;
	String currDescr_d;

	virtual bool goTo(const String &value, Int &index, String &descr) = 0;
	virtual bool goTo(Int index, String &value, String &descr) = 0;
};

class FmInListCheck : public FmInCheck {
	typedef FmInCheck inherited;
public:
	FmInListCheck(FmValueObj *field, const EnumObject *enumObj,
			bool negated = false, bool ignore = false);
	FmInListCheck(FmValueObj *field, gtype(OptionValueArray, ValueVal) *val,
			bool negated = false, bool ignore = false);

	~FmInListCheck();

	FmInListCheck *inList();

	Int numValues() const;
	const String &value(Int n) const;
	const String &descr(Int n) const;

	Int maxValueLength() const;
	Int maxDescrLength() const;
	Int minDescrLength() const;

	gtype(OptionValueArray, ValueVal) &list() const;

	static BaseOptionValueArray *createInListValue(InCheck *chk);

	Int descrLength() const;

private:
	gtype(OptionValueArray, ValueVal) &list_d;

	bool goTo(const String &value, Int &index, String &descr);
	bool goTo(Int index, String &value, String &descr);
};

class FmInTableCheck : public FmInCheck {
public:
	FmInTableCheck(FmValueObj *field, IxIntegrity *chk, bool ignore = false);
	FmInTableCheck(FmValueObj *field, InTableVal *val,
				bool hegated = false, bool ignore = false);
	~FmInTableCheck();

	FmInTableCheck *inTable();

	bool remote() const;
	bool opened() const;

	InTableVal &select() const;

	String value();
	String descr(const String &value);

	Int descrLength() const;

private:
	InTableVal &select_d;
	dbcursor cursor;
	Int dbRecord, numRecords;

	bool goTo(const String &value, Int &index, String &descr);
	bool goTo(Int index, String &value, String &descr);
	bool openSequence();
	void closeSequence();
	
	bool seekFirst();
	bool seekLast();
	bool seekNext();
	bool seekPrev();
};

#include <ideafix/priv/fminchk.icc>

#endif

