// -----------------------------------------------------------------------
// Copyright (c) 1995 InterSoft Co.  All Rights Reserved
// THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF InterSoft Co.
// The copyright notice above does not evidence any actual or intended 
// publication of such source code.
//
// $Id: fmmult.h,v 1.7 1996/07/31 14:02:35 kostya Exp $
//
// Description:
// -----------------------------------------------------------------------

#ifndef FMMULT_H
#define FMMULT_H

#ifdef __GNUC__
#pragma interface
#endif

#include <ideafix/priv/fmobj.h>

class FmMultiScrollBar;
class FmMultiScrollBarX;
class FmMultiLineHeader;
class FmMultiLineFooter;

//////////////////////////////////////////////////////////////////

class FmMultiLine : public FmObjContainer {
	typedef FmObjContainer inherited;

	friend class FmMultiScrollBar;

	Int numItems_;
	Int	firstRow_;				// first row in multiple field.
	Int	needFirstRow_;			// to try to set this when enter
	Int	currRow_;				// current row in multiple field.
	Int	deltaY_;				// delta Y on screen

	bool ignoreAdd_;			// Add a new row
	bool ignoreDel_;			// Delete a row
	bool ignoreIns_;			// Insert a row

	FmValueObj *rector_;
	
	FmMultiLineHeader *header_;
	FmMultiLineFooter *footer_;

	FmMultiScrollBar *scrollBar;
	Int deltaRows_;				// temporary add/del last row
	Int lastFirstRow_;

	FmMultiScrollBarX *scrollBarX;
	Int valueX_;				// current value of scrollBarX
	Int valueX0_;				// value of scrollBarX when objects are
								// at the default position
	Int addLengthX_;			// current length of scrollBarX - geom_W()
	
	Int maxRows_d;				// max rows number added for
						// compatibility with Ideafix's
						// multiline
	
	InTableVal *join_d;

 	void setValueX(Int offset);	// set value of scrollBarX

	void scrollToRow(Int row);

	Int currTotalRows() const;
	Int validateFirstRow(Int) const;
	void validateFirstRow();
	
	void updateNumRows(FmBufferRow br);
	void calculateDeltaRow(FmBufferRow br);
	void updateScrollBarPos();
	void updateScrollBarLen();

	bool isNullAll(FmBufferRow br);	// is row -> rowIsClear, else _isNull
	bool validateDest();

	void getBuffer();
	void setBuffer(char *buffer,Int rowSize, Int bufferRows);
	
	FmBuffer *currChildBuffer() const;

	void dumpValues(const String &mess);
	
protected:
	FM_DECLARE_BUILDING
	FM_DECLARE_WXOBJECTS

	void addChild(FmObject *obj, Int position = ERR);
	void delChild(FmObject *obj);
	
public:
	FmMultiLine();
	~FmMultiLine();
	
	FM_DECLARE_CLASS

	virtual String nfmItemName() const;

	virtual FieldType fType() const;
	virtual FmMultiLine *getMultiLine();
	
	void numItems(int numItems);

	inline FmValueObj *rector() const;

	inline FmObjCollection *header() const;
	inline FmObjCollection *footer() const;
	FmObjCollection *headerNotNull();
	FmObjCollection *footerNotNull();

	inline void addToHeader(FmObject *obj);
	inline void addToFooter(FmObject *obj);

	bool inBody(FmFieldObj *obj) const;
	bool inHeader(FmFieldObj *obj) const;
	bool inFooter(FmFieldObj *obj) const;

	bool _isNull(FmBufferRow br) const;
	void _clear(FmBufferRow br);
	void _allocBuffer(FmBufferRow br);
	void _freeBuffer(FmBufferRow br);
	
	bool _copyFieldToDb(FmBufferRow br, InTableVal *join);
	bool _copyFieldFromDb(FmBufferRow br, InTableVal *join);
	bool _removeFieldFromDb(FmBufferRow br, InTableVal *join);

	void _sendBuffer(RequestConnection &conn, FmBufferRow br);
	void _receiveBuffer(RequestConnection &conn, FmBufferRow br);

	virtual void _onConnectBuffer();
	virtual void _onDisconnectBuffer();

	virtual Int rowFromChild(Int numRow) const;

	bool canLeaveRector();
	virtual void enterChildNotify();
	void enterRectorNotify();
	void leaveRectorNotify();
	void onChangeRector(FmBufferRow br);

	virtual bool validate(Int row, const String &value);

	inline bool ignoreAdd() const;
	inline void ignoreAdd(bool theIgnAdd);
	inline bool ignoreDel() const;
	inline void ignoreDel(bool theIgnDel);
	inline bool ignoreIns() const;
	inline void ignoreIns(bool theIgnIns);

	void notifyScrollBar(Int pos);
	void notifyScrollBarX(Int pos);

	virtual FmObjContainer *containerForChild(FmObject *obj);

	virtual Int childNumItems() const;
	virtual Int childNumRows(FmBufferRow br) const;
	virtual Int childFirstRow() const;
	virtual Int childCurrRow() const;
	virtual Int childDeltaY() const;

	Int	nFields() const;

	void deltaY(Int theDeltaY);
	inline Int deltaRelY() const;

	void addRow();
	void delRow();

	virtual void calcNextObjRow(FmFieldObj *&obj, Int &row);
	virtual void calcPrevObjRow(FmFieldObj *&obj, Int &row);
	
	virtual bool onFunctionKey(wxKeyEvent &event);

	virtual bool canShowChild(FmObject *obj, Int row);
	virtual void showChild(FmObject *obj);
 	virtual Int offsetX() const;
 	inline Int addLengthX() const;
 	inline Int lengthX() const;

	InTableVal *join() const;
	bool checkRector();
	
	bool enumerateDown(EnumerateObjects &s);
	bool enumerateUp(EnumerateObjects &s);
	bool enumerateDown(EnumerateObjectsRows &s, FmBufferRow br);
	bool enumerateUp(EnumerateObjectsRows &s, FmBufferRow br);
	
	inline void maxRows(Int maxRows_);	// set & get maxRows
	inline Int  maxRows() const;

};

/////////////////////////////////////////////////////////////

class FmMultiLineHeader : public FmObjCollection {
	typedef FmObjCollection inherited;
	
public:
	FmMultiLineHeader();
	
	FM_DECLARE_CLASS

	bool enumerateDown(EnumerateObjectsRows &s, FmBufferRow br);
	bool enumerateUp(EnumerateObjectsRows &s, FmBufferRow br);

	String nfmItemName() const;
};

class FmMultiLineFooter : public FmObjCollection {
	typedef FmObjCollection inherited;
	
public:
	FmMultiLineFooter();
	
	FM_DECLARE_CLASS

	bool enumerateDown(EnumerateObjectsRows &s, FmBufferRow br);
	bool enumerateUp(EnumerateObjectsRows &s, FmBufferRow br);

	String nfmItemName() const;
};

#include <ideafix/priv/fmmult.icc>

#endif

