/********************************************************************
* Copyright (c) 1994 InterSoft Co.  All Rights Reserved
* THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF InterSoft Co.
* The copyright notice above does not evidence any
* actual or intended publication of such source code.
*
* $Id: fmobj.h,v 1.4 1996/07/15 19:53:42 eugen Exp $
*
* DESCRIPTION
*********************************************************************/

#ifndef FMOBJ_H
#define FMOBJ_H

#ifdef __GNUG__
#	pragma interface
#endif

#include <ideafix.h>
#include <ideafix/priv/fmbuff.h>
#include <ideafix/priv/optnobj.h>

class FmObject;
class FmFieldObj;
class FmValueObj;
class FmObjCollection;
class FmObjContainer;
class FmMultiLine;
class FormRep;
class Form;
class Expr;
class FmExpr;
class RequestConnection;
class InTableVal;
class FmOpt;

class wxItem;
class wxPanel;
class wxFrame;
class wxColour;
class wxKeyEvent;
class IFont;

class gtype(PtrArray, FmObject);

void DUMP(const FmObject *obj);
void DUMP(Int i);

// *************************************************
// these methods should be redefined in EVERY class
// *************************************************
#define FM_DECLARE_ABSTRACT_CLASS	\
	virtual String type() const;

#define FM_DECLARE_ABSTRACT_DATA_CLASS						\
	FM_DECLARE_ABSTRACT_CLASS								\
	virtual bool addOption_(const ObjectOption &option);	\
	virtual bool delOption_(const ObjectOption &option);	\
	virtual bool checkOption_(const ObjectOption &option);

#define FM_NO_OPTIONS(Class)							\
bool Class::addOption_(const ObjectOption &option)		\
{ return inherited::addOption_(option); }				\
bool Class::checkOption_(const ObjectOption &option)	\
{ return inherited::checkOption_(option); }				\
bool Class::delOption_(const ObjectOption &option)		\
{ return inherited::delOption_(option); }

#define FM_NO_CHECK_OPTION(Class)						\
bool Class::checkOption_(const ObjectOption &option)	\
{ return inherited::checkOption_(option); }

#define FM_DECLARE_CLASS			\
	FM_DECLARE_ABSTRACT_DATA_CLASS	\
	virtual Int classId() const;	\
	static const Int classId_;

#define FM_IMPLEMENT_TYPE(Class)	\
String Class::type() const { return #Class; }

#define FM_DECLARE_BUILDING			\
	virtual void _stopBuilding();	\
	virtual void _startBuilding();

	// create/destroy all wxObjects
#define FM_DECLARE_WXOBJECTS		\
	virtual void _wxCreate();		\
	virtual void _wxDestroy();
	
class FmObject : public Resource, public ObjectWithOptions {
	friend class FormRep;

	FmObject(const FmObject&);	   // just declare this in private section
	const FmObject &operator=(const FmObject&);		// to prevent copying.

	FormRep *fm_;					// Pointer to the form
	FmObjCollection *parent_;		// Pointer to parent object
	FmObjContainer *container_;		// Container this object is managed by

	Int style_;
	wxColour *background_;
	wxColour *foreground_;
	Int geom_X_, geom_Y_, geom_W_, geom_H_;
	IFont *font_x;
	Int PAD_0;

protected:
	// make form workable or form editable
	
	FM_DECLARE_BUILDING
	FM_DECLARE_WXOBJECTS

public:
	FmObject();
	virtual ~FmObject();

	FM_DECLARE_ABSTRACT_DATA_CLASS
	virtual Int classId() const;

	inline bool built() const;		// is workable, not allowed to change
	inline bool building() const;	// while building form
	inline bool wxCreated() const;

	void store(FILE *) const;
	bool load(FILE *);
	virtual void storeHeader(FILE *) const;
	virtual bool loadHeader(FILE *);
	void storeChildren(FILE *) const;
	bool loadChildren(FILE *);
	
	static Int ldId(FILE *in);
	static void stId(FILE *out, const FmObject *obj);

	virtual void storeNfm(FILE *out) const;
	void storeNfmChildren(FILE *out) const;

	virtual String nfmItemName() const;
	virtual String nfmHeader() const;

	// options related stuff
	
	enum {
		typeFmExpr = ObjectWithOptions::typeLast,
		typeFunction,
		typeInTable,
		typeFmField,
		typeFmAction,
		typeFmEnum,
		typeLast
	};

	typedef FmOpt Opt;
	static const OptionTable optTable[];
	const OptionTable *optionTable() const;
	OptionValue *createOptionValue(OptionType type);
	
	bool addFmFieldOption(OptionName name,
			const String &field, bool interenal = false);
	bool addFmExprOption(OptionName name,
			const String &expr, bool interenal = false);
	bool addFmExprOption(OptionName name,
			const Expr &expr, bool interenal = false);

	// **********************************
	// methods for interaction with forms
	// **********************************

	void _formRep(FormRep *theForm);
	inline FormRep *formRep() const;
	wxPanel *panel() const;
	wxFrame *frame() const;
	
	// ***********************
	// methods for fields tree
	// ***********************

	// set & get object's parent
	void _parent(FmObjCollection *obj);
	inline FmObjCollection *parent() const;

	// This method should be redefined by container objects
	virtual Int numChildren() const;
	virtual FmObject *child(Int num) const;
	virtual Int childNum(const FmObject *obj) const;

	virtual void addChild(FmObject *obj, Int position = ERR);
	virtual void delChild(FmObject *obj);
	bool isParentOf(FmObject *obj) const;

	// *****************************
	// methods to mantain multilines
	// *****************************

	// set & get object's container
	inline FmObjContainer *container() const;

	inline Int numItems() const;
	inline bool inMult() const;
	inline Int deltaY() const;
	inline Int _itemToRow(Int itemNum) const;	// private
	inline Int _rowToItem(Int rowNum) const;	// private
	inline Int firstRow() const;

	// ********************************
	// methods to change object's state
	// ********************************

	// ***********************
	// general virtual methods
	// ***********************

	struct EnumerateObjects {
		virtual bool execute() = 0;
		virtual bool operator()(FmObject *obj) = 0;
		EnumerateObjects();
	private:
		EnumerateObjects(const EnumerateObjects &);
		const EnumerateObjects &operator=(const EnumerateObjects &);
	};

	struct EnumAll : EnumerateObjects {
		FmObject *object;
		virtual bool operator()(FmObject *obj);
	};
	
	struct EnumFieldObj : EnumerateObjects {
		FmFieldObj *object;
		virtual bool operator()(FmObject *obj);
	};
	
	struct EnumValueObj : EnumerateObjects {
		FmValueObj *object;
		virtual bool operator()(FmObject *obj);
	};
	
	struct EnumerateObjectsRows {
		FmBufferRow br;
		virtual bool execute() = 0;
		virtual bool operator()(FmObject *obj) = 0;
		inline bool operator()(FmObject *obj, FmBufferRow theBr);
		EnumerateObjectsRows();
	private:
		EnumerateObjectsRows(const EnumerateObjectsRows &);
		const EnumerateObjectsRows &operator=(const EnumerateObjectsRows &);
	};
	
	struct EnumFieldObjRows : EnumerateObjectsRows {
		FmFieldObj *object;
		virtual bool operator()(FmObject *obj);
	};
	
	struct EnumValueObjRows : EnumerateObjectsRows {
		FmValueObj *object;
		virtual bool operator()(FmObject *obj);
	};
	
	virtual bool enumerateDown(EnumerateObjects &s);
	virtual bool enumerateUp(EnumerateObjects &s);
	virtual bool enumerateDown(EnumerateObjectsRows &s, FmBufferRow br);
	virtual bool enumerateUp(EnumerateObjectsRows &s, FmBufferRow br);

	// ************************
	// type cast methods
	// ************************

	// get field type
	enum FieldType {
		f_none		= ERR,
		f_alpha		= 0,
		f_num,
		f_date,
		f_time,
		f_mult,
		f_sform,
		f_grouped,
		f_flt,
		f_boolean,
		f_ref,
		maxFieldType
	};
	virtual FieldType fType() const;

	virtual FmFieldObj *getFieldObj();
	virtual FmValueObj *getValueObj();
	virtual FmMultiLine *getMultiLine();

	// **********
	// properties
	// **********

public:

	String nt() const;

	wxColour *background() const;
    wxColour *foreground() const;
	inline wxColour *getBackground() const;
    inline wxColour *getForeground() const;
    void setBackground(wxColour *color);
    void setForeground(wxColour *color);
    
	void style( Int style );
	inline Int style() const;
	
	inline Int geom_X() const;
	inline Int geom_Y() const;
	inline Int geom_W() const;
	inline Int geom_H() const;
	virtual void geometry(Int x = ERR, Int y = ERR, Int w = ERR, Int h = ERR);
	inline IFont *font() const;
	// ****************************
	// methods for managing wxItems
	// ****************************

	// get object's associated item(s)
	virtual wxItem *item(Int rowNum) const;

	void setFmObject(wxItem *item, Int itemNum);
	void resetFmObject(wxItem *item);
	static FmObject *getFmObject(wxItem *item);
	static Int getItemNum(wxItem *item);

 	void updateOffsetAll();

	// ************************
	// specific virtual methods
	// ************************

	bool invalidOption(const ObjectOption &option) const;

	int orientationStyle(int def = 0) const;
	
	virtual void updateOffset(); // update position of field inside its parent

	// **********************
	// action virtual methods
	// **********************

	void print(Int level = 0) const;
	
	void _assert(const char *, const char *, int) const;
	void DUMP() const;
};

class FmFieldObj : public FmObject {
	typedef FmObject inherited;

	// Field type.
	String name_;					// this object's name

	Int objectId_;					// Offset in form's array
	Int offsetInBuffer_;			// offset of buffer

	Int nextObjId_;					// Next object (default traversal).
	Int prevObjId_;					// Prev object (default traversal).

	FmExpr *relCheck_;				// pointer to compiled relational check
	FmExpr *errMsg_;				// Error msg. when check fails

	FmExpr *skipExpr_;				// when to skip
	bool alwaysSkip_;				// skip will be always true (compiled)
	
	bool notNull_;					// true if does not allow null values
	bool unique_;

	bool neverHasBefore_;			// (compiled)
	bool neverHasAfter_;			// (compiled)
	FmExpr *hasBefore_;
	FmExpr *hasAfter_;

public:
	enum IType {
		it_null = ERR,
		it_char,
		it_byte,
		it_short,
		it_long,
		it_double,
		it_date,
		it_time,
		it_num
	};

	IType iType() const;

protected:
	void setIType(IType it);

private:
	IType iType_d;

protected:
	void hasBefore(FmExpr *expr);
	void hasAfter(FmExpr *expr);

	FM_DECLARE_BUILDING

public:
	FmFieldObj();
	~FmFieldObj();

	FM_DECLARE_ABSTRACT_DATA_CLASS

	// ********************************
	// methods for working with buffers
	// ********************************

	virtual Int alignSize() const;
	virtual Int bufferSize() const;
	inline Int offsetInBuffer() const;

	FmBufferRow buffer(Int row) const;		// bufferRow in selected form
	FmBufferRow buffer(FmBuffer *formBuff, Int row) const; 
				// bufferRow in specified form buffer
	Int currentRow() const;

	// *************************
	// multiline related methods
	// *************************

	inline Int numRows(FmBufferRow br) const;
	inline Int currNumRows() const;
	inline Int rowFrom(const FmFieldObj *obj, Int row) const;

	inline void clear(FmBuffer *formBuff, Int row);
	inline void clear(Int row);
	inline bool isNull(FmBuffer *formBuff, Int row) const;
	inline bool isNull(Int row) const;

	virtual void _value(const String &theValue, FmBufferRow br);
	virtual String _value(FmBufferRow br) const;
	virtual bool _isNull(FmBufferRow br) const;
	virtual void _clear(FmBufferRow br);
	
	inline void value(const String &val, FmBuffer *formBuff, Int row);
	inline String value(FmBuffer *formBuff, Int row);
	inline void value(const String &val, Int row);
	inline String value(Int row);

	virtual bool _copyFieldToDb(FmBufferRow br, InTableVal *join);
	virtual bool _copyFieldFromDb(FmBufferRow br, InTableVal *join);
	virtual bool _removeFieldFromDb(FmBufferRow br, InTableVal *join);

	bool copyAllFieldsToDb(FmBufferRow br, InTableVal *join);
	bool copyAllFieldsFromDb(FmBufferRow br, InTableVal *join);
	bool removeAllFieldsFromDb(FmBufferRow br, InTableVal *join);
	
	virtual bool _recalc(Int row, bool *skipFlag = 0);
	void recalcAll(Int row);

	virtual void _onConnectBuffer();	// called when current form's buffer
	virtual void _onDisconnectBuffer();	// is changed
	virtual void _allocBuffer(FmBufferRow br);	// allocate buffer memory
	virtual void _freeBuffer(FmBufferRow br);	// free all buffer memory
	virtual void _sendBuffer(RequestConnection &conn, FmBufferRow br);
	virtual void _receiveBuffer(RequestConnection &conn, FmBufferRow br);

	void allocBufferAll(FmBufferRow br);
	void freeBufferAll(FmBufferRow br);
	void clearAll(FmBufferRow br);
	void clearAll();
	
	void onConnectBufferAll();		// called when current buffer is changed
	void onDisconnectBufferAll();	// called when current buffer is changed

	// **********************************
	// methods for interaction with forms
	// **********************************

	// get & set object's name
	inline const String &name() const;
	inline String nt() const;

	// set & get object descriptor
	void newObjectId();
	void clearObjectId();
	inline Int objectId() const;
	
	const String &defaultSchema() const;

	// **********
	// properties
	// **********

	// get help message's number
	String errorMsg(const String &val, Int row);
	
 	// get hasIs_ flag
	inline bool alwaysSkip() const;
	bool haveToSkip(Int row);
	bool hasBefore(Int row);
	bool hasAfter(Int row);

	inline bool notNull() const;
	inline bool unique() const;

	// ***********************************
	// methods for editing form
	// ***********************************

	bool fieldError(const String &msgName) const;
	virtual bool validate(Int row, const String &value);
	bool validate(Int row);
	bool validateRelCheck(Int row, const String &value);

	bool canEnter(Int row);
	bool canLeave(Int row, bool reset = false);
	virtual void enterNotify();
	virtual void leaveNotify();
	bool userBefore(Int row);
	bool userAfter(Int row);

	virtual bool canHaveFocus() const;

	virtual void savePrevValue(Int row);
	virtual bool assignDefault(Int row);
	virtual String descrMsg(Int row);
	void showDescription(Int row);
	void hideDescription(Int row);
	virtual void saveSkipValues(Int row);
	virtual void restoreSkipValues(Int row);

	virtual Fm::Event doSubform(Int row, Int subf_index, bool manual,
								bool invoke_always);

	void assignDefaultsAll(FmBufferRow br);
	void assignDefaultsAll(Int row);

	void undo();

	// ************************
	// type cast method
	// ************************

	virtual FmFieldObj *getFieldObj();

	// **********************
	// action virtual methods
	// **********************

	virtual void updateFirstRow();
	
	// *************************
	// traversal related methods
	// *************************
	
	void setFocus(Int row);
	
	bool isCurrent(Int row) const;
	bool isCurrent() const;
	
	virtual void calcNextObjRow(FmFieldObj *&obj, Int &row);
	virtual void calcPrevObjRow(FmFieldObj *&obj, Int &row);
	
	virtual bool dontProcessKey(wxKeyEvent &event);
	
	virtual bool onFunctionKey(wxKeyEvent &event);
	virtual bool onHelp();	// returns true if new value was set
};


class FmObjCollection : public FmFieldObj {
	typedef FmFieldObj inherited;

protected:

	gtype(PtrArray, FmObject)	&objs_;

public:
	FmObjCollection();
	virtual ~FmObjCollection();
	
	FM_DECLARE_CLASS

	// ***********************
	// methods for fields tree
	// ***********************

	virtual FmObjContainer *containerForChild(FmObject *obj);

	virtual Int rowFromChild(Int numRow) const;

	virtual Int numChildren() const;
	virtual FmObject *child(Int num) const;
	virtual Int childNum(const FmObject *obj) const;
	virtual void addChild(FmObject *obj, Int position = ERR);
	virtual void delChild(FmObject *obj);

	// find child in path to descendant
	FmObject *childInPath(FmObject *obj) const;

	virtual void calcNextObjRow(FmFieldObj *&obj, Int &row);
	virtual void calcPrevObjRow(FmFieldObj *&obj, Int &row);
	
	virtual void enterChildNotify();
	virtual void leaveChildNotify();

	virtual bool canShowChild(FmObject *obj, Int row);
	virtual void showChild(FmObject *obj);
 	virtual Int offsetX() const;

	virtual bool enumerateDown(EnumerateObjects &s);
	virtual bool enumerateUp(EnumerateObjects &s);
	virtual bool enumerateDown(EnumerateObjectsRows &s, FmBufferRow br);
	virtual bool enumerateUp(EnumerateObjectsRows &s, FmBufferRow br);
	
	FmFieldObj *lastIdObject();
};

// forms and multilies are derived from FmObjCollection
// they are manages numRows, numItems, deltaY and buffers
class FmObjContainer : public FmObjCollection {
	typedef FmObjCollection inherited;

	Int rowSize_;
	
protected:
	FM_DECLARE_BUILDING

public:
	FmObjContainer();
	~FmObjContainer();
	
	FM_DECLARE_ABSTRACT_CLASS

	inline Int rowSize() const;

	virtual Int allocOffset(const FmFieldObj *obj);
	virtual Int alignSize() const;
	virtual Int bufferSize() const;

	virtual FmBuffer *childBuffer(FmBufferRow br) const;

	virtual void allocBuffer(FmBuffer *buff);
										// called by buffer when created
	virtual void freeBuffer(FmBuffer *buff);
										// --//--//--//--//--//- destroyed

	void _onConnectBuffer();	// when buffer becomes current
	void _onDisconnectBuffer(); // when no buffer becomes current
	void _allocBuffer(FmBufferRow br);	// allocate buffer memory
	void _freeBuffer(FmBufferRow br);	// free own part of buffer
	void _sendBuffer(RequestConnection &conn, FmBufferRow br);
	void _receiveBuffer(RequestConnection &conn, FmBufferRow br);

	virtual FmObjContainer *containerForChild(FmObject *obj);

	virtual Int childNumRows(FmBufferRow br) const;
	virtual Int childNumItems() const;
	virtual Int childFirstRow() const;
	virtual Int childCurrRow() const;
	virtual Int childDeltaY() const;

	virtual bool enumerateDown(EnumerateObjects &s);
	virtual bool enumerateUp(EnumerateObjects &s);
	virtual bool enumerateDown(EnumerateObjectsRows &s, FmBufferRow br);
	virtual bool enumerateUp(EnumerateObjectsRows &s, FmBufferRow br);
	
	virtual String getBuffer(const FmValueObj *, FmBufferRow br) const;
	virtual void setBuffer(const String &, const FmValueObj *, FmBufferRow br);

	void setModified(const FmFieldObj *, FmBufferRow br) const;
	void clearModified(const FmFieldObj *, FmBufferRow br) const;

	Int modifiedRows(const FmBuffer *buff) const;
	Int modifiedFields(FmBufferRow br) const;

	inline bool isModified(const FmBuffer *buff) const;
	inline bool isModified(FmBufferRow br) const;
	bool isModified(const FmFieldObj *, FmBufferRow br) const;

	static Int numRows(const FmBuffer *);

private:
	friend class FmBuffer;
	friend class FmSubform;
	friend class FmObjReference;
	
	union Line {
		UnShort modifiedFields;
		char data[1];
	};
	
	virtual Line *rowBuffer(FmBufferRow br) const;
	virtual void insBufferRow(FmBufferRow br) const;
	virtual void delBufferRow(FmBufferRow br) const;

	virtual void *getBuffer(const FmFieldObj *, FmBufferRow br) const;

	static void incModifiedRows(FmBuffer *buff);
	static void decModifiedRows(FmBuffer *buff);
	static void incModifiedFields(Line *line, FmBuffer *buff);
	static void decModifiedFields(Line *line, FmBuffer *buff);
	static void setModifiedBit(const FmFieldObj *obj, 
			Line *line, FmBuffer *buff);
	static void clearModifiedBit(const FmFieldObj *obj, 
			Line *line, FmBuffer *buff);
};

#include <ideafix/priv/fmobj.icc>

#endif

