// -----------------------------------------------------------------------
// Copyright (c) 1995 InterSoft Co.  All Rights Reserved
// THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF InterSoft Co.
// The copyright notice above does not evidence any actual or intended 
// publication of such source code.
//
// $Id: fmopval.h,v 1.5 1996/07/15 19:53:44 eugen Exp $
//
// Description:
// -----------------------------------------------------------------------

#ifndef FMOPVAL_H
#define FMOPVAL_H

#ifdef __GNUG__
#	pragma interface
#endif

#include <ideafix/priv/opval.h>
#include <ideafix/priv/fmrep.h>

class wxColour;
class OptionValue;
class FmOptionValue;
class FmExpr;
class FmFieldObj;
class FormRep;
class InCheck;
class IxIntegrity;
class InTableVal;
class BaseOptionValueArray;
class BaseFmOptionValueArray;

class FmOptionValue : public OptionValue {
	typedef OptionValue inherited;
public:
	typedef FormRep ObjectOwner;

	FormRep *formRep() const;

protected:
	FmOptionValue(FormRep *formRep, ObjectOption::Type);

private:
	FormRep *formRep_x;
};

//
// *** class FmExprVal
//
class FmExprVal : public FmOptionValue {
	typedef FmOptionValue inherited;

public:
	FmExprVal(FormRep *form, const String &s);
	FmExprVal(FormRep *form); // to load later

	DECLARE_OPTION_VALUE(FmObject::typeFmExpr);

	virtual bool resolveFields();

	FmExpr &expr() const;
	const String &exprString() const;

private:
	FmExpr &expr_d;
};

//
// *** class FmIdVal
//
class FmIdVal : public FmOptionValue {
	typedef FmOptionValue inherited;

public:
	const String &name() const;
	Int id() const;

	DECLARE_OPTION_VALUE(FmObject::typeVoid);
	
	bool resolveFields();

	virtual Int nameToId();
	virtual const String &idToName();

protected:
	FmIdVal(FormRep *form, Type t, const String &s);

	void setId(Int id);

private:
	String  name_d;

	// resolved values
	Int		id_d;
};

//
// *** class FmFieldVal
//
class FmFieldVal : public FmIdVal {
	typedef FmIdVal inherited;

public:
	static const Type Type_ = FmObject::typeFmField;

	FmFieldVal(FormRep *form, const String &s = NULL_STRING);

	virtual Int nameToId();
	virtual const String &idToName();
};

//
// *** class FmActionVal
//
class FmActionVal : public FmIdVal {
	typedef FmIdVal inherited;

public:
	static const Type Type_ = FmObject::typeFmAction;

	FmActionVal(FormRep *form, const String &s = NULL_STRING);

	virtual Int nameToId();
	virtual const String &idToName();
};

//
// *** class FmEnumVal
//
class FmEnumVal : public FmIdVal {
	typedef FmIdVal inherited;

public:
	static const Type Type_ = FmObject::typeFmEnum;

	FmEnumVal(FormRep *form, const String &s = NULL_STRING);

	virtual Int nameToId();
	virtual const String &idToName();
};

class gtype(OptionValueArray, FmExprVal);

//
// *** class FunctionVal
//
class FunctionVal : public FmOptionValue {
	typedef FmOptionValue inherited;

public:
	FunctionVal(FormRep *form, const String &name = NULL_STRING);
	~FunctionVal();

	DECLARE_OPTION_VALUE(FmObject::typeFunction)

	bool resolveFields();

	const String& name() const;

	Int numArgs() const;
	FmExpr &argExpr(Int n) const;

	void addArg(FmExprVal *val);
	void addArg(const String &val);
	void addArgs(gtype(OptionValueArray, FmExprVal) *val);

	void insArg(Int pos,const String &val);
	void delArg(Int pos);

private:
	String name_d;
	gtype(OptionValueArray, FmExprVal) &args_d;
};

//
// *** class InTableVal
//
class InTableVal : public FmOptionValue {
	typedef FmOptionValue inherited;

public:
	InTableVal(FormRep *form);
	InTableVal(FormRep *form, const String &schema, const String &table);
	InTableVal(FormRep *form, dbfield fld, IxIntegrity *chk);
	~InTableVal();

	DECLARE_OPTION_VALUE(FmObject::typeInTable);

	bool resolveFields();

	bool setIndex(const String &index);
	
	void setDescending(bool d);
	void addKey(FmExprVal *val);
	void addKey(const String &val);
	void insKey(Int pos,FmExprVal *val);
	void delKey(Int pos);
	void addKeys(gtype(OptionValueArray, FmExprVal) *val);
	void addDescr(FmExprVal *val);
	void addDescr(const String &val);
	void insDescr(Int pos,FmExprVal *val);
	void delDescr(Int pos);
	void addDescrs(gtype(OptionValueArray, FmExprVal) *val);

	Int numKeyExprs() const;
	FmExpr &keyExpr(Int i) const;

	Int numDescrExprs() const;
	FmExpr &descrExpr(Int i) const;

	const String &schemaName() const;
	const String &tableName() const;
	const String &indexName() const;
	bool descending() const;

	dbtable dbTable() const;
	dbindex dbIndex() const;

	dbcursor createCursor(FmFieldObj *obj, Int row, Int mode);
	bool setFixedKeys(FmFieldObj *obj, Int row);

	bool checkFixedKeys(FmFieldObj *obj, Int row) const;

	bool setKey(Int index, const String &value);
	bool setFreeKey(Int index, const String &value);

	String getKey(Int index) const;
	String getFreeKey(Int index) const;

	Int numFreeKeys() const;
	Int totalKeys() const;

	Int currentFreeKey() const;
	Int leftFreeKeys() const;
	void rewindFreeKeys(Int keyNum = 0);
	
	bool allKeysSet() const;
	bool dataReady() const;
	void setDataReady();

	dbfield keyDbField(Int i) const;
	dbfield freeKeyDbField(Int i) const;
	dbfield currentFreeKeyDbField() const;

	bool canSet(dbfield fld);
	bool canGet(dbfield fld) const;

private:
	String s_d;
	String t_d;
	String i_d;
	bool   desc_d;

	gtype(OptionValueArray, FmExprVal) &keys_d;
	gtype(OptionValueArray, FmExprVal) &dsps_d;

	// resolved values
	Int totalKeys_d;
	Int currFreeKey_d;

	dbindex index_d;
};

#include <ideafix/priv/fmopval.icc>

#endif

