// -----------------------------------------------------------------------
// Copyright (c) 1995 InterSoft Co.  All Rights Reserved
// THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF InterSoft Co.
// The copyright notice above does not evidence any actual or intended 
// publication of such source code.
//
// $Id: fmref.h,v 1.2 1996/07/15 19:53:46 eugen Exp $
//
// Description:
// -----------------------------------------------------------------------

#ifndef FMREF_H
#define FMREF_H

#ifdef __GNUC__
#pragma interface
#endif

#include <ideafix/priv/fmobj.h>

class FmObjReference : public FmObjContainer {
	typedef FmObjContainer inherited;

public:
	FmObjReference();

	FM_DECLARE_CLASS
	FM_DECLARE_BUILDING

	String nfmItemName() const;
	FieldType fType() const;

	bool validate(Int row, const String &value);
	
	void addChild(FmObject *obj, Int position = ERR);
	void delChild(FmObject *obj);

	// set & get the current value
	void _value(const String &theValue, FmBufferRow br);
	String _value(FmBufferRow br) const;
	void _clear(FmBufferRow br);
	bool _isNull(FmBufferRow br) const;
	
	bool _recalc(Int row, bool *skipFlag);

	void _onConnectBuffer();
	void _onDisconnectBuffer();
	void _allocBuffer(FmBufferRow br);
	void _freeBuffer(FmBufferRow br);
	void _sendBuffer(RequestConnection &conn, FmBufferRow br);
	void _receiveBuffer(RequestConnection &conn, FmBufferRow br);

	Int bufferSize() const;
	Int allocOffset(const FmFieldObj *obj);
	FmObjContainer *containerForChild(FmObject *obj);
	Int rowFromChild(Int row) const;
	FmBuffer *childBuffer(FmBufferRow br) const;

	bool enumerateDown(EnumerateObjectsRows &s, FmBufferRow br);
	bool enumerateUp(EnumerateObjectsRows &s, FmBufferRow br);

	void calcNextObjRow(FmFieldObj *&obj, Int &row);
	void calcPrevObjRow(FmFieldObj *&obj, Int &row);

	String getBuffer(const FmValueObj *, FmBufferRow br) const;
	void setBuffer(const String &, const FmValueObj *, FmBufferRow br);

///////////////////////////////////////////////////////////////////
//                         new methods
///////////////////////////////////////////////////////////////////

	inline void selectChild(Int n, Int row);
	inline Int selectedChild(Int row) const;
	inline void selectChild(const FmValueObj *obj, Int row);
	inline FmValueObj *dereference(Int row) const;

	inline void selectChild(Int n);
	inline Int selectedChild() const;
	inline void selectChild(const FmValueObj *obj);
	inline FmValueObj *dereference() const;

	void selectChild(Int n, FmBufferRow br);
	Int selectedChild(FmBufferRow br) const;
	void selectChild(const FmValueObj *obj, FmBufferRow br);
	FmValueObj *dereference(FmBufferRow br) const;

	FmValueObj *valueChild(Int i) const;

	bool canShowChild(FmObject *obj, Int row);

private:
	FmExpr *isExpr_;

	struct Data {
		Byte index;
		bool changed;
	};
	
	void *getBuffer(const FmFieldObj *, FmBufferRow br) const;
	Data *getData(FmBufferRow br) const;

	inline void selectChild(Int n, Data *data) const;
	inline Int selectedChild(Data *data) const;
};

#include <ideafix/priv/fmref.icc>

#endif

