// -----------------------------------------------------------------------
// Copyright (c) 1995 InterSoft Co.  All Rights Reserved
// THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF InterSoft Co.
// The copyright notice above does not evidence any actual or intended 
// publication of such source code.
//
// $Id: fmrep.h,v 1.6 1996/07/15 19:53:47 eugen Exp $
//
// Description:
// -----------------------------------------------------------------------

#ifndef WXFMREP_H
#define WXFMREP_H

#ifdef __GNUG__
#	pragma interface
#endif

#include <ideafix/priv/fmobj.h>
#include <ifound/mainobj.h>
#include <ideafix/priv/ifont.h>

class FormRep;
class CFormRep;
class FmAction;
class FmKeyZone;
class FmDataZone;
class Form;
class FmFrame;
class FmPanel;
class FmToolbar;
class EnumObject;
class FmMsgTable;
class ValueVal;

class wxPanel;
class wxFrame;
class wxKeyEvent;

class FmCallBackEvent;
struct FmCallBack;

struct FormalArg {
	enum Type {
		in		= 1 << 0,
		out		= 1 << 1,
		inout	= in | out
	};
	Type type_;
	String name_;
	FmValueObj *field_;
	inline FormalArg();
	inline FormalArg(Type theType, const String &theName);
	inline FormalArg(Type theType, FmValueObj *obj);
	void store(FILE *out);
	bool load(FILE *in);
};

class gtype(PtrArray, FmFieldObj);
class gtype(SimpleArray, String);
class gtype(PtrArray, FormRep);
class gtype(OptionValueArray, ValueVal);
class gtype(PtrArray, FmAction);
class gtype(SimpleArray, FormalArg);
class gtype(PtrArray, EnumObject);
class gtype(PtrArray, IFont);
class gtype(OptionValueArray, StringVal);
class gtype(PtrArray, FmMsgTable);

class FormRep : public FmObjContainer, public FramedObject,
				public FontContainer {
	typedef FmObjContainer inherited;

	friend class FmFrame;
	friend class FmPanel;
//	friend class FmSubform;

	friend class fgen;
	friend int fgenpar_parse();

	FmKeyZone *key_;
	FmDataZone *data_;
	Time timeStamp_d;

protected:

	FmSubform	*formParent_;					// parent of subform (field)
	Int			subformNum_;					// subform identification

	FmBuffer	baseBuffer_d;

	FmBuffer	*selectedBuffer_;
	Int			subformRow_;					// row in parent form

	FormRep		*currentSubform_;

	gtype(PtrArray, FmAction)		&actns_;	// array of form actions
	gtype(OptionValueArray, ValueVal) *msgs_d;	// Form messages (old style)

	gtype(SimpleArray, String)		&schemaNames_d;
												// Schemas used within this form

	gtype(SimpleArray, FormalArg)	&fArgs_;	// formal arguments
	gtype(PtrArray, FmFieldObj)		&idObjs_;	// array of id objects
	gtype(PtrArray, FmObject)		&allObjs_;	// array of all objects
	gtype(PtrArray, EnumObject)		&enums_;	// enums
	gtype(SimpleArray, String)		&subforms_;	// subforms names

	// message table
	static MsgTable *fmsg;

	schema							*schemas_d;	// Schema handles

	Int frameStyle_;
	FmFrame *frame_;					// Frame surrounding us
	FmPanel *panel_;					// Our panel
	FmToolbar *toolbar_;				// Our tool bar

	Int	flags_;							// flsgs from OpenForm
	String label_;						// Window label

	bool invalid_;						// used to check constructor's result

	bool displayed_;

	// whether the read action found a record or not
	bool exists_;

	bool needRecalcIs_;
	bool needRecalcMask_;
	bool autoWrite_;
	bool dspStatus_;

	String		prevVal_;			// prev value
	FmFieldObj	*prevValObj_;
	Int			prevValRow_;

	FmFieldObj	*currObj_;			// current object
	Int			currRow_;			// current instance of object (for multis)
	
	FmFieldObj	*lastObj_;			// previous object (needed for traversal)
	Int 		lastRow_; 			// previous row (needed for traversal)
	
	FmFieldObj	*dstObj_;
	Int			dstRow_;

	FmFieldObj 	*desiredObj_; 		//	If NULL, the form
	Int			desiredRow_;		//	follows the normal sequence,
									//	otherwise it is the next object to
									//	be processed.

	wxKeyEvent	*wxKey_d;			//	last pressed function key,
	Int			keyCode_d;			//	returned by FmKeyCode

	Int nextSubformIndex_;			// subform to select (can be changed in
									// user's after()
	
	InTableVal	*join_d;

	void initValues();					// assignment of default values

	void _toDb(Int tab, Int from, Int to, Int row);
	void _fromDb(Int tab, Int from, Int to, Int row, bool disp);

	void _setCurrObject(FmFieldObj *obj, Int row);
	
	
	friend class CallBackIterator;
	FmCallBack *callBacks;
	
	gtype(PtrArray, FmMsgTable)		&msgTables_d;
	
public:
	enum TraversalKey {
		k_none,
		k_enter,
		k_next,
		k_prev,
		k_left,
		k_right,
		k_down,
		k_up,
		k_toDest
	};
	static TraversalKey traversalKey(wxKeyEvent &ev);

protected:
	static void disableSetFocusEvent();
	static void enableSetFocusEvent();

	bool _traverseResetTo(FmFieldObj *obj);
	bool _traverseTo(FmFieldObj *obj, Int row);
	void _traverse(TraversalKey key);

	void _onConnectBuffer();
	void _onDisconnectBuffer();

	void _allocBuffer(FmBufferRow br);
	void _freeBuffer(FmBufferRow br);
	void _clear(FmBufferRow br);
	bool _copyLineToDb(FmBufferRow br, InTableVal *join);
	bool _copyLineFromDb(FmBufferRow br, InTableVal *join);
	bool _removeLineFromDb(FmBufferRow br, InTableVal *join);

	FM_DECLARE_BUILDING
	FM_DECLARE_WXOBJECTS

public:
	static Int const MAGIC;

	static Int char_width;
	static Int char_height;
	static Int char_descent;

	FormRep();
	FormRep(const String &formName);
	~FormRep();
	
	FM_DECLARE_CLASS

	void stopBuilding();
	void startBuilding();

	Time timeStamp() const;
	
	void storeHeader(FILE *) const;
	bool loadHeader(FILE *);
	void storeNfm();
	void storeNfm(FILE *out);
	void storeNfmEnums(FILE *out) const;
	void storeNfmActions(FILE *out) const;

	bool openAndAlloc(const String &theName, Int theFlags = 0);
	void FormRep::createRootBuffer(void);
	void FormRep::freeRootBuffer(void);
	virtual CFormRep *getCFormRep();

	virtual Fm::Action callUserAction(Fm::Event event);

	Int mainLoop();
	Int subformLoop();
	void exitSubform(Int result);
	virtual void executeAction(FmAction &);
	inline void executeAction(Fm::Event id);
	
	static FormRep *msgCheckerForm;
	static int msgChecker(const char *id);

private:	
    void onShow(); // called just before showing
    void show(bool s);
    bool shown() const;
	void disable();
	void enable();

public:
	inline void flags(Int theFlags);
	inline FmSubform *formOwner() const;
	void formOwner(FmSubform *obj);
	inline FormRep *formParent() const;
	inline bool isSubform() const;

	void selectBuffer(FmBuffer *theBuffer, Int subformRow);
	FmBuffer *selectedBuffer() const;
	inline Int subformRow() const;
	
	inline Int subformNum() const;

	inline FormRep *currentSubform() const;
	Int getSubformId(const String &name) const;

	virtual FmBuffer *childBuffer(FmBufferRow br) const;

	void clearAll();
	void clearKeyFields();
	void clearDataFields();

	FmFieldObj *firstDataField() const;
	
	virtual Int numItems() const;

	virtual Int rowFromChild(Int numRow) const;

	// building facilities
	Int numIdObjects() const;
	bool addIdObject(FmFieldObj *theObject);
	void assignObjIds();
	void clearObjIds();

	Int numAllObjects() const;
	void addObject(FmObject *theObject);
	void delObject(FmObject *theObject);
	FmFieldObj *objectByName(const String &name) const;
	
	Int numSchemas() const;
	Int schemaNum(const String &name) const;
	const String &schemaName(Int index) const;
	schema schemaHandle(Int index) const;

	Int numFormalArgs() const;
	const FormalArg &formalArg(Int n) const;
	void addFormalArg(const String &argName, FormalArg::Type type);
	
	Int numEnums() const;
	gtype(PtrArray, EnumObject) &enums(void);
	Int enumNumber(const String &name) const;
	EnumObject *getEnum(Int num) const;
	EnumObject *getEnum(const String &name) const;
	EnumObject *getEnum(const ObjectOption &opt) const;
	void addEnum(EnumObject *enumObj);

	// set & get exists_ flag
	inline void exists(bool v);
	inline bool exists() const;

	inline String prevValue() const;
	inline void saveValue();
	inline void restoreValue();
	inline bool fieldWasChanged() const;

	// set/get 'needRecalc' flag
	inline void setNeedRecalcIs();
	inline void setNeedRecalcMask();
	inline bool needRecalcIs() const;
	inline bool needRecalcMask() const;

	inline bool autoWrite() const;

	virtual void display(bool s);
	inline bool displayed() const;
	inline bool editable() const;
	// set/get form flags
	void setFlags(Int theFlags);
    inline Int flags() const;

	virtual void addChild(FmObject *obj, Int position = ERR);
	virtual void delChild(FmObject *obj);
	
	// set & get key container object
	void key(FmKeyZone *k);
	inline FmKeyZone *key() const;

	void addToKey(FmObject *obj);

	// check if some object is in a specific zone
	bool inDataZone(FmObject *theObj = NULL) const;	// NULL - current
	bool inKeyZone(FmObject *theObj = NULL) const;	// NULL - current
	bool goFromKeyZone();
	bool goFromDataZone();

	// set & get the current object
	inline bool isCurrent(const FmFieldObj *obj, Int row) const;
	inline FmFieldObj *currObj() const;
	inline Int currRow() const;
	inline FmFieldObj *lastObj() const;
	inline Int lastRow() const;
	inline bool lastMoveWasUp() const;
	void setLastObj(FmFieldObj *obj, Int row);
	
	inline wxItem *currentItem() const;

	// set & get the desired object
	inline void resetDesiredObject();
	void gotoDesiredObject();
	inline void desiredObject(FmFieldObj *theObject, Int rowNum);
	inline FmFieldObj *desiredObject() const;
	inline Int desiredRow() const;
	inline FmFieldObj *dstObj() const;
	inline Int dstRow() const;

	// go to specific object disallowing going to key zone
	bool goToDataObject(FmFieldObj *theObj, Int theNum);

	// go to specific object disallowing going to data zone
	bool goToKeyObject(FmFieldObj *theObj, Int theNum);

	// new traversal functions
	bool inTraverse() const;
	bool traverseToOrBack(FmFieldObj *obj, Int row);
	bool traverseToOrNext(FmFieldObj *obj, Int row);
	bool traverseToOrPrev(FmFieldObj *obj, Int row);
	bool traverseResetTo(FmFieldObj *obj);
	void traverse(wxKeyEvent &event);
	void traverse(TraversalKey code);
	bool nextField(FmFieldObj *obj, Int row);

	void setFocus();

	void clearFlds(Int from = 0, Int to = INT_MAX, Int rowno = 0);

	bool hasKeyZone() const;

	// privilege checking
	bool canUpdate() const;
	bool canAdd() const;
	bool canDel() const;

	// predefined actions
	inline FmAction &nullAction() const;

	// user defined actions
	FmAction &action(Fm::Event id) const;
	inline FmAction &action(Int id) const;
	FmAction &action(const String &theName) const;
	String actionName(Int num);
	Int numActions(void);
	bool delAction(Int theId);
	FmAction *getAction(Int num);
	FmAction &action(const ObjectOption &opt) const;

	FmAction *addAction(Int theId, const String &theName);
	FmAction *addAction(const String &theName);

	// propagate the exists_ flag to each subform
	void markSubforms();

	// return multiline row number of parent

	// assign default values to each object
	void assignDataDefaults();
	void assignKeyDefaults();
	void recalcCurrent();

	bool hasStatus() const;

	// message displaying
	enum MsgType {
		InfoMsg,
		ErrorMsg,
		StatusMsg
	};

	void displayMessage(const String &m, MsgType type = InfoMsg);
	void clearMessage();
	void displayError(const String &m);
	void clearError();
	void displayStatus(const String &m);
	void clearStatus();

	// check for null not-null objects
	FmValueObj *checkKeyNotNulls();
	FmValueObj *checkNotNulls();

	// check if current object is valid
	bool currObjectValid() const;

	inline bool ok() const;
    
	virtual void calcNextObjRow(FmFieldObj *&obj, Int &row);
	virtual void calcPrevObjRow(FmFieldObj *&obj, Int &row);
	
	virtual void onMenuCommand(Int commandId);
	virtual bool onFunctionKey(wxKeyEvent &ev);
	
	bool onHelp(FmValueObj *obj, Int row);
	bool validateInTable(FmValueObj *obj, Int row, const String &val);

	// find the youngest common ancestor between the two objects
	FmFieldObj *youngestCommonAncestor(FmFieldObj *object1, Int rowNum1,
			FmFieldObj *object2, Int rowNum2);

	FmObject *newObjectByClassId(Int classId);
	
	FmFieldObj *idObject(Int n) const;
	FmFieldObj *idObject(const String &name) const;
	FmFieldObj *idObject(const ObjectOption &opt) const;
	Int nKeys() const;

	wxFrame *frame() const;
	wxPanel *panel() const;
	inline FmToolbar * toolbar() const;
	void wxCreateToolBar(FmToolbar *tb);

	// move up & down the object hierarchy
	bool upFromLeaf(FmFieldObj *fObj, Int fNum, FmFieldObj *tObj);
	bool downToLeaf(FmFieldObj *fObj, FmFieldObj *tObj, Int tNum);

	// WxForm message gathering
	static String getMessage(const char *mindex);
	static String getMessage(const String &mindex);
	static String getMessage(const char *mindex, bool infErr);
	static String getMessage(const String &mindex, bool infErr);

	// old form messages
	Int numFormMessages() const;
	const String &formMessage(Int msgNum) const;
	const String &formMessageName(Int msgNum) const;
	const String &formMessage(const String &msgName) const;
	Int formMessageNum(const String &msgName) const;

	Fm::Action errorMessage(Int msgNum, ...);
	Fm::Action errorMessage(Int msgNum, va_list ap);
	void setError(FmFieldObj *f, ...);
	void setError(FmFieldObj *f, va_list ap);
	void setErrorFld(FmFieldObj *f, Int row);

	// message tables
	String getMsg(const String &tab, const String &sec, const String &msg) const;
	void addMsgTable(const String &name, const String &tab, const String &sec);
	void deleteMsgTables();
	void storeMsgTables(FILE *out) const;
	void storeNfmMsgTables(FILE *out) const;
	bool loadMsgTables(FILE *in);

	inline Int keyCode() const;
	inline void keyCode(Int code);
	inline wxKeyEvent *wxKey() const;
	void wxKey(wxKeyEvent &event);
	void wxKeyClear();

	virtual bool canShowChild(FmObject *obj, Int row);
	virtual void enterChildNotify();
	virtual void leaveChildNotify();

	virtual void showChild(FmObject *obj);
 	virtual Int offsetX() const;

	void toDb(Int from, Int to, Int rowno);
	void toTable(Int tab, Int rowno);
	void fromDb(Int from, Int to, Int rowno);
	void fromTable(Int tab, Int rowno);
	void setDisplayOnly(Int from, Int to, bool value);
	bool open(const String &theName, FmSubform *theParent);
	virtual FormRep *createSubform();

	Int checkSum();
	bool verifyCheckSum(Int chksum);
	void recalc(Int curr_row);
	                                 
	enum CallBack {
		callBefore,
		callAfter
	};

	void setCallBack(FmCallBack *cback, bool set);
	bool invokeCallBack(const FmCallBackEvent &ev, FmFieldObj *obj, Int row);
	bool haveCallBack(FmFieldObj *obj, const FmCallBackEvent &ev);

	void setCallBack(FmCallBackEvent *ev,
			bool (FormRep::*fpFmCallBack)( FmFieldObj *, Int ), 
			FmFieldObj *from, FmFieldObj *to );

	bool invokeCallBack( CallBack fType,
			FmFieldObj *obj = NULL, Int row = 0 );
	bool invokeOnKeyCallBack( Int key,
			FmFieldObj *obj = NULL, Int row = 0 );
	bool invokeOnKeyCallBack(const wxKeyEvent &k_event,
			FmFieldObj *obj = NULL, Int row = 0 );

	void setCallBack( CallBack fType, 
		bool (FormRep::*fpFmCallBack)( FmFieldObj* obj, Int row ) = NULL, 
		FmFieldObj *from = NULL, FmFieldObj *to = NULL );
	void setOnKeyCallBack( Int key, 
		bool (FormRep::*fpFmCallBack)( FmFieldObj* obj, Int row ) = NULL, 
		FmFieldObj *from = NULL, FmFieldObj *to = NULL );
	void setOnKeyCallBack(const wxKeyEvent &keyevent, 
		bool (FormRep::*fpFmCallBack)( FmFieldObj* obj, Int row ) = NULL, 
		FmFieldObj *from = NULL, FmFieldObj *to = NULL );

	bool haveCallBack( FmFieldObj *obj, CallBack fType );
	bool haveOnKeyCallBack( FmFieldObj *obj, Int key);
	bool haveOnKeyCallBack( FmFieldObj *obj, const wxKeyEvent &keyevent );
	
	
	inline Int nextSubformIndex() const;
	void nextSubformIndex(Int index);

	//  load and store methods
	void store() const;
	void store(FILE *out) const;
	void storeFormalArgs(FILE *out) const;
	bool loadFormalArgs(FILE *in);
	void storeEnums(FILE *out) const;
	bool loadEnums(FILE *in);
	void storeActions(FILE *out) const;
	bool loadActions(FILE *in);
	
	// From FramedObject
	bool canUseWxWindows() const;
	bool wxWindowsCreate();
	void wxWindowsDestroy();
	inline bool wxCreated() const;

	bool canUseCracker() const;
	bool crackerCreate(RequestConnection &conn);
	void crackerDestroy(RequestConnection &conn);
	bool processRequest(RequestConnection &conn);
	
	void syncFields(RequestConnection &conn, Int cmd);
	void receiveFields(RequestConnection &conn);

	struct EnumerateSubforms : public EnumerateObjects {
		virtual bool execute() = 0;
		virtual bool operator()(FmObject *obj);
		virtual bool execute(FormRep *obj) = 0;
	};
	
	struct EnumSubforms : public EnumerateSubforms {
		FormRep *form;
		virtual bool execute() = 0;
		virtual bool execute(FormRep *obj);
	};
	
	bool enumerateSubforms(EnumerateSubforms &e);
	bool enumerateMeAndSubforms(EnumerateSubforms &e);
	bool loadSubforms();
	
	virtual AnyValue remoteDbField(const String &name);
	virtual bool remoteOnHelp(FmValueObj *obj, Int row);
	virtual bool remoteInTableValidate(FmValueObj *obj, Int row, const String &val);
	
	Fm::Action copyFormFromDb(find_mode mode, bool lock);
	void copyFormToDb();
	void removeFromDb();
	void unlockDb();

	InTableVal *join() const;
	
	IFPCCP getMsgChecker();

private:
	// new private zone
	void setSchema(const String &schName);			// in case of ONE schema
	void addSchemas(gtype(OptionValueArray, StringVal) *arr);
	void openSchemas();
	void closeSchemas();
};

class FmKeyZone : public FmObjCollection {
	typedef FmObjCollection inherited;
	
public:
	FM_DECLARE_CLASS
	
	virtual void calcPrevObjRow(FmFieldObj *&obj, Int &row);
	
	String nfmItemName() const;
};

#include <ideafix/priv/fmrep.icc>

#endif

