// -----------------------------------------------------------------------
// Copyright (c) 1995 InterSoft Co.  All Rights Reserved
// THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF InterSoft Co.
// The copyright notice above does not evidence any actual or intended 
// publication of such source code.
//
// $Id: fmval.h,v 1.7 1996/07/15 19:53:53 eugen Exp $
//
// Description:
// -----------------------------------------------------------------------

#ifndef FMVAL_H
#define FMVAL_H

#ifdef __GNUC__
#pragma interface
#endif

#include <ideafix/priv/fmobj.h>

class FmMultiLine;
class FmSubform;
class FmCheck;
class FmValueEditor;
class EnumObject;
class gtype(OptionValueArray, ValueVal);
class FmInCheck;
class FmInListCheck;
class FmInTableCheck;
class gtype(SimpleArray, String);
class DbFieldVal;

class FmValueObj : public FmFieldObj {
	typedef FmFieldObj inherited;

public:
	enum EditorType {
		Internal,
		Text,
		ListBox,
		RadioBox,
		Choice,
		Slider,
		CheckBox,
		InvEditor
	};

	FmValueObj();
	~FmValueObj();
	
	FM_DECLARE_ABSTRACT_DATA_CLASS

	virtual String nfmDataTypeName() const = 0;
	String nfmEditorName() const;
	String nfmItemName() const;

	FmValueObj *getValueObj();

	virtual FieldType fType() const = 0;
	
	Int bufferSize() const;

	inline EditorType editorType() const;
	inline FmValueEditor *editor() const;
	inline bool internal() const;

	wxItem *item(Int row) const;
	bool dontProcessKey(wxKeyEvent &ev);
	bool canHaveFocus() const;
	void updateOffset();

	void _onConnectBuffer();
	void _onDisconnectBuffer();

	// set & get the current value
	void _value(const String &theValue, FmBufferRow br);
	String _value(FmBufferRow br) const;
	void _clear(FmBufferRow br);
	bool _isNull(FmBufferRow br) const;

	bool _copyFieldToDb(FmBufferRow br, InTableVal *join);
	bool _copyFieldFromDb(FmBufferRow br, InTableVal *join);
	
	bool _recalc(Int row, bool *skipFlag);
	bool isDisplayOnly(Int row);

	void _sendBuffer(RequestConnection &conn, FmBufferRow br);
	void _receiveBuffer(RequestConnection &conn, FmBufferRow br);

	void rectorOf(FmMultiLine *multi);
	inline bool isRector() const;
	inline FmMultiLine *multi() const;

	bool modified(Int row) const;
	void modified(Int row, bool flag);

    bool validate(Int row, const String &value);
    
	// set & get the manual subform attribute
	void manualSform(bool v);
	inline bool manualSform() const;

	// set & get the manual help attribute
	void manualHelp(bool v);
	inline bool manualHelp() const;

	// get the data base field
	dbfield dbField() const;

	// display field's help message
	bool onHelp();

	String descrMsg(Int row);
	String helpMsg(Int row);
	String defaultHelpMsg() const;

	FormRep *useSubform(Int index, Int row = 0);

	// toDo: is this method necessary?
	// fm_cmd doSubform(Int index, Int row = 0);

	inline FmSubform *sform() const;

	enum DisableReason {
		dueIsAlwaysSkip,
		dueInKeyZone,
		dueInMultiLine,
		dueInForm,
		dueInReference
	};

	void enable(DisableReason,bool enable = true);
	inline bool enabled() const;

	// get display only flag
	inline void setDspOnly(bool flag);
	inline bool dspOnly() const;

	// set & get recalc when modified
	inline void needRecalcIs(bool v);
	inline bool needRecalcIs() const;
	inline void needRecalcMask(bool v);
	inline bool needRecalcMask() const;

	void defExpr(const String &expr);
	FmExpr *defExpr() const;
	bool hasDefault() const;

	bool hasIs() const;
	void recalcIs(Int row);
	virtual void recalcMask();
	virtual bool hasMask() const;

	void recalc(Int row);
	void recalcIsAll();
	void recalcAll();
	
	bool changed() const;

	inline Int dataSize() const;

	// get object's associated item(s)

	bool canBeCurrent() const;

	virtual void enterNotify();
	virtual void leaveNotify();

	virtual void savePrevValue(Int row);
	virtual bool assignDefault(Int row);
	virtual void saveSkipValues(Int row);
	virtual void restoreSkipValues(Int row);
	void copySkipValues(Int row);
	void freeSkipValues();

	void toggleSubform();
	Fm::Event doSubform(Int row, Int subf_index, bool manual, bool invoke_always);
	
	void updateFirstRow();

	void onChangeValue(Int row);	// should be called on any change of
									// wxItem's value w/o changing buffer

	// add & query values
	bool haveInCheck() const;
	bool haveInList() const;
	bool haveInTable() const;

	FmInCheck *inCheck() const;
	FmInListCheck *inList() const;
	FmInTableCheck *inTable() const;

	// in list functions
	Int numInLists() const;
	const String &inListValue(Int index) const;
	const String &inListDescr(Int index) const;
	void addInListValue(const String &value, const String &descr = NULL_STRING);
	Int inListIndex(Int row);
	
	// in table functions
	bool remoteInTable() const;

	void clearField();	// to call from action
    void onSetFocus(Int row);

private:	
	friend class FmCheck;
	
	DbFieldVal *dbField_;			// The data base field this form field is
									// related to.  ERR ==> no db field.

	bool dspOnly_;					// true if should not allow modification
									// _but_ should not skip
	bool needRecalcIs_;				// Form's 'IS' need to recalc when modified
	bool needRecalcMask_;			// Form's 'MASK' need to recalc when modified
	bool manualHelp_;				// The user provides the help procedure
	bool manualSform_;				// subform will be opened by user
	bool lazyDefault_;				/*
										Lazy Default
										default that can not be
										calculated at the beginning
										of the input (because it references
										another field),
										and therefore must be delayed
									*/
									
	bool defltMin_;					// Default MIN
	bool defltMax_;					// Default MAX
	
	bool dbValidate_;				// we should to validate inTable
	bool callUserAfter_;			// we should to call user's after

	Char argType_;					// Arg. type (in, out, inout)

	FmExpr *hlpMsg_;				// Help message
	FmExpr *descrMsg_;				// Description message
	
	FmExpr *defExpr_;				// 'default' expression

	Int disabled_;					// 'disable' flags: see DisableReason

	Int dataSize_;

	FmMultiLine *multi_;

	FmSubform *sform_;

	EditorType	editorType_;
	FmValueEditor *editor_;

	FmInCheck		*inCheck_d;
	
	class gtype(SimpleArray, String) *savedSkipFields_d;
	
	FmExpr *isExpr_;

	void initValues();

	void updateItemValue(Int row);

	inline String getBuffer(FmBufferRow br) const;
	inline void setBuffer(const String &val, FmBufferRow br) const;

	void setValue(Int row, const String &value);
	String getValue(Int row) const;
	void updateDisable();

	bool setInEnum(OptionName option, const EnumObject *obj);
	bool setInList(OptionName option, gtype(OptionValueArray, ValueVal) *val);
	bool setInTable(OptionName option, InTableVal *val);
	bool setInList(FmInListCheck *check);
	void deleteInCheck();

	bool isSkipField(const FmValueObj *obj) const;

public:
	virtual String packValue(const String &v) const;
	virtual String formatValue(const String &v) const;
	virtual Int formattedLength() const;

	bool validateUnique(Int row, const String &value);
	bool validateInCheck(Int row, const String &value);

protected:
	// type related functions
	void setNumIType(Int len);

	inline String getBuffer(Int row) const;

	void dataSize(Int dataSize);

	String typeHelpMsg() const;
	virtual String posHelpMsg() const;
	
	FM_DECLARE_BUILDING
	FM_DECLARE_WXOBJECTS

	void setEditorType(EditorType type);
	virtual bool inheritFrom(DbFieldVal *val);
};

/////////////////////////////////////////////////////////////////////
//                  deneral data fields: Fm*Field
/////////////////////////////////////////////////////////////////////

class FmCharField : public FmValueObj {
	typedef FmValueObj inherited;

	FmExpr *maskExpr_;				// 'mask' expression
	
	String testMask_;
	String outputMask_;
	
protected:
	virtual String posHelpMsg() const;
    bool validate(Int row, const String &value);

//	virtual String packValue(const String &v) const;
//	virtual String formatValue(const String &v) const;
//	virtual Int formattedLength() const;

public:
	FmCharField();
	~FmCharField();
	
	FM_DECLARE_CLASS
	
	void _stopBuilding();

	virtual String nfmDataTypeName() const;

	virtual FieldType fType() const;

	virtual bool inheritFrom(DbFieldVal *val);

	inline Int length() const;
	void length(Int len);
	
	void maskExpr(const String &expr);
	FmExpr *maskExpr() const;
	void setMask(const String &expr);

	void recalcMask();
	bool hasMask() const;
	
	inline const String &testMask() const;
	inline const String &outputMask() const;
};

class FmNumField : public FmValueObj {
	typedef FmValueObj inherited;

	Int decimal_;
	bool signed_;
	bool useComma_;
	Num minValue_, maxValue_;

protected:
	virtual String posHelpMsg() const;
    bool validate(Int row, const String &value);

	virtual String packValue(const String &v) const;
	virtual String formatValue(const String &v) const;
	virtual Int formattedLength() const;

public:
	FmNumField();

	FM_DECLARE_CLASS

	virtual String nfmDataTypeName() const;

	virtual FieldType fType() const;

	virtual bool inheritFrom(DbFieldVal *val);

	void digits(Int digits, Int decimal = 0);
	inline Int digits() const;
	inline Int decimal() const;
	
	void setSigned(bool v = true);
	inline bool isSigned() const;

	void useComma(bool v = true);
	inline bool usingComma() const;
	
	inline Num minValue() const;
	inline Num maxValue() const;
};

class FmFloatField : public FmValueObj {
	typedef FmValueObj inherited;

protected:
	virtual String posHelpMsg() const;

//	virtual String packValue(const String &v) const;
//	virtual String formatValue(const String &v) const;
//	virtual Int formattedLength() const;

public:
	FmFloatField();
	
	FM_DECLARE_CLASS

	virtual String nfmDataTypeName() const;

	virtual FieldType fType() const;

	virtual bool inheritFrom(DbFieldVal *val);

	void length(Int length);
	inline Int length() const;
};

class FmBoolField : public FmValueObj {
	typedef FmValueObj inherited;

	static String yes_;
	static String no_;

protected:

	virtual String packValue(const String &v) const;
	virtual String formatValue(const String &v) const;
	virtual Int formattedLength() const;

public:
	FmBoolField();
	
	FM_DECLARE_CLASS

	virtual String nfmDataTypeName() const;

	virtual FieldType fType() const;

	virtual bool inheritFrom(DbFieldVal *val);

	static inline const String &yes();
	static inline const String &no();
};

class FmDateTimeField : public FmValueObj {
	typedef FmValueObj inherited;

protected:
	virtual char delimiter() const = 0;

	virtual String packValue(const String &v) const;
	virtual String formatValue(const String &v) const;
	virtual Int formattedLength() const;

public:
	FmDateTimeField();
};

class FmDateField : public FmDateTimeField {
	typedef FmDateTimeField inherited;

protected:
	virtual char delimiter() const;

public:
	FmDateField();
	
	FM_DECLARE_CLASS

	virtual String nfmDataTypeName() const;

	virtual FieldType fType() const;

	virtual bool inheritFrom(DbFieldVal *val);

	void digits(Int digits);
	inline Int digits() const;
};

class FmTimeField : public FmDateTimeField {
	typedef FmDateTimeField inherited;

protected:
	virtual char delimiter() const;

public:
	FmTimeField();
	
	FM_DECLARE_CLASS

	virtual String nfmDataTypeName() const;

	virtual FieldType fType() const;

	virtual bool inheritFrom(DbFieldVal *val);

	void digits(Int digits);
	inline Int digits() const;
};

#include <ideafix/priv/fmval.icc>

#endif
