// -------------------------------------------------------------------------
// Copyright (c) 1995 InterSoft Co.  All Rights Reserved
// THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF InterSoft Co.
// The copyright notice above does not evidence any actual or intended 
// publication of such source code.
//
// $Id: gridcol.h,v 1.5 1996/07/30 15:11:13 ksa Exp $
//
// Description:
// -------------------------------------------------------------------------

#ifndef GRIDCOL_H
#define GRIDCOL_H

class GridIcon;

class GridColumn : public Resource {
protected:

	String head_d;         // Header
	String mask_d;         // Format mask
	Int width_d;           // Width of the column
	Int type_d;            // Type of the column
	String buff_d;         // Buffer for this column
          
	Int real_offset_d;
	Int real_width_d;
	bool locked_d;
	bool showed_d;

	bool clickable_d;
	bool sizeable_d;
	bool lockable_d;
	bool printable_d;

public:

	GridColumn(const String &h, Int w, Int t = TY_STRING,
			Int flags = ERR, String mask = NULL_STRING);
	virtual ~GridColumn();
	
	virtual void* userData() { return NULL;}  // pointer to user data (if any)
	virtual String value()	 { return NULL_STRING; } // string of ^^( no format)
	virtual void value(const String&) {}             // string to ^^
	inline const String& mask() const;
	inline bool locked() const;
	inline void locked(bool lk);

	inline bool show() const;
	inline void show(bool sh);

	inline bool clickable() const;
	inline void clickable(bool cl);

	inline bool sizeable() const;
	inline void sizeable(bool cl);

	inline bool lockable() const;
	inline void lockable(bool cl);

	inline bool printable() const;
	inline void printable(bool cl);

	inline Int type() const;
	inline const String &header() const;

	inline String buff();
	virtual String format(String str);

	inline Int width() const;
	void width(Int width);

	inline Int offset() const;
	inline void offset(Int offset);

	inline void flags(Int f);
	inline Int  flags();

	virtual void operator<<(const String &);
	virtual void operator<<(const char *);
};

class SpecialCol : public GridColumn
{

public:

	SpecialCol();
	~SpecialCol();
	void lock(bool) {};
};

class StringColumn : public GridColumn
{

	String *theString_d;

public:

	StringColumn(String &theString, const String &header, Int w,Int f,
		String mask);
	StringColumn(const String &header, Int w, Int f, String mask);
	~StringColumn();
	virtual void* userData();
	virtual String value();
	virtual void value(const String&);
	virtual String format(String str);

};

class NumColumn : public GridColumn
{

	Num *theNum_d;
public:
	bool is_mask_d;
	Int  ndec_d, ndig_d, sign_d;
	char filler_d;
	bool separ_d;

public:
	NumColumn(Num &theNum, const String &header, Int w, Int f,
			String mask );
	NumColumn(Num &theNum, const String &header, Int w, Int f, Int ndig,
			  Int ndec , bool separ, Int sign, char filler);
	NumColumn(const String &header, Int w, Int f, String mask);
	NumColumn(const String &header, Int w, Int f, Int ndig,
			  Int ndec , bool separ, Int sign, char filler);
	~NumColumn();
	virtual void* userData();
	virtual String value();
	virtual void value(const String&);
	virtual String format(String str);
};

class DateColumn : public GridColumn
{

	Date *theDate_d;

public:

	DateColumn(Date &theDate, const String &header, Int w, Int f,
			String mask);
	DateColumn(const String &header, Int w, Int f, String mask);
	~DateColumn();
	virtual void* userData();
	virtual String value();
	virtual void value(const String&);
	virtual String format(String str);
};

class TimeColumn : public GridColumn
{

	Time *theTime_d;

public:

	TimeColumn(Time &theTime, const String &header, Int w, Int f,
			String mask);
	TimeColumn(const String &header, Int w, Int f, String mask);
	~TimeColumn();
	virtual void* userData();
	virtual String value();
	virtual void value(const String&);
	virtual String format(String str);
};

class BoolColumn : public GridColumn
{

	Boolean *theBoolean_d;

public:

	BoolColumn(Boolean &theBoolean, const String &header, Int w, Int f,
			String mask);
	BoolColumn(const String &header, Int w, Int f, String mask);
	~BoolColumn();
	virtual void* userData();
	virtual String value();
	virtual void value(const String&);
	virtual String format(String str);
};

class IntColumn : public GridColumn
{
	Int *theInt_d;

public:

	IntColumn(Int &theInt, const String &header, Int w, Int f,
			String mask);
	IntColumn(const String &header, Int w, Int f, String mask);
	~IntColumn();
	virtual void* userData();
	virtual String value();
	virtual void value(const String&);
	virtual String format(String str);
};

class IconColumn : public GridColumn
{

	GridIcon *theIcon_d;

public:

	IconColumn(GridIcon &theIcon, const String &header, Int w, Int f,String mask);
	IconColumn(const String &header, Int w, Int f,String mask);
	~IconColumn();
	virtual void* userData();
	virtual String value();
	virtual void value(const String&);
	virtual String format(String str);
};

#include <ideafix/priv/gridcol.icc>

#endif
