// -------------------------------------------------------------------------
// Copyright (c) 1995 InterSoft Co.  All Rights Reserved
// THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF InterSoft Co.
// The copyright notice above does not evidence any actual or intended 
// publication of such source code.
//
// $Id: gridrep.h,v 1.7 1996/07/30 15:11:19 ksa Exp $
//
// Description:
// Declaration of the Grid object and related functions, for
// creating grid filled with text strings.  
// -------------------------------------------------------------------------
                                           
#ifndef GRIDREP_H
#define GRIDREP_H

#ifdef __GNUC__
#	pragma interface
#endif

#define OUT_PIPE		(UnShort) 0
#define OUT_FILE		(UnShort) 1
#define OUT_PRINTER		(UnShort) 2
#define OUT_TERMINAL	(UnShort) 3
#define OUT_DEFAULT		(UnShort) 4
#define OUT_WX			(UnShort) 5
#define OUT_PS			(UnShort) 6
#define OUT_DUMP		(UnShort) 7
#define OUT_DELIMITED	(UnShort) 8
#define OUT_FIXED		(UnShort) 9
#define OUT_FORMATTED	(UnShort) 10
#define OUT_CRACKER		(UnShort) 11

#define CGridRep BaseOutput

class GridColumn;
class GridDev;
class WxGridDev;
class Grid;   
class CGridRep;
class String;
class GridIcon;
typedef String *StringPtr;
class gtype(PtrArray, StringPtr);
class gtype(PtrArray, GridColumn);
class gtype(PtrArray, Int);

class GridRep : public Resource {
protected:

	Grid *grid_d;
	String name_d;

	gtype(PtrArray,StringPtr) *lines__d;   // Buffer for all
	StringPtr* pool__d;                    // added array
	gtype(PtrArray,GridColumn) *columns__d;// Columns
	gtype(PtrArray,Int) *slines__d;        // Selected lines

	FILE *fp_d;
	FILE *fp_index_d;

	Int       currCol_d;		// Current output column
	Int       currRow_d;		// Current output row
	Int       poolRow_d;		// Row in pool
	Int       lastLocked_d;		// Last locked column
	Int       clickOnRow_d;		// 

	Int       pageHeight_d;		// page Height
	Int       pageWidth_d;		// page Width
	Int       virtualWidth_d;	// virtual Width
	Int       virtualHeight_d;	// virtual Height

	Int       base_d;
	Int       items_d;			// Number of items listed
	Int       offset_d;			// Current offset

	bool gridIsBuild_d;			// flag of grid ending
	bool gridIsDone_d;			// flag of grid ending
	bool gridIsAbort_d;			// flag of grid aborting
	bool showNumbers_d;			// flag shows numbers
	
	GridDev *currDev_d;			// current device
	void *clickedOn_d;

	void _restart();
	void createTmpFiles();

public:

	GridRep(Grid *grid, const String &name);
	virtual ~GridRep();

///////////////////// ALL ABOUT GRIDREP //////////////////

	inline Int pageWidth() const;
	inline Int pageHeight() const;
	Int virtualWidth();
	Int virtualHeight();
	bool isNumbers() const;
	Int numbersWidth();
	inline Int base() const;
	inline Int items() const;
	inline bool gridIsDone() const;
	inline bool gridIsAbort() const;
	inline void endGrid();
	virtual CGridRep *cGridRep();

	void build(bool full=true);
	void prepare();
	bool execute();
	virtual bool restart();
	virtual void abortGrid();
	void showNumbers(bool show);
	Int scroll(Int n = 1);
	void setLines(Int y = ERR, Int x = ERR);
	void base(Int newbase);

	virtual void onClick(Int col, Int row);
	bool clickedOn(void *) const ;
	Int  idOf(void *) const;
	bool row(Int r);
	inline Int row() const;
	const String& value(Int row, Int col);
	const String& value(Int colId) const;
	void value(Int colId, const String &v);

///////////////////// ALL ABOUT LINES   //////////////////

	void freeLines();
	void selectLine(Int row);
	void unSelectLine(Int row);
	bool lineSelected(Int row);
	bool isSelection();
	Int  nLinesSelected();
	Int  selected(Int);

///////////////////// ALL ABOUT COLUMNS //////////////////

	Int addColumn(GridColumn *);

	Int addCol(String &theString, const String &title, Int width,
			Int flags = 0, const String &mask = NULL_STRING);
	Int addCol(Date &theDate, const String &title, Int width, Int flags = 0, 
			const String &mask = NULL_STRING);
	Int addCol(Time &theTime, const String &title, Int width,
			Int flags = 0, const String &mask = NULL_STRING);
	Int addCol(Boolean &theBoolean, const String &title, Int width,
			Int flags = 0, const String &mask = NULL_STRING);
	Int addCol(Num &theNum, const String &title, Int width,
			Int flags = 0, const String &mask = NULL_STRING);
	Int addCol(Num &theNum, const String &title, Int width, Int flags, 
			Int ndig, Int ndec = 0, bool separ = true,
			Int sign = SIGN_LEADING, char filler = ' ');
	Int addCol(Int &theInt, const String &title, Int width,
			Int flags = 0, const String &mask = NULL_STRING);
	Int addCol(GridIcon &theIcon, const String &title, Int width,
			Int flags=0 , const String &mask = NULL_STRING);

	Int addTypedCol(Int ty, const String &title, Int width, Int flags=0,
			const String &mask = NULL_STRING);
	Int addTypedCol(Int ty, const String &title, Int width, Int flags,
			Int ndig, Int ndec = 0, bool separ = true,
			Int sign = SIGN_LEADING, char filler = ' ');

	void flags(Int column, Int flags);

	Int columnWidth(Int n) const;
	Int columnOffset(Int n) const;
	bool columnLocked(Int n) const;
	Int columnType(Int n) const;
	GridColumn *gridColumn(Int n) const;

	Int columnNdec(Int n);
	Int nCols() const;
	inline Int currCol() const;
	void showColumn(Int c,bool sh);
	bool columnShowed(Int c) const;
	bool columnClickable(Int c) const;
	bool columnSizeable(Int c) const;
	bool columnLockable(Int c) const;
	bool columnPrintable(Int c) const;
	void lockColumn(Int c);
	Int lastLocked();
	void columnWidth(Int col, Int width);

///////////////////// ALL ABOUT OUTPUT  //////////////////

	void outputLine(bool full = true);
	void readFromFile(Int n, Int qn = 1);
	void logInFile(StringPtr *);
	String formattedValue(Int, Int);
	virtual bool onNextLine();
	virtual void getLinesFromUser(Int startRow, Int n = 1);
	void getLinesFromUser_(Int n);
    bool getRows(Int from,Int to);
//////////////////// ALL ABOUT DEVICES ///////////////////

	inline GridDev *currDev() const;
	inline void currDev(GridDev *dev);
	virtual bool memorizeDev(GridDev *);
	bool openWxGridDev(Int x = ERR, Int y = ERR, Int w = ERR,Int h = ERR);
	bool openPSGridDev();
	bool openCDGridDev(Int w, Int h, int dest, int ncopies);
	bool openDLGridDev(int dest, int ncopies);
	bool openFSGridDev(int dest, int ncopies);
	bool openFMGridDev(String heading, String footing, int height,
						int topm, int botm, int leftm, int dest, int ncopies);
	
};

#include <ideafix/priv/gridrep.icc>

#endif

