// -------------------------------------------------------------------------
// Copyright (c) 1995 InterSoft Co.  All Rights Reserved
// THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF InterSoft Co.
// The copyright notice above does not evidence any actual or intended 
// publication of such source code.
//
// $Id: gridwin.h,v 1.8 1996/08/20 14:06:09 eugen Exp $
//
// Description:
// -------------------------------------------------------------------------

#ifndef GRIDWIN_H
#define GRIDWIN_H

#ifdef __GNUC__
#	pragma interface
#endif

#define gridSMALL_FONT   0
#define gridMEDIUM_FONT  1
#define gridLARGE_FONT   2

#include <wxwin/enter_wx.h>
#include <wx_canvs.h>
#include <wx_frame.h>
#include <wxwin/leave_wx.h>

#include <ideafix/priv/gridwx.h>

class wxItem;
class wxBrush;
class wxPen;
class wxFont;
class wxColour;
class wxKeyEvent;
class wxMouseEvent;

class FontBox;
class ShowBox;
class SaveBox;


class GCanvas : public wxCanvas {
public:

	enum COLORS {
		WHITE = 0,
		FORE  = 1,
		HALF  = 2,
		BACK  = 3,
		DEEP  = 4,
		BLACK = 5
	};

	enum FONTS  {
		NO_FONT     = -1,
		SMALL_FONT  = 0,
		MEDIUM_FONT = 1,
		LARGE_FONT  = 2
	};

protected:

	WxGridDev  *gdev_;
	wxPen      *pen_;
	wxBrush    *brush_;
	wxColour   *pen_colour_;
	wxColour   *brush_colour_;
	wxColour   *text_colour_;
	wxFont     *font_;

	wxCursor *size_c_;
	wxCursor *norm_c_;
	wxCursor *curr_c_;
	wxCursor *wait_c_;

	UnInt fore_;
	UnInt half_;
	UnInt back_;
	UnInt deep_;
	UnInt black_;
	UnInt white_;
	enum FONTS font_size_;
	Bool bold_;
	Int char_width;
	Int char_height;
	Int char_descent;
	Int interLines_;

	Int start_x_;
	Int last_x_;
	Int selected_column_;

	Int type__;

public:

	/*explicit*/ GCanvas(WxGridDev *table);
	virtual ~GCanvas();

	void SetClippingRegion(Int x,Int y,Int w,Int h);

	virtual void makeColors();
	void SetPen(enum GCanvas::COLORS color = BLACK,int style = wxSOLID,int width = 0);
	void SetBrush(enum GCanvas::COLORS color = BACK,int style = wxSOLID);
	wxCursor *SetCursor(wxCursor *cursor);
	virtual void setText(enum GCanvas::COLORS color = BLACK);
	Bool Show(Bool show);
	virtual void Clear();
	wxFont* font();

	virtual void setFontAttributes(GCanvas::FONTS font = SMALL_FONT,Bool bold = FALSE);
	virtual void getFontAttributes(GCanvas::FONTS *,int *);

	virtual Int addOffset() const = 0;

	virtual void drawButton(int x,int y,int w,int h,bool press = false,bool fill = false);

	virtual	bool checkDelimiter(Int c,Int x,Int w);
	virtual	bool checkColumn(Int c,Int x,Int w);
};

class GridFrame : public wxFrame
{

private:

	WxGridDev *gdev_;
	FontBox *fontBox_;
	ShowBox *showBox_;
	SaveBox *saveBox_;

	Bool fontChanged_;
	wxMenuBar *menuBar_;

public:

	GridFrame(WxGridDev *table, wxFrame *parent, char *title, int x=-1, int y=-1,
			  int width=-1, int height=-1, int style=wxSDI|wxDEFAULT_FRAME);

	virtual ~GridFrame();

	void SetSizeHints(int minW, int minH,int maxW, int maxH, int incW, int incH);
	virtual Bool OnClose();	
	virtual void OnSize(int w, int h);
	void updateSizeHints();
	void updateNumbersMenu();
	void updateShowBox();
	void printGrid();
	void saveGrid();
	void showCharts();
	
	void OnMenuCommand(int choise);

	virtual Bool isFontChanged();
	virtual void setFontAttributes(GCanvas::FONTS hfont = GCanvas::SMALL_FONT,
								   Bool hbold = TRUE,
								   GCanvas::FONTS gfont = GCanvas::SMALL_FONT,
								   Bool gbold = FALSE,
								   Bool mode = FALSE);
};


class GridCanvas : public GCanvas
{

protected:

	Int select_row_;
	Int select_col_;
	Int start_row_;
	Int last_row_;
	Int start_col_;

public:

	/*explicit*/ GridCanvas(WxGridDev *table);
	virtual ~GridCanvas();

	void OnPaint();
	void OnChar(wxKeyEvent &);
	void OnEvent(wxMouseEvent &);

	void SetClippingRegion(Int x, Int y, Int w, Int h);
	virtual void Clear();
	virtual void Clear(Int, Int, Int, Int, GCanvas::COLORS color = WHITE,
					   int style = wxCOPY);
	void Scroll(int, int);

	virtual Int addOffset() const { return 0; };
	Int sbarWidth();

	Int charWidth() const	{ return char_width;	};
	Int charHeight() const	{ return char_height;	};
	Int charDescent() const	{ return char_descent;	};

	virtual Int xCharToPixel(Int xpos=1)const;
	virtual Int yCharToPixel(Int ypos=1)const;
	virtual Int xPixelToChar(Int xpos)const;
	virtual Int yPixelToChar(Int ypos)const;
	virtual void where(Int *,Int *,Int *);

	virtual void drawColumn(Int col,Int x,Int from,Int w,Int to);
	virtual void drawColumns(Int from,Int to);
	virtual void drawColumns(Int x,Int y,Int w,Int h);
	virtual void drawDelimiter(Int col,Int from,Int qn);
	virtual void drawCell(Int col,Int row);
	virtual void drawCellSelection();
	virtual void scrollText(Int col,Int row);
	virtual void drawSelectedLine(Int row);

	virtual void drawLeadings(Int col,Int from,Int qn);
	virtual Int stringOffset(char *str,Int col,Int c_off);

	virtual void drawBoundCursor(Int x);
	virtual void drawTriangle(Int x,Int y,Int dir = 0);

	Int interLines() const	{ return interLines_; };
	void interLines(Int i)		{ interLines_ = i; };

};

class LGridCanvas : public GridCanvas
{

public:

	/*explicit*/ LGridCanvas(WxGridDev *table);
	~LGridCanvas();
	virtual void updateSize();
	virtual void scroll(Int n);
	virtual void OnPaint();
	void OnEvent(wxMouseEvent &);
	void Scroll(int, int);
	Bool Show(Bool show);

	virtual void drawColumns(Int x, Int y, Int w, Int h);
	virtual void drawCell(Int row, Int col);
	virtual void drawBoundCursor(Int x);

	virtual Int width();
	virtual Int addOffset() const;
};

class HeadCanvas : public GCanvas
{

	Int popupColumn_d;
	wxMenu* popupMenu_;
	
public:

	/*explicit*/ HeadCanvas(WxGridDev *table);
	virtual ~HeadCanvas();

	void OnPaint();
	void OnChar(wxKeyEvent &);
	void OnEvent(wxMouseEvent &);
	void popupMenu(float x,float y);
	void popupFunction(Int command);
	void SetClippingRegion(Int x, Int w);
	void Clear();
	virtual void Clear(Int, Int, GCanvas::COLORS color = BACK,
					   int style = wxCOPY);

	virtual Int addOffset() const;

	virtual void drawDelimiter(Int col,bool inverse=false,bool fill=true);
	virtual void drawHeader(Int col,Int x,Int w);
	virtual void drawHeaders(Int x,Int w);
	virtual void moveArea(Int x1,Int w,Int x2);

	virtual void where(Int *,Int *);
	virtual void drawBoundCursor(Int x);

	Int	interLines() const	{ return interLines_; };
	void interLines(Int i)	    { interLines_ = i; };

	virtual Int height();
	
};

class LHeadCanvas : public HeadCanvas
{

public:

	/*explicit*/ LHeadCanvas(WxGridDev *table);
	virtual ~LHeadCanvas();

	void OnPaint();
	virtual void updateSize();
	virtual void drawHeaders(Int x, Int w);
	virtual void drawBoundCursor(Int x);
	virtual Int addOffset() const { return 0; };

};

#endif
