// -----------------------------------------------------------------------
// Copyright (c) 1995 InterSoft Co.  All Rights Reserved
// THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF InterSoft Co.
// The copyright notice above does not evidence any actual or intended 
// publication of such source code.
//
// $Id: ifont.h,v 1.2 1996/07/15 19:54:03 eugen Exp $
//
// Description:
// -----------------------------------------------------------------------

#ifndef IFONT_H
#define IFONT_H

#ifdef __GNUG__
#	pragma interface
#endif

#include <ideafix/priv/optnobj.h>

class wxFont;
class FontContainer;
class Connection;

class gtype(PtrArray, IFont);

class IFont : public ObjectWithOptions {
public:
	enum Family {
 		defaultf,
		decorative,
  		roman,
		script,
		swiss,
		modern,
		teletype
	};

	enum Style {
 		normal,
 		italic
	};

	enum Weight {
  	 	medium,
  		light,
  		bold
	};

	struct Opt {
		// options
		enum {
			size,
			multiply,
			style,
			family,
			weight,
		};
	};

	IFont(FontContainer &owner, const String &name,
			IFont *base = NULL, bool internal = false);
	~IFont();

	FontContainer &owner() const;

	const String &name() const;
	IFont *baseFont() const;

	bool name(const String &);
	bool baseFont(IFont *);

	bool internal() const;
	void setExternal();

	const Num &size() const;
	const Num &multiply() const;
	Family family() const;
	Style style() const;
	Weight weight() const;

	Num realSize() const;
	int pointSize() const;
	int wxFamily() const;		
	int wxStyle() const;
	int wxWeight() const;		
	wxFont *getWxFont();
	void clearWxFont();

	const Num &scale() const;
	void scale(const Num &theScale);
  	void operator =(const IFont& src);
	friend Connection &operator<<(Connection &c, const IFont &f);
	friend Connection &operator>>(Connection &c, IFont &f);

	void store(FILE *out) const;
	bool load(FILE *in);
	void storeText(FILE *out) const;
	bool addOption_(const ObjectOption &option);
	bool delOption_(const ObjectOption &option);

	// interface to RpDev
	void dump(FILE *out) const;
	void store_(FILE *out) const;
	bool load_(FILE *in);

	void reInherit(void);

private:
	FontContainer &owner_x;
	String  name_d;
	IFont	*baseFont_x;
	bool	internal_d;
  	Num 	size_d;
  	Num		multiply_d;
	Family 	family_d;
	Style	style_d;
	Weight	weight_d;
	Num		scale_d;

	bool	inherit();

	static const OptionTable optTable[];
	const OptionTable *optionTable() const;

	wxFont	*wxFont_d;
};

class FontContainer {
public:                              
	FontContainer();
	virtual ~FontContainer();

	Int numFonts() const;
	const String& fontName(Int i) const;
	IFont *getFont(Int i) const;
	IFont *findFont(const String &name) const;
	IFont *addFont(const String &name);
	IFont *addFont(const String &name, const String &baseName);
	bool addFont(IFont *font);
	void delFont(const String &name);
	
	void clearFonts();
	void sendFonts(Connection &c) const;
	void receiveFonts(Connection &c);

	void storeFonts(FILE *out) const;
	bool loadFonts(FILE *in);
	void storeFontsText(FILE *out) const;
	
	virtual Int defaultFontSize();

private:
	gtype(PtrArray, IFont) 	&fonts_d;
};                       

#include <ideafix/priv/ifont.icc>

#endif /* IFONT_H */
