// -----------------------------------------------------------------------
// Copyright (c) 1995 InterSoft Co.  All Rights Reserved
// THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF InterSoft Co.
// The copyright notice above does not evidence any actual or intended 
// publication of such source code.
//
// $Id: optnobj.h,v 1.3 1996/08/15 16:37:44 eduardoc Exp $
//
// Description:
// -----------------------------------------------------------------------

#ifndef OPTNOBJ_H
#define OPTNOBJ_H

#ifdef __GNUG__
#	pragma interface
#endif


#include <ifound/itypes.h>

typedef Int	schema;
typedef Int	dbtable;
typedef Int	dbfield;
typedef Int	dbindex;
typedef Int	dbcursor;

class ObjectOption;
class OptionValue;
class gtype(PtrArray, ObjectOption);

class ObjectWithOptions {
	ObjectWithOptions(const ObjectWithOptions &);
	void operator=(const ObjectWithOptions &);

public:
	ObjectWithOptions();
	virtual ~ObjectWithOptions();

	typedef Int OptionName;
	typedef Int OptionType;

	// global types
	enum {
		typeVoid,
		typeList,
		typeBool,
		typeInt,
		typeNum,
		typeString,
		typeFileName,
		typeValue,
		typeGeometry,
		typeColor,
		typeExpr,
		typeDbField,
		typeLast,
	};

	virtual bool addOption_(const ObjectOption &opt);
	virtual bool delOption_(const ObjectOption &opt);
	virtual bool checkOption_(const ObjectOption &opt);

	struct OptionTable {
		const char	*name;		// last = NULL
		OptionType	optType;
		OptionName	optName;
		bool		array;
	};

	virtual const OptionTable *optionTable() const = 0;
	virtual OptionValue *createOptionValue(OptionType t);

	Int numOptions() const;
	bool haveTextOptions() const;
	const ObjectOption &optionByNum(Int num) const;

	const ObjectOption *option(OptionName name) const;

	void storeTextOptions(FILE *) const;
	void storeOptions(FILE *) const;
	bool loadOptions(FILE *);

	String optionName(OptionName name) const;
	String optionString(const ObjectOption &opt) const;
	String debugString(const ObjectOption &opt) const;

	// help functions
	bool addOption(const ObjectOption &opt);
	
	bool addOption(OptionName name, OptionValue *value, bool internal = false);
	bool addOption(const String &name, OptionValue *value);
	bool addOption(const OptionTable &t, OptionValue *value, bool internal);

	const OptionTable &findOptionTable(OptionName name) const;
	const OptionTable &findOptionTable(const String &name) const;

	bool addBoolOption(OptionName name,
			bool value, bool internal = false);
	bool addIntOption(OptionName name,
			Int value, bool internal = false);
	bool addNumOption(OptionName name,
			const Num &value, bool internal = false);
	bool addStringOption(OptionName name,
			const String &value, bool internal = false);
	bool addFileNameOption(OptionName name,
			const String &value, bool internal = false);
	bool addExprOption(OptionName name,
			const String &value, bool internal = false);
	bool addGeometryOption(OptionName name,
			Int w, Int h, Int x, Int y, bool internal = false);
	bool addColorOption(OptionName name,
			const String &colorName, bool internal = false);
	bool addColorOption(OptionName name,
			UnChar r, UnChar g, UnChar b, bool internal = false);
	bool addDbFieldOption(OptionName name,
			const String &s, const String &t, const String &f,
			const String &defSch, bool internal = false);
	bool addDbFieldOption(OptionName name,
			dbfield fld, bool internal = false);
	bool addValueOption(OptionName name,
			const String &value, const String &descr, bool internal = false);

	bool delOption(const ObjectOption *opt);

	bool checkAllOptions();

	static String startString;
	static void incTab();
	static void decTab();
	static tabprintf(FILE *f, const char *fmt, ...);

private:
	static const OptionTable nullOption;

	gtype(PtrArray, ObjectOption) &options_d;
};

#endif


