// -----------------------------------------------------------------------
// Copyright (c) 1995 InterSoft Co.  All Rights Reserved
// THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF InterSoft Co.
// The copyright notice above does not evidence any actual or intended 
// publication of such source code.
//
// $Id: opval.h,v 1.3 1996/08/15 16:37:50 eduardoc Exp $
//
// Description:
// -----------------------------------------------------------------------

#ifndef OPVAL_H
#define OPVAL_H

#ifdef __GNUG__
#	pragma interface
#endif

#include <ifound/itypes.h>
#include <ideafix/priv/optnobj.h>

typedef Int	schema;
typedef Int	dbtable;
typedef Int	dbfield;
typedef Int	dbindex;
typedef Int	dbcursor;

class OptionTable;
class OptionValue;
class BaseOptionValueArray;
class Expr;
class wxColour;

class ObjectOption {
public:
	typedef Int Name;
	typedef Int Type;

	ObjectOption();
	ObjectOption(const ObjectOption &opt);
	ObjectOption(Name name, OptionValue *value, bool internal = false);
	~ObjectOption();

	const ObjectOption &operator=(const ObjectOption &opt);

	inline static Type type(const String &name);
	inline static Type type(Name name);
	inline static bool isArray(const String &name);
	inline static bool isArray(Name name);

	inline Name name() const;
	inline operator Name() const;
	inline bool operator==(Name) const;
	inline bool operator!=(Name) const;

	inline OptionValue *operator->() const;
	inline OptionValue *value() const;

	inline Type type() const;

	inline operator bool() const;
	inline bool operator!() const;

	inline bool internal() const;

	// members to access to values

	BaseOptionValueArray *valueArray(Type t) const;
	Int numListValues() const;
	OptionValue *listValue(Int i) const;

	bool boolVal() const;
	Int intVal() const;
	const Num &numVal() const;

	const String &stringVal() const;

	Expr &exprVal() const;

	const String &fileNameVal() const;

	void getGeometryVal(Int &x, Int &y, Int &w, Int &h) const;

	Int colorType() const;
	Int colorRed() const;
	Int colorGreen() const;
	Int colorBlue() const;
	const String &colorName() const;
	wxColour &colorVal() const;

	const String &dbFieldSchema() const;
	const String &dbFieldTable() const;
	const String &dbFieldName() const;
	dbfield dbField() const;

private:
	Name name_d;
	OptionValue *value_d;
	bool internal_d;

	static const OptionTable nullOption;

	void init(const OptionTable &t, OptionValue *value);
	void clear(OptionValue *value);

	static const OptionTable &find(const String &name);
	static const OptionTable &find(Name name);
};

//
// *** class OptionValue
//
class OptionValue : public Resource {
	OptionValue(const OptionValue &); // to prevent copying
	operator=(const OptionValue &);

public:
	typedef ObjectOption::Type Type;

	virtual ~OptionValue();

	inline Type type() const;

#define DECLARE_OPTION_VALUE(type)  	\
	static const Type Type_ = type;		\
	virtual String toString() const;	\
	virtual void store(FILE *out) const;\
	virtual bool load(FILE *out);

	DECLARE_OPTION_VALUE(ObjectWithOptions::typeVoid);

	virtual bool resolveFields();

protected:
	OptionValue(Type t);

private:
	Type type_d;
};

#define OPTION(value, Class) \
static_cast<Class*>(value->type()==Class::Type_? value: 0)

class gtype(PtrArray, OptionValue);

//
// OptionValueArray
//
class BaseOptionValueArray : public OptionValue {
	typedef OptionValue inherited;

public:
	BaseOptionValueArray(Type t);
	~BaseOptionValueArray();

	DECLARE_OPTION_VALUE(ObjectWithOptions::typeList);

	String toStringOpen(const String &delim) const;
	bool load(FILE *out, ObjectWithOptions *owner);

	bool resolveFields();

	Type subType() const;

	Int numValues() const;
	OptionValue *value(Int num) const;
	void delValue(Int num);
	void addValue(OptionValue *value);
	void insValue(Int pos,OptionValue *value);

protected:
	void addValue_(OptionValue *value);
	void insValue_(Int pos,OptionValue *value);
	void merge_(BaseOptionValueArray *value);

private:
	Type subType_d;
	gtype(PtrArray, OptionValue) &list_d;
};

#define OptionValueArray(CONTENT)	name2(CONTENT, OptionValueArray)

#define	OptionValueArraydeclare(CONTENT)						\
class OptionValueArray(CONTENT) : public BaseOptionValueArray {	\
	typedef BaseOptionValueArray inherited;						\
public:															\
	explicit OptionValueArray(CONTENT)()						\
	:	inherited(CONTENT::Type_) {}							\
	CONTENT *value(Int n) const									\
	{ return static_cast<CONTENT*>(inherited::value(n)); }		\
	void addValue(CONTENT *v)									\
	{ addValue_(v); }											\
	void insValue(Int pos,CONTENT *v)							\
	{ insValue_(pos,v); }											\
	void merge(OptionValueArray(CONTENT) *v)					\
	{ merge_(v); }												\
};

#define OptionValueArrayimplement(CONTENT)

#define OPTION_ARRAY(option, Class)	\
static_cast<OptionValueArray(Class)*>(option.valueArray(Class::Type_))

//
// *** class BoolVal
//
class BoolVal : public OptionValue {
	typedef OptionValue inherited;

public:
	BoolVal(bool b = false);

	DECLARE_OPTION_VALUE(ObjectWithOptions::typeBool);

	bool value() const;

private:
	bool val_d;
};

//
// *** class IntVal
//
class IntVal : public OptionValue {
	typedef OptionValue inherited;

public:
	IntVal(Int i = 0);

	DECLARE_OPTION_VALUE(ObjectWithOptions::typeInt);

	Int value() const;

private:
	Int val_d;
};

//
// *** class NumVal
//
class NumVal : public OptionValue {
	typedef OptionValue inherited;

public:
	NumVal();
	NumVal(Num n);

	DECLARE_OPTION_VALUE(ObjectWithOptions::typeNum);

	const Num &value() const;

private:
	Num val_d;
};

//
// *** class StringVal
//
class StringVal : public OptionValue {
	typedef OptionValue inherited;

public:
	StringVal(const String &s = NULL_STRING);

	DECLARE_OPTION_VALUE(ObjectWithOptions::typeString);

	const String &value() const;

protected:
	StringVal(Type t, const String &s = NULL_STRING);

	void setValue(const String &s);

private:
	String val_d;
};

//
// *** class FileNameVal
//
class FileNameVal : public StringVal {
	typedef StringVal inherited;

public:
	const Type Type_ = ObjectWithOptions::typeFileName;
	
	FileNameVal(const String &s = NULL_STRING);
};

class ValueVal : public StringVal {
	typedef StringVal inherited;

public:
	ValueVal(const String &s = NULL_STRING, const String &d = NULL_STRING);
	
	DECLARE_OPTION_VALUE(ObjectWithOptions::typeValue);

	const String &descr() const;

private:
	String descr_d;
};

//
// *** class GeometryVal
//
class GeometryVal : public OptionValue {
	typedef OptionValue inherited;

public:
	GeometryVal();
	GeometryVal(Int w, Int h, Int x = ERR, Int y = ERR);

	DECLARE_OPTION_VALUE(ObjectWithOptions::typeGeometry);

	void getValue(Int &w, Int &h, Int &x, Int &y) const;

private:
	Int w_d;
	Int h_d;
	Int x_d;
	Int y_d;
};

//
// *** class ColorVal
//
class ColorVal : public OptionValue {
	typedef OptionValue inherited;

public:
	ColorVal();
	ColorVal(UnChar r, UnChar g, UnChar b);
	ColorVal(const String &col);
	~ColorVal();
	
	DECLARE_OPTION_VALUE(ObjectWithOptions::typeColor);

	const String &colorName() const;
	Int red() const;
	Int green() const;
	Int blue() const;

	enum ColorType {
		colorIsVoid,
		colorIsNum,
		colorIsStr
	};
	ColorType colorType() const;

	bool resolveColor();

	wxColour &value();

private:
	ColorType colorType_d;

	String colorName_d;
	Int r_d, g_d, b_d;

	// resolved values
	wxColour *val_d;
};

//
// *** class ExprVal
//
class ExprVal : public OptionValue {
	typedef OptionValue inherited;

public:
	ExprVal();
	ExprVal(const String &expr);

	DECLARE_OPTION_VALUE(ObjectWithOptions::typeExpr);

	Expr &expr() const;
	const String &exprString() const;
	
	bool resolveFields();

private:
	Expr &expr_d;
};

//
// *** class DbFieldVal
//
class DbFieldVal : public OptionValue {
	typedef OptionValue inherited;

public:
	DbFieldVal(const String &defaultSchema = NULL_STRING);
	DbFieldVal(const String &s, const String &t, const String &f,
			const String &defaultSchema = NULL_STRING);
	DbFieldVal(dbfield fld);

	DECLARE_OPTION_VALUE(ObjectWithOptions::typeDbField);

	const String &schemaName() const;
	const String &tableName() const;
	const String &fieldName() const;
	const String &defaultSchemaName() const;

	dbfield dbField() const;

private:
	String s_d, t_d, f_d, def_s_d;

	// resolved values
	dbfield fld_d;
	
	schema findSchema() const;
};

#include <ideafix/priv/opval.icc>

#endif


