#ifndef RPDACTN_H
#define RPDACTN_H

#ifdef __GNUG__                                                                 
#pragma interface
#endif

#include <ideafix/priv/rpdev.h>

class RpDevAction
{
protected:	
	// The device this action belongs to
	RpDevice *repDev_d;
	
	// This action's id
	RpDevice::Event id_d;
	
	// This action's internal name (for furure use)
	String name_d;
	// Action's title (for menu etc.)
	String title_d;
	// This action's short description
	String descr_d;
	// Another thing that could be here is the action's icon
	String bitmapName_d, pushBitmapName_d;
protected:	
	RpDevAction(RpDevice *theDevice, 
			RpDevice::Event id,
			const String &name,
			const String &title,
			const String &descr, 
			const String &bitmap,
			const String &pushedBitmap = NULL_STRING);

public:
	virtual ~RpDevAction();
	
	// return action's id
	inline RpDevice::Event id(void) const;

	// action's name
	inline const String &name() const;

	// action's title
	inline const String &title() const;
	inline void title(const String &newTitle);
	
	// action's description
	inline const String &descr() const;
	inline void descr(const String &newDescr);
	
	// bitmap's name
	inline const String &bitmapName() const;
	inline void bitmapName(const String &theName);
	inline const String &pushBitmapName() const;
	inline void pushBitmapName(const String &theName);

	// try to execute this action
	virtual bool process() = 0;

	virtual bool ignore() = 0;

};


class RpDevActionReadNext : public RpDevAction
{
public:
	RpDevActionReadNext(RpDevice *rpd);
	
	virtual bool process();
	virtual bool ignore();
	
};


class RpDevActionReadPrev : public RpDevAction
{
public:
	RpDevActionReadPrev(RpDevice *rpd);
	
	virtual bool process();
	virtual bool ignore();
	
};

class RpDevActionReadFirst : public RpDevAction
{
public:
	RpDevActionReadFirst(RpDevice *rpd);
	
	bool process();
	virtual bool ignore();
	
};

class RpDevActionReadLast : public RpDevAction
{
public:
	RpDevActionReadLast(RpDevice *rpd);
	
	virtual bool process();
	virtual bool ignore();
	
};


class RpDevActionClose : public RpDevAction
{
public:
        RpDevActionClose(RpDevice *rpd);
                
        virtual bool process();
		virtual bool ignore();
                        
};
                        
#include <ideafix/priv/rpdactn.icc>

#endif

