#ifndef RPDEV_H
#define RPDEV_H

#pragma interface

#include <ifound/mainobj.h>
#include <cracker/reqconn.h>
#include <ideafix/priv/ifont.h>
#include <ideafix/priv/rptypes.h>


class RpRep;
class RpDevObject;
class RpDevLabelObj;
class RpDevLineObj;
class RpDevRectObj;
class RpDevArcObj;
class RpDevEllipseObj;
class RpDevBitmapObj;
class RpDevAction;
class RepCanvas;
class RepFrame;

class wxDC;
class wxFont;
class wxColour;

class gtype(SimpleArray, Int);
class gtype(PtrArray, String);
class gtype(PtrArray, IBitmap);
class gtype(PtrArray, RpDevAction);

#define LAST_PAGE	INT_MAX


wxColour *findColor(const String &name);

class RpDevice :	public Resource, public FontContainer 
{                                
public:
	enum Event {
		noAction = -1,
		readFirst = 0,
		readLast,
		readPrev,
		readNext,
		close,
		lastEvent,
		print,
		readPage
	};		

private:
	RpRep *rpRep_d;

protected:
	// Start of dictionary
	String name_d;                      // name of report
	Int repWidth_d;						// width of report
	Int repHeight_d;            		// height of report
	Int leftMarg_d;						// left marging
	Int rightMarg_d;					// right marging
	Int topMarg_d;						// top marging
	Int botMarg_d;						// bottom marging
  	String foreground_d;                // default foreground
  	String background_d;    			// default background
	String	font_d;						// default font name
	RpUnit	unit_d;						// vertical unit
	RpPaperSize paperSize_d;			// paper size
	bool	noFFeed_d;		 			// true if no use ffeed '\f'
	// End of dictionary

private:
	gtype(SimpleArray, Int) &pageIndex_d;  	// page indexes in metafile
	Int currentDspPage_d;				// current displayed page
	Int currPageOffset_d;
	
	bool isDone_d;         				// all pages were loaded
	bool isAborted_d;                   // aborted by uses

	// For use in loadPages
	Int  pageToShow_d;                  // page to show
	Int  lastPageToLoad_d;              // last page to load
	bool noPrint_d;                     // no print pages
	bool storeData_d;

	gtype(PtrArray, RpDevAction) &actns_d;

    void initValues();
    virtual bool build() = 0;
 	void store(FILE *out) const;
	bool load(FILE *in); 
protected:
	FILE *tempFile_d;   				// metafile
	bool needToPrint_d;
	void createTmpFile();
	RpDevObject *loadObject(FILE *in);
public:
	RpDevice();
	RpDevice(RpRep *theRpRep);
	RpDevice(const String& fname);
	void rpToDevice(RpRep *theRpRep); // serjov
	virtual ~RpDevice();

	inline RpRep *rpRep() const; 
	inline Int repWidth() const;
	inline Int repHeight() const;
	inline Int leftMarg() const;
	inline Int rightMarg() const;
	inline Int topMarg() const;
	inline Int botMarg() const;
	inline float frepWidth() const;
	inline float frepHeight() const;
	inline const String &name() const;
	inline const String &foreground() const;
	inline const String &background() const;
	inline const String &font() const;
	inline RpUnit unit() const;
	inline RpPaperSize paperSize() const;
	inline bool noFormFeed() const;
    
	Int numPages() const;
    inline Int currentDspPage() const;
    void showPage(Int page);
	virtual void loadPages(Int lastPage, Int thePageToShow);
    Int pageIndex(Int page) const;
    bool pageIsLoaded(Int page);
        
    inline bool isDone() const;
	inline bool isAborted() const;
	inline void isAborted(bool v);
	inline bool noPrint() const;
	inline bool inLoading() const;
	inline bool noRpRep() const;        
	
	RpDevObject *createDevObject(Int obType);

	virtual void operator << (RpDevObject &obj);
	virtual void condition(Report::When when, Report::WhenType type, Int pageNum); 
	virtual void startReport();   		// methods
	virtual void endReport();       	//    were
	virtual void startPage(Int page);   //       called on
	virtual void endPage(Int page);     //          before/after page/report 

	RpDevAction *action(Event id) const;
	RpDevAction *action(Int n) const;
	Int numActions() const;
	void addAction(RpDevAction *actn);
	bool executeAction(Event theId);	    
	bool ignoreAction(Event theId) const;

	virtual Num toExternVert(Int val) const;
	virtual Num toExternHoriz(Int val) const;
    virtual void showStatus();
	virtual void clear();
	virtual void exit();

	// drawing methods
	virtual void drawString(RpDevLabelObj *obj);
	virtual void drawLine(RpDevLineObj *obj);
	virtual void drawRect(RpDevRectObj *obj);
	virtual void drawArc(RpDevArcObj *obj);
	virtual void drawEllipse(RpDevEllipseObj *obj);
	virtual void drawBitmap(RpDevBitmapObj *obj);

};  


class WxRpDevice :	public RpDevice
{                                               
	typedef RpDevice inherited;

	wxFont *wxfont_d;				// default font
	wxColour *wxforeground_d;       // default foreground
	wxColour *wxbackground_d;       // default background
	wxColour *currWxfg_d;           // current foreground for drawing
	wxColour *currWxbg_d;           // current background for drawing
	wxBitmap *defaultBitmap_d;      // default bitmap
	gtype(PtrArray, IBitmap) &bitmaps_d;	// bitmaps in report	   
	
	bool isDefFont_d;                // Is default font setted to canvas

	virtual bool build() = 0;
protected:
	wxDC* dc_d;
	void initValues();
	void checkFonts();
	
	inline gtype(PtrArray, IBitmap) *bitmaps() const; // serjov
	inline bool isDefFont() const; //serjov
	inline void isDefFont(bool); //serjov
	inline wxColour *currWxfg() const; //serjov
	inline wxColour *currWxbg() const; //serjov
public:
	WxRpDevice();
	WxRpDevice(RpRep *theRpRep);
	WxRpDevice(const String& fname);
	void rpToWxDevice(RpRep *theRpRep); // serjov
	~WxRpDevice();
    
	static Int fontWidth;
	static Int fontHeight;

	inline wxFont *wxfont() const;
	inline wxColour *wxforeground() const;
	inline wxColour *wxbackground() const;
	wxBitmap *defaultBitmap();
	void getCurrColors(RpDevObject *obj, bool fore, bool back);
	void delCurrColors();

	inline void repDC(wxDC *);
	inline wxDC *repDC() const;

	virtual void clear();

	virtual void drawString(RpDevLabelObj *obj);
	virtual void drawLine(RpDevLineObj *obj);
	virtual void drawRect(RpDevRectObj *obj);
	virtual void drawArc(RpDevArcObj *obj);
	virtual void drawEllipse(RpDevEllipseObj *obj);
	virtual void drawBitmap(RpDevBitmapObj *obj);

	virtual Int defaultFontSize();
};

class WxFramedRpDevice :	public WxRpDevice, public FramedObject
{                                               
	typedef WxRpDevice inherited;
	
	RepFrame *repFrame_d;
	RepCanvas *repCanvas_d;

protected:
	virtual bool build();
	void initValues();
public:
	WxFramedRpDevice();
	WxFramedRpDevice(RpRep *theRpRep);
	WxFramedRpDevice(const String& fname);
	~WxFramedRpDevice();
    
	void startMessageLoop();
	void stopMessageLoop(Int ret);
  	inline RepFrame *repFrame() const;
	void show();

	void makeFrame( void );

	virtual void loadPages(Int lastPage, Int thePageToShow);

	virtual void endPage(Int page);
	virtual void endReport();

	virtual Num toExternVert(Int val) const;
	virtual Num toExternHoriz(Int val) const;
    virtual void showStatus();
	virtual void exit();

	// From FramedObject
	virtual bool canUseWxWindows() const;
	virtual bool wxWindowsCreate();
	virtual void wxWindowsDestroy();

	// printing postscript
	virtual void printReport();
	void doPrint();
};


class CrkCliRpDevice :	public RpDevice, public FramedObject
{                                               
	typedef RpDevice inherited;
	RequestConnection		&conn_d;
	
	virtual bool build();
public:
	CrkCliRpDevice(RpRep *theRpRep, RequestConnection &conn);
	~CrkCliRpDevice();
    
	void startMessageLoop();
	void stopMessageLoop(Int ret);
	inline RequestConnection &conn() const;

	virtual void exit();

	virtual void startReport();
	virtual void endReport();
	virtual void startPage(Int page);
	virtual void endPage(Int page);
	inline void draw(RpDevObject *obj);

	// drawing methods
	virtual void drawString(RpDevLabelObj *obj);
	virtual void drawLine(RpDevLineObj *obj);
	virtual void drawRect(RpDevRectObj *obj);
	virtual void drawArc(RpDevArcObj *obj);
	virtual void drawEllipse(RpDevEllipseObj *obj);
	virtual void drawBitmap(RpDevBitmapObj *obj);

	// From FramedObject
	virtual bool canUseCracker() const;
	virtual bool crackerCreate(RequestConnection &conn);
	virtual void crackerDestroy(RequestConnection &conn);
};
  
  
class WxPSRpDevice : public WxRpDevice
{
	typedef WxRpDevice inherited;

	virtual bool build();
public:
	WxPSRpDevice(RpRep *theRpRep);
	WxPSRpDevice(const String& fname);
	~WxPSRpDevice();

	virtual Num toExternVert(Int val) const;
	virtual Num toExternHoriz(Int val) const;

	virtual void startReport();
	virtual void endReport();
	virtual void startPage(Int page);
	virtual void endPage(Int page);

};

class CharRpDevice :	public RpDevice
{                                               
	typedef RpDevice inherited;
	Int	nRows_d;
	Int nCols_d;
	char **rows_d;
	FILE *outFile_d;

protected:
	void initValues();
public:
	CharRpDevice(RpRep *theRpRep);
	CharRpDevice(const String& fname);
	~CharRpDevice();   
	
	inline FILE *outFile() const;
	inline void outFile(FILE *file);
    
	virtual void startPage(Int page);
	virtual void endPage(Int page);

	virtual void clear();

	virtual Num toExternVert(Int val) const;
	virtual Num toExternHoriz(Int val) const;

	void putChar(char c, Int x, Int y);
	
	virtual void drawString(RpDevLabelObj *obj);
	virtual void drawLine(RpDevLineObj *obj);
	virtual void drawRect(RpDevRectObj *obj);

};

class FileRpDevice :	public CharRpDevice
{                                               
	typedef CharRpDevice inherited;

	String fileName_d;
	virtual bool build();
public:
	FileRpDevice(RpRep *theRpRep, const String& outFile);
	FileRpDevice(const String& fname, const String& outFile);
	~FileRpDevice();
};

class PipeRpDevice :	public CharRpDevice
{                                               
	typedef CharRpDevice inherited;
                                    
	String command_d;
	virtual bool build();
public:
	PipeRpDevice(RpRep *theRpRep, const String& command);
	PipeRpDevice(const String& fname, const String& command);
	~PipeRpDevice();
};

class PrinterRpDevice :	public CharRpDevice
{                                               
	typedef CharRpDevice inherited;

	Int nCopies_d;
  	String options_d;
	virtual bool build();
public:
	PrinterRpDevice(RpRep *theRpRep, Int nCopies, const String& options);
	PrinterRpDevice(const String& fname, Int nCopies, const String& options);
	~PrinterRpDevice();

};

#include <ideafix/priv/rpdev.icc>

#endif /* RPDEV_H */

