/*
 *
 * $Id: rpdobj.h,v 1.2 1996/07/15 19:54:09 eugen Exp $
 *
 */
#ifndef RPDOBJ_H
#define RPDOBJ_H

#pragma interface

#include <ideafix/priv/rpdev.h>

class RpZoneObj;                 
class RpZoneDrawedObj;
class RpZoneTextObj;
class RpZoneLineObj;
class RpZoneRectObj;
class RpZoneArcObj;
class RpZoneEllipseObj;
class RpZoneBitmapObj;

class RpDevObject;
class RpDevLabelObj;
class RpDevLineObj;
class RpDevRectObj;
class RpDevArcObj;
class RpDevEllipseObj;
class RpDevBitmapObj;

class Connection;

class RpDevObject :	public Resource 
{
public:
	enum DevObjectType {
	 	rd_none,
	 	rd_label,
	 	rd_line,
	 	rd_rect,
	 	rd_ellipse,
	 	rd_arc,
	 	rd_bitmap,
	};	 	 
private:
	RpDevice	*dev_d;
	DevObjectType	objType_d;
	Int geom_X_d, geom_Y_d, geom_W_d, geom_H_d;	
  	String foreground_d;    
  	String background_d;    
protected:
	void objType(DevObjectType theType);
public:
	RpDevObject();
	RpDevObject(RpZoneObj *zobj);
	virtual ~RpDevObject();
	
	void initValues();
 	virtual void store(FILE *out) const;
	virtual bool load(FILE *in);
	virtual void draw() = 0;

	void rpDevice(RpDevice *theDevice);
	inline RpDevice *rpDevice() const; 

  	void geometry(Int x, Int y, Int w = ERR, Int h = ERR);
	inline Int geom_X() const;
	inline Int geom_Y() const;
	inline Int geom_W() const;
	inline Int geom_H() const;
	inline float fgeom_X() const;
	inline float fgeom_Y() const;
	inline float fgeom_W() const;
	inline float fgeom_H() const;
	inline Int igeom_X() const;
	inline Int igeom_Y() const;
	inline Int igeom_W() const;
	inline Int igeom_H() const;
	inline const String &foreground() const;
	inline const String &background() const;

	inline DevObjectType objType() const;

	virtual void send(Connection &c) const;
	virtual void receive(Connection &c);
};                                              


class RpDevLabelObj	:	public RpDevObject 
{
	typedef RpDevObject inherited;	

	String	value_d;
	Alignment alignment_d;
	String	font_d;
	
public:
	RpDevLabelObj();
	RpDevLabelObj(RpZoneTextObj *zobj);
	~RpDevLabelObj();
	
	void initValues();
 	virtual void store(FILE *out) const;
	virtual bool load(FILE *in); 
	virtual void draw();

  	inline void value(String& theValue);
	inline const String &value() const;
	inline Alignment alignment() const;
	inline const String &font() const;

	virtual void send(Connection &c) const;
	virtual void receive(Connection &c);
};

class RpDevDrawedObj :	public RpDevObject
{
	typedef RpDevObject inherited;	

	Int	thickness_d;
	DrawStyle style_d;
	FillStyle fillStyle_d;

public:
	RpDevDrawedObj();
	RpDevDrawedObj(RpZoneDrawedObj *zobj);
	~RpDevDrawedObj();

	void initValues();
 	virtual void store(FILE *out) const;
	virtual bool load(FILE *in); 
	
	inline void thickness(Int n);
	inline Int thickness() const;

	inline void style(DrawStyle n);
	inline DrawStyle style() const;
	inline void fillStyle(FillStyle n);
	inline FillStyle fillStyle() const;

	virtual void send(Connection &c) const;
	virtual void receive(Connection &c);
};
  
class RpDevRectObj : public RpDevDrawedObj
{
	typedef RpDevDrawedObj inherited;

public:
	RpDevRectObj();
	RpDevRectObj(RpZoneRectObj *zobj);
	~RpDevRectObj();

	void initValues();
	void draw();

};

class RpDevLineObj :	public RpDevDrawedObj
{
	typedef RpDevDrawedObj inherited;	
	Orientation orientation_d;

public:
	RpDevLineObj();
	RpDevLineObj(RpZoneLineObj *zobj);
	~RpDevLineObj();

	void initValues();
 	virtual void store(FILE *out) const;
	virtual bool load(FILE *in); 
	void draw();

	inline void orientation(Orientation v);
	inline Orientation orientation() const;

	virtual void send(Connection &c) const;
	virtual void receive(Connection &c);
};

class RpDevEllipseObj :	public RpDevDrawedObj
{
	typedef RpDevDrawedObj inherited;	

public:
	RpDevEllipseObj();
	RpDevEllipseObj(RpZoneEllipseObj *zobj);
	~RpDevEllipseObj();
	
	void initValues();
	void draw();
};

class RpDevArcObj :	public RpDevDrawedObj
{
	typedef RpDevDrawedObj inherited;	

	Int startAngle_d,endAngle_d;
public:
	RpDevArcObj();
	RpDevArcObj(RpZoneArcObj *zobj);
	~RpDevArcObj();
	
	void initValues();
 	virtual void store(FILE *out) const;
	virtual bool load(FILE *in); 
	virtual void draw();
	
	inline void startAngle(Int);
	inline Int startAngle() const;

	inline void endAngle(Int);
	inline Int endAngle() const;

	virtual void send(Connection &c) const;
	virtual void receive(Connection &c);
};

class RpDevBitmapObj :	public RpDevObject 
{
	typedef RpDevObject inherited;	

	String	fileName_d;

public:
	RpDevBitmapObj();
	RpDevBitmapObj(RpZoneBitmapObj *zobj);
	~RpDevBitmapObj();
	
	void initValues();
 	virtual void store(FILE *out) const;
	virtual bool load(FILE *in); 
	virtual void draw();

	inline void fileName(String&);
	inline String& fileName();

	virtual void send(Connection &c) const;
	virtual void receive(Connection &c);
};

#include <ideafix/priv/rpdobj.icc>

#endif /* RPDOBJ_H */
