/********************************************************************
* Copyright (c) 1994 InterSoft Co.  All Rights Reserved
* THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF InterSoft Co.
* The copyright notice above does not evidence any
* actual or intended publication of such source code.
*
* $Id: rpobj.h,v 1.2 1996/07/15 19:54:11 eugen Exp $
*
* DESCRIPTION
*********************************************************************/

#ifndef RPOBJ_H
#define RPOBJ_H

#ifdef __GNUG__
#	pragma interface
#endif

#include <ideafix.h>
#include <ideafix/priv/optnobj.h>

class RpRep;
class Expr;
class RpExpr;         
class RepZoneRep;
class RpOpt;

#define RP_DECLARE_CLASS									\
	virtual String className() const;						\
	virtual bool addOption_(const ObjectOption &opt);    	\
	virtual bool delOption_(const ObjectOption &opt);		\
	virtual bool checkOption_(const ObjectOption &opt);		\
	virtual String nrpItemName() const;

#define RP_IMPLEMENT_CLASS_NAME(Class)						\
String Class::className() const { return #Class; }                                  

#define RP_NO_OPTIONS(Class)								\
bool Class::addOption_(const ObjectOption &option)			\
{ return inherited::addOption_(option); }					\
bool Class::checkOption_(const ObjectOption &option)		\
{ return inherited::checkOption_(option); }					\
bool Class::delOption_(const ObjectOption &option)			\
{ return inherited::delOption_(option); }

#define RP_NO_CHECK_OPTION(Class)							\
bool Class::checkOption_(const ObjectOption &option)    	\
{ return inherited::checkOption_(option); }

                                                            
class RpObject : public Resource, public ObjectWithOptions {
	RpRep 		*rpRep_d;	 		// Pointer to the report
	String		name_d;				// Object name

public:
	RpObject(RpRep *rpRep);
	virtual ~RpObject();

	RP_DECLARE_CLASS
	
	inline RpRep *rpRep() const;
	inline void name(const String &newName);
	inline String nt() const;
	inline const String &name() const;

	bool invalidOption(const ObjectOption &opt) const;

	enum {
		typeRpExpr = ObjectWithOptions::typeLast
	};                                               
	
	typedef RpOpt Opt;
	static const OptionTable optTable[];
	const OptionTable *optionTable() const;
	OptionValue *createOptionValue(OptionType type);

	bool addRpExprOption(OptionName name, 
			const String &expr, bool interenal = false);
	bool addRpExprOption(OptionName name, 
		   	const Expr &expr, bool interenal = false);
	bool addRpExprOption(OptionName name, RepZoneRep *zrep,
		   	const Expr &expr, bool interenal = false);

	String nrpDataTypeName(Int type) const;

	bool checkAllOptions();
	
	virtual bool hasChildren() const;                       
	virtual bool stopBuilding();
	bool building() const;
	bool loading() const;

	void store(FILE *out) const;
	bool load(FILE *in);
	void dump(FILE *out) const;
	void storeText(FILE *out) const;
	void storeNrp(FILE *out) const;

	virtual void storeHeader(FILE *out) const;
	virtual bool loadHeader(FILE *in);
	virtual void storeChildren(FILE *out) const;
	virtual bool loadChildren(FILE *in);

	virtual String nrpHeader() const;
	virtual void storeNrpChildren(FILE *out) const;
	
};

#include <ideafix/priv/rpobj.icc>

#endif

