/********************************************************************
* Copyright (c) 1994 InterSoft Co.  All Rights Reserved
* THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF InterSoft Co.
* The copyright notice above does not evidence any
* actual or intended publication of such source code.
*
* $Id: rprep.h,v 1.2 1996/07/15 19:54:14 eugen Exp $
*
* DESCRIPTION:
*********************************************************************/

#ifndef RPREP_H
#define RPREP_H

#ifdef __GNUC__
#	pragma interface
#endif

// Default values for page dimensions
#define DEFAULT_LEFTMARG	0
#define DEFAULT_RIGHTMARG	0
#define DEFAULT_BOTMARG		(2*U_V_CHARACTER)
#define DEFAULT_TOPMARG		(2*U_V_CHARACTER)
#define DEFAULT_FLENGTH	  	(66*U_V_CHARACTER)
#define DEFAULT_WIDTH		(80*U_H_CHARACTER)

/* Check digit codifiers */
#define RP_NO_CDIGIT	0
#define RP_CDIGIT		1
#define RP_CDIGIT_DASH	2
#define RP_CDIGIT_SLASH	3
#define RP_CDIGIT_UNDEF	4

// Macros & defines
#define PRT_CURR_VAL	1
#define PRT_PREV_VAL	2

#define ENV_VAR_CHAR '$'


#include <ifound/anyvalue.h>
#include <ideafix/priv/rpobj.h>
#include <ideafix/priv/ifont.h>
#include <ideafix/priv/rptypes.h>

class AnyValue;
class MsgTable;
class FmOption;

class RepZoneRep;
class RpFldRep;
class RpAccum;
class RpZoneObj;
class RpZoneExprObj;
class RpDevice;

class gtype(PtrArray, String);
class gtype(PtrArray, RpFldRep);
class gtype(PtrArray, RpAccum);
class gtype(PtrArray, RpExpr);
class gtype(PtrArray, RepZoneRep);
class gtype(PtrArray, RpZoneObj);


class RpAccum : public Resource {

	typedef RpObject inherited; 

	friend class RpRep;
	friend class RepZoneRep;

	// Start of dictionary
	Int			type_d;			// Accumulator type

	/*
		accumulator argument, using the following format

		byte1 byte2 byte3 byte4
		\_________/ \_________/
		  zoneNo       exprNo
	*/
	Int argNo_d;

	bool		noReset_d; 		// flag to not reset Acc.
	// End of dictionary

	RepZoneRep	*zone_d;	 	// Pointer to zone representation
	Int			accNo_d;	 	// accumulator descriptor
	Int			count_d;	 	// Number of times printed
	Num			acc_d;			// Function accumulator

	// Resolved argument
	RpZoneExprObj *arg_d;		// Accumulator's expression argument

public:

	enum {
		sum,
		avg,
		count,
		min,
		max
	};

	RpAccum(RepZoneRep *zrep, Int type = ERR, bool noReset = false,
			RpZoneExprObj *zexpr = NULL);
	~RpAccum();

	inline RpRep *rpRep() const;
	inline Int accNo() const;
	inline Int type() const;

	Num value();
	void accumulate(const Num &val);
};


class RpFldRep : public RpObject{

	typedef RpObject inherited; 

	friend class RpRep;

	// Start of dictionary
	Int			type_d;			// Type (TY_STRING, TY_DATE, etc.)
	Int			len_d;			// Length
	Int			nDec_d;			// Number of decimal places
	Int			vDbfield_d;		// Data base field descrip.
								// ERR ==> No DB field
	// toDo: nullZ_d is not used... we leave it for the moment to
	// avoid incompatibilities with previous versions
	bool		nullZ_d;		// flag for null zero

	RpExpr		*maske_d;		// mask
	// End of dictionary

	AnyValue	value_d;		// Actual value
	AnyValue	prevVal_d;		// Previous value
	bool		changed_d;		// If field has changed
	Int			fldNo_d;  		// field descriptor

public:

	RpFldRep(RpRep *rprep, const String &nm = NULL_STRING);
	~RpFldRep();

	RP_DECLARE_CLASS

	void setNull();
//	inline void length(Int l);
	inline Int length() const;
//	inline void nDec(Int nd);
	inline Int nDec() const;
//	inline void type(Int t);
	inline Int type() const;
	inline void dbField(Int dbf);
	inline Int dbField() const;
//	inline void fldNo(Int fld);
	inline Int fldNo() const;
	inline RpExpr *maskExpr() const;
//	inline void maskExpr(RpExpr *mask);
	inline bool nullZ() const;
//	inline void nullZ(bool v);
	
	void set(const char *);
	const AnyValue &value();
	inline void value(const AnyValue &v);
	const AnyValue &prevVal();
	inline void prevVal(const AnyValue &v);

	virtual bool stopBuilding();
	bool checkDbType() const;

	virtual String nrpHeader() const;
};

class RepZoneRep : public RpObject {

	typedef RpObject inherited; 

	friend class RpRep;

	// Start of dictionary
	Int			lines_d;				// Lines ocupied by this zone
	Int			needLines_d;			// Lines needed in page to print z.
	Int			atLine_d;				// Line to write the zone
	bool		eject_d;  				// Eject a page when print zone
	bool		before_d;	 			// True if must be ejected before
	bool		noPrint_d;				// Don't print this zone
	bool		befAftCond_d;			// before() or after() used in 'if' expression
	Report::When when_d;				// always, after or before
	Report::WhenType type_d;			// page, field, report
	RpExpr *ifexpr_d;					// if expression
	gtype(PtrArray, RpFldRep) &flds_d;  // used for after or before field
	gtype(PtrArray, RpAccum) &accs_d;	// Pointers to accumulators
	gtype(PtrArray, RpZoneObj) &zobjs_d;// Array with report expressions
	// End of dictionary

	Int			zoneNo_d;				// zone descriptor
	RepZoneRep	*groupZone_d;			// zone grouped with;

public:

	RepZoneRep(RpRep *rprep, const String &nm = NULL_STRING);
	~RepZoneRep();

	RP_DECLARE_CLASS

	void print(int valmode);
	void clear();
	Int nAccumulators() const;
	RpAccum *accumulator(Int n);
	Int accumNo(Int atype, bool noReset, const String &argName);
	Int addAccumulator(Int atype, bool noReset, const String &argName);
	void accumulate(RpZoneExprObj *obj, const Num &a);
	void accumulate(RpFldRep *fr, const Num &a);
	void resetAccumulators();
	RpZoneExprObj *findExpr(Int n) const;
	RpZoneExprObj *findExpr(const String &nm) const;

	inline bool isAlways() const;
	inline void nLines(Int n);
	inline Int nLines() const;
	inline Int zoneNo() const;
//	inline void zoneNo(Int n);
	Int nExprs() const;				// Should count number of expressions 
									// in zobjs (skipping labels)
	inline void needLines(Int n);
	inline Int needLines() const;
	void addCondField(RpFldRep *fr);
	Int add(RpZoneObj *obj);
//	inline void noPrint(bool f);
	inline bool noPrint() const;
//	inline void eject(bool f);
	inline bool eject() const;
//	inline void before(bool f);
	inline bool before() const;
	inline Report::When condWhen() const;
//	inline void condWhen(Report::When w);
	inline Report::WhenType condType() const;
//	inline void condType(Report::WhenType wt);
//	inline RpExpr &ifExpr() const;
//	inline void atLine(Int n);
	inline Int atLine() const;
	Int nObjs() const;
	RpZoneObj *object(Int i) const;
	inline void befAftCond();
	inline bool automatic() const;

	bool checkAllOptions();
	virtual bool stopBuilding();
	virtual bool hasChildren() const;

	virtual void storeHeader(FILE *out) const;
	virtual bool loadHeader(FILE *in);
	virtual void storeChildren(FILE *out) const;
	virtual bool loadChildren(FILE *in);

	virtual String nrpHeader() const;
	virtual void storeNrpChildren(FILE *out) const;
};                                                 

class RpRep : public RpObject, public FontContainer {

	typedef RpObject inherited; 

protected:
// 	start of dictionary

	Report::OutputType	inpFrom_d;	  		// PRINTER, FILE or PIPE
	Report::OutputType	outTo_d;	  		// PRINTER, FILE or PIPE
	String			inp_d;					// Input source
	String			output_d;				// Output destination
	gtype(PtrArray, String) &schemas_d;		// Related schemas
	gtype(PtrArray, RepZoneRep) &zones_d;	// Pointer to zones array
	gtype(PtrArray, RpFldRep) &flds_d;		// Pointer to fields
	gtype(PtrArray, RpFldRep) &forder_d;	// Fields order for input read

	Int				flength_d;				// Form length 
	Int				topmarg_d;				// Top margin 
	Int				botmarg_d;				// Bottom margin
	Int				leftmarg_d;				// Left margin
	Int				rightmarg_d;			// Right margin
	Int				width_d;				// Width
	
	Int				maxLinAft_d;			// Maximun lines for after pages
	bool			noFFeed_d;				// true if no use ffeed '\f'
	
  	String 			foreground_d;			// default foreground
  	String 			background_d;			// default background
	String			font_d;					// default font
	RpUnit			unit_d;					// unit of report
	RpPaperSize 	paperSize_d;			// paper size
// end of dictionary

	RpDevice		*rpDev_d;               // Printing device 

	Report::When	when_d;
	Report::WhenType whenType_d;
	bool			doEject_d;				// Flag a pending eject page
	Int				currY_d;  				// Line actually in print
	Int				currPage_d;				// Page  "    "   "   "
	Int				lstPZone_d;				// Last printed zone
	bool			repBody_d;
	bool			started_d;
	bool			eabort_d;				// If RP_EABORT used on open
	bool			befReport_d; 			// Before Report Flag
	bool			exitByUser_d;			// In WxFramedRpDevice: if user exec "Exit"
	bool			status_d;
	bool			ejectingPage_d;			// true if I am ejecting a page
	Time			beginTime_d;			// Time when the report was opened
	bool			building_d;				// true while building
	bool			loading_d;				// true if building is while reading
											// .rpo file

	static MsgTable *rmsg_d;				// message table
	
	void printCondition(Report::When when, Report::WhenType t, 
			bool use_prev);
	void setChangeFlag(bool flag);
	void setChangedFields();
	void setNull();
	bool evalIf(Int zone);
	void initValues();

public:

	static Int const MAGIC;
	
	RpRep();
	RpRep(const String &nm, Report::Flag flags, Int ncopies = 1, 
			  const String &option = NULL_STRING);
	~RpRep();

	RP_DECLARE_CLASS

	void open(const String &nm, Report::Flag flags, Int ncopies,
			  const String &option);

	int begin(Int ncopies, const String &fmt);
	int execute(UnShort zone);
	int end();

	void clearZone(int zone);
	void ejectPage(bool use_prev = false);
	void skipLines(Int nlines);
	void fromDb(Int from, Int to, Int rowno = 0);
	Int checkSum() const;
	void verifyCheckSum(Int chksum);
	void setOutput(Report::OutputType out_to, const String &desp);
	void accumulate(RpZoneExprObj *obj, const Num &a);
	void accumulate(RpFldRep *f, const Num &a);
	void copyPrevious();
	void drawZ(UnChar c);

	inline bool ok() const;

	// messager methods
	static String msg(const char *mindex);
	static String msg(const String &mindex);

	// Used by rgen utility
	Int nSchemas() const;
	Int indOfSchema(const String &n) const;
	Int addSchema(const String &n);
	RpZoneExprObj *findExpr(const String &name) const;
	Int add(RepZoneRep *zr);
	Int add(RpFldRep *fr);
	RpFldRep *field(const String &fName) const;
	const String &schemaName(Int i) const;
	Int fieldNo(const String &fName) const;
	Int variableNo(const String &vName) const;
	AnyValue varValue(Int varNo) const;
	RpFldRep *field(Int n) const;
	RpFldRep *orderField(Int n) const;
	Int nextField(RpFldRep *fld) const;
	void clearOrderFlds();
	Int nOrderFlds() const;
	RepZoneRep *zone(const String &zName) const;
	RepZoneRep *zone(Int n) const;
	Int nZones() const;
	inline Int toInternVert(Num val) const;
	inline Int toInternHoriz(Num val) const;
	inline Num toExternVert(Int val) const;
	inline Num toExternHoriz(Int val) const;

//	inline void inputFrom(Report::OutputType v);
//	inline void outputTo(Report::OutputType v);
//	inline void noFormFeed(bool v);
	inline bool noFormFeed() const;
	inline void maxLinesAfter(Int n);
	inline Int maxLinesAfter() const;
	Int nFields() const;
//	inline void width(Int v);
	inline Int width() const;
//	inline void leftMarg(Int v);
	inline Int leftMarg() const;
//	inline void rightMarg(Int v);
	inline Int rightMarg() const;
//	inline void fLength(Int v);
	inline Int fLength() const;
//	inline void topMarg(Int v);
	inline Int topMarg() const;
//	inline void botMarg(Int v);
	inline Int botMarg() const;
	inline void currY(Int y);
	inline Int currY() const;
	inline void lstPZone(Int v);
	inline Int lstPZone() const;
//	inline const String &output() const;
	inline const Time &beginTime() const;
	inline bool	doEject() const;
	inline void	doEject(bool v);
	inline const String &foreground() const;
//	inline void foreground(const String &colour);
	inline const String &background() const;
//	inline void background(const String &colour);
	inline const String &font() const;
//	inline void font(const String &name);                        
	inline bool exitByUser() const;	
	inline void exitByUser(bool v);	                     
	inline RpUnit unit() const;
//	inline void unit(RpUnit v);
	inline RpPaperSize paperSize() const;
//	inline void paperSize(RpPaperSize v);

	// Used to evaluate expressions
	inline bool beforeReport() const;
	inline bool beforePage() const;
	bool beforeField(Int fldno) const;
	inline bool afterReport() const;
	inline bool afterPage() const;
	bool afterField(Int fldno) const;
		
	inline RpDevice *rpDevice() const;
	inline void rpDevice(RpDevice *);

	void startBuilding(bool forLoading = false);
	virtual bool stopBuilding();
	inline bool building() const;
	inline bool loading() const;

	bool checkAllOptions();
	virtual bool hasChildren() const;

	bool load(const String &nm, Report::Flag flags);
	void store() const;
	void storeNrp() const;
	void storeNrp(FILE *out) const;

	virtual void storeHeader(FILE *out) const;
	virtual bool loadHeader(FILE *in);
	virtual void storeChildren(FILE *out) const;
	virtual bool loadChildren(FILE *in);

	virtual void storeNrpChildren(FILE *out) const;

};

void rperror(const String &fmt, ...);
void rpwarning(const String &fmt, ...);

#include <ideafix/priv/rprep.icc>

#endif
