/*
 *
 * $Id: rpzobj.h,v 1.2 1996/07/15 19:54:16 eugen Exp $
 *
 */

#ifndef RPZOBJ_H
#define RPZOBJ_H

#ifdef __GNUC__
#	pragma interface
#endif

#include <ideafix/priv/rpobj.h>
#include <ideafix/priv/rptypes.h>

class RpRep;
class RpExpr;
class RepZoneRep;
class RpFldRep;
class RpDevObject;
class RpZoneExprObj;

// --------- RpZoneObjects -----------------------------------

class RpZoneObj :	public RpObject
{
	typedef RpObject inherited; 
private:
	RepZoneRep 	*zone_d;

	Int geom_X_d, geom_Y_d, geom_W_d, geom_H_d;	
  	String foreground_d;    
  	String background_d;    

public:
	static RpZoneObj *create(RepZoneRep *r, FILE *in);
	RpZoneObj(RepZoneRep *zr);
	virtual ~RpZoneObj();
	
	RP_DECLARE_CLASS

	enum ZoneObjectType {
	 	r_none,
	 	r_label,
	 	r_line,
	 	r_rect,
	 	r_ellipse,
	 	r_arc,
	 	r_bitmap,
		r_expr
	};	 	 

	inline RepZoneRep *zone() const;

	void geometry(Int x, Int y, Int w, Int h);
	inline Int geom_X() const; 	
	inline Int geom_Y() const; 	
	inline Int geom_W() const; 	
	inline Int geom_H() const; 	

	inline const String &foreground() const;
//	inline void foreground(const String &colour);
	inline const String &background() const;
//	inline void background(const String &colour);
	
	virtual RpZoneExprObj *getRpZoneExpr();				
	
	void output(RpDevObject *obj);

	virtual ZoneObjectType objType() const;
	virtual void print(bool noprint, int valmode);

	virtual String nrpHeader() const;
};

class RpZoneTextObj : public RpZoneObj {

	typedef RpZoneObj inherited; 

	String value_d;
	Alignment 	alignment_d;
	String font_d;

public:

	RpZoneTextObj(RepZoneRep *zr);
	~RpZoneTextObj();

	RP_DECLARE_CLASS

	inline void value(const String &theValue);
	inline String &value();
//	inline void alignment(Alignment v);
	inline Alignment alignment() const;
//	inline void font(const String &name);
	inline const String &font() const;

	virtual ZoneObjectType objType() const;
	virtual void print(bool noprint, int valmode);
};

class RpZoneExprObj :	public RpZoneTextObj 
{                                      
                               
	typedef RpZoneTextObj inherited; 
                               
	// Start of dictionary
	Int			type_d;
	Int 		chkDigit_d;		// check digit attrib.
	Int 		nDec_d;
	RpExpr		*maske_d;		// mask
	bool		fillZeros_d;	// fill with zeros
	bool		nullZeros_d;	// null zeros
	bool		thousandSep_d;	// thousands separator
	bool		showSign_d;		// represent negative numbers
	bool		doRound_d;		// round to the number of decimals
	Int			length_d;		// length
	RpExpr		*expr_d;		// expression

	// End of dictionary

	String		mask_d;			// compiled mask
	Int			exprNo_d;
	bool		noMask_d;
	RpFldRep	*field_d;

public:
	RpZoneExprObj(RepZoneRep *zr);
	~RpZoneExprObj(); 

	RP_DECLARE_CLASS

	// Used for rgen utility
	inline Int type() const;
//	inline void type(Int t);
	inline Int nDec() const;
//	inline void nDec(Int n);
	inline Int length() const;
//	inline void length(Int l);
	inline Int checkDigit();
	inline void checkDigit(Int cd);
	inline bool fillZeros() const;
//	inline void fillZeros(bool v);
	inline RpExpr *maskExpr() const;
	inline void nullZeros(bool v);
	inline RpExpr *expr() const;
//	inline void showSign(bool v);
	inline bool showSign() const;
//	inline void doRound(bool v);
	inline bool doRound() const;
//	inline void thousandSep(bool v);
	inline bool thousandSep() const;
	inline RpFldRep *field() const;
//	RpFldRep *symField();
//	inline void noMask(bool v);
	inline bool noMask() const;
	inline const String &mask() const;
	inline void mask(const String &theMask);

	inline Int exprNo() const;
	inline void exprNo(Int n);
//	bool setMask(RpExpr &mask);
	virtual RpZoneExprObj *getRpZoneExpr();

	virtual ZoneObjectType objType() const;
	virtual void print(bool noprint, int valmode);
};

class RpZoneLabelObj : public RpZoneTextObj {

	typedef RpZoneTextObj inherited; 

public:

	RpZoneLabelObj(RepZoneRep *zr);
	~RpZoneLabelObj();

	RP_DECLARE_CLASS

	virtual ZoneObjectType objType() const;
};

class RpZoneDrawedObj : public RpZoneObj {

	typedef RpZoneObj inherited; 

	Int thickness_d;
	DrawStyle style_d;
	FillStyle fillStyle_d;

public:

	RpZoneDrawedObj(RepZoneRep *zr);
	~RpZoneDrawedObj();

	RP_DECLARE_CLASS

//	inline void thickness(Int n);
	inline Int thickness() const;

//	inline void style(DrawStyle s);
	inline DrawStyle style() const;
//	inline void fillStyle(FillStyle s);
	inline FillStyle fillStyle() const;

	virtual ZoneObjectType objType() const;
};

class RpZoneRectObj : public RpZoneDrawedObj {

	typedef RpZoneDrawedObj inherited; 

public:

	RpZoneRectObj(RepZoneRep *zr);
	~RpZoneRectObj();

	RP_DECLARE_CLASS

	virtual ZoneObjectType objType() const;
	virtual void print(bool noprint, int valmode);
};

class RpZoneLineObj : public RpZoneDrawedObj {

	typedef RpZoneDrawedObj inherited; 
	Orientation orientation_d;
public:

	RpZoneLineObj(RepZoneRep *zr);
	~RpZoneLineObj();

	RP_DECLARE_CLASS

//	inline void orientation(Orientation v);
	inline Orientation orientation() const;

	virtual ZoneObjectType objType() const;
	virtual void print(bool noprint, int valmode);
};

class RpZoneArcObj : public RpZoneDrawedObj {

	typedef RpZoneDrawedObj inherited; 

	Int startAngle_d, endAngle_d;
public:

	RpZoneArcObj(RepZoneRep *zr);
	~RpZoneArcObj();

	RP_DECLARE_CLASS

//	inline void startAngle(Int);
	inline Int startAngle() const;
//	inline void endAngle(Int);
	inline Int endAngle() const;

	virtual ZoneObjectType objType() const;
	virtual void print(bool noprint, int valmode);
};

class RpZoneEllipseObj : public RpZoneDrawedObj {

	typedef RpZoneDrawedObj inherited; 

public:

	RpZoneEllipseObj(RepZoneRep *zr);
	~RpZoneEllipseObj();

	RP_DECLARE_CLASS

	virtual ZoneObjectType objType() const;
	virtual void print(bool noprint, int valmode);
};

class RpZoneBitmapObj : public RpZoneObj {

	typedef RpZoneObj inherited; 

	String fileName_d;
public:

	RpZoneBitmapObj(RepZoneRep *zr);
	~RpZoneBitmapObj();

	RP_DECLARE_CLASS

//	inline void fileName(String&);
	inline const String& fileName() const;

	virtual ZoneObjectType objType() const;
	virtual void print(bool noprint, int valmode);
};

#include <ideafix/priv/rpzobj.icc>

#endif /* RPZOBJ_H */

