//!Package InterSoft.found.types
// -----------------------------------------------------------------------
// Copyright (c) 1995 InterSoft Co.  All Rights Reserved
// THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF InterSoft Co.
// The copyright notice above does not evidence any actual or intended 
// publication of such source code.
//
// $Id: Boolean.h,v 1.1 1996/05/31 15:59:18 valeria Exp $
//
// Description:
// Class definitions for the Boolean data type
// -----------------------------------------------------------------------

#ifndef	BOOLEAN_H
#define	BOOLEAN_H

#include <ifound.h>

#ifdef __GNUC__
#	pragma interface
#endif

/**
 * This class implements a booleand object, which can take three values:
 * <ul>
 * <li> true (1)
 * <li> false (0)
 * <li> BOOL_NULL
 * </ul>
 * It was created because the C++ didn't have a boolean
 * data type. Now C++ has a bool type but it is not capable of holding a
 * null value. <p>
 * @version $Revision: 1.1 $
 * @author $Author: valeria $
 */
class Boolean {

// private:

	int val;

public:

	Boolean();
    Boolean(const Boolean &b);
	Boolean(bool v);

#ifdef HAVE_BOOL
	Boolean(int i);
#endif 

	Boolean(const char *s);

	friend Boolean toBoolean(Int i);
	friend Boolean toBoolean(const Num &v);
	friend Boolean toBoolean(const String &s);

	friend Int  toInt(const Boolean &t);

	friend const char *toCharPtr(Boolean t);
	const char *format(const char *mask = NULL) const;

	friend String toString(Boolean t);
	String format(const String &mask) const;

	friend Int cmp(const Boolean &a, const Boolean &b);
	bool operator< (Boolean b) const;
	bool operator> (Boolean b) const;
	bool operator<=(Boolean b) const;
	bool operator>=(Boolean b) const;
	bool operator==(Boolean b) const;
	bool operator!=(Boolean b) const;

	Boolean &operator=(const Boolean &b);
	operator bool() const;
	bool isNull() const;
	bool operator!() const;

	void store(OutputStream &stream) const;
	void load(InputStream &stream);

	static Int storeLength();
	void store(char *&buff) const;
	static Boolean load(const char *&buff);
};

#include <ifound/Boolean.icc>

#endif
