/********************************************************************
*
* Copyright (c) 1995 InterSoft Co.  All Rights Reserved
* THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF InterSoft Co.
* The copyright notice above does not evidence any
* actual or intended publication of such source code.
*
* $Id: dbobj.h,v 1.5 1996/08/23 19:56:37 gustavof Exp $
*
* DESCRIPTION:
*
*********************************************************************/

#ifndef TABLE_H
#define TABLE_H

#include <ifound.h>
#include <ifound/str.h>
#if IDEA_VERSION > 431
#include <ifound/anyvalue.h>
#else
#include <anyvalue.h>
#endif
#include <ifound/parray.h>
#include <ifound/array.h>

#include <ifound/cf.h>

#define TablesArray PtrArray(TableData)
#define SchemasArray PtrArray(SchemaData)
#define FieldsArray Array(AnyValue)
#define StringsArray Array(String)

declare(Array, AnyValue);
declare(Array, String);

struct TableData {
	String name;
	
	union {
		dbtable tabId_I; // IdeaFix
		FILE *  tabId_D; // ASCII
	};
	const char *openMode; // ASCII; (ufa !)

	Int *fldPos;
	FieldsArray *fields;

	inline TableData(const String &n, Int t = ERR);
	inline ~TableData();

};

declare(PtrArray, TableData);

struct SchemaData {

	String name;
	char format; // interf.esquemas.formato like.

	union {
		schema schId_I; // IdeaFix
		Int    schId_D; // ASCII
	};
	
	TablesArray tables;

	inline SchemaData(const String &n, char fmt = 0, Int id = ERR);
	inline ~SchemaData();
};

declare(PtrArray, SchemaData);

class Table {

//private:

	static Table *first;
	Table *next;

	static String setDataSources;
	static void unsetDataSources(char fmt = '\0');

	AnyValue &fld(Int, Int);

protected:

	static schema dd;
	static SchemasArray schemas;
	static String defaultSch;
	static String errorDescr_;

	static Int addTable(TablesArray &, const String &, Int,
						Int, Int const *, Int);
	static Int findSchema(const String &);
	static Int findTable(SchemaData &, const String &);

	Int tabId;

	inline bool ok() const;

public:

	static void setDataSource(const String &ds, char ty);
	static Int openSchema(const String &);
	static inline void setDefaultSchema(const String &);

	static Table *find(const String &, const String &);
	static Table *find(Int);
	static inline const String &errorDescr();

	static void dispose();

	explicit Table();
	virtual ~Table();

	inline Int id() const;
	inline const String &name() const;
	virtual bool fldId(const String &, Int &, Int &) const;

	inline const AnyValue &getFld(Int, Int row = 0);
	inline AnyValue &setFld(Int, Int row = 0, const AnyValue *val = NULL);

	// Cursor handling
	virtual inline bool createCursor(const String &);
	virtual inline void setFrom(Int, FieldsArray &);
	virtual inline void setTo(Int, FieldsArray &);
	virtual inline void use(StringsArray &); // Last one should be "" !!!
	virtual inline void use(Int *);			 // Last one should be ERR !!!
	virtual inline Int fetch() = 0;

	virtual Int get(Int) = 0;
	virtual void put() = 0;
	virtual void add() = 0;
	virtual void del() = 0;
	virtual void init() ;
};

#include <ifound/dbobj.icc>

#endif //TABLE_H


