/********************************************************************
* Copyright (c) 1992 InterSoft Co.  All Rights Reserved
* THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF InterSoft Co.
* The copyright notice above does not evidence any
* actual or intended publication of such source code.
*
* $Id: expr.h,v 1.28 1996/08/22 23:27:42 gustavof Exp $
*
* DESCRIPTION
* Class definitions for Expr Type.
*********************************************************************/

#ifndef EXPR_H
#define EXPR_H

#define YYSTYPE_PTR void *

#include <ifound.h>

#ifdef __GNUC__
#pragma interface
#endif

#include <ifound/instruct.h>
#include <ifound/anyvalue.h>
#include <ifound/str.h>

class Num;
class Date;
class Time;
class Connection;
class Expr;
class ReportExpr;
class FormExpr;
class FmExpr;
class RpExpr;
class Properties;

union Datum {
	UnShort *s;
	Int 	*i;
	String  *str;
	Num		*num;
};

enum ExprFlags {
	HAVE_LVALUE		= 1 << 0,
	HAVE_COMPLEX	= 1 << 1,
	HAVE_ENVVARS	= 1 << 2,
	HAVE_VOID		= 1 << 3,
	HAVE_MSG		= 1 << 4,
};

class ExprObject {

// private:

	Instruction::Id ty_d;
	String *name_d;
	Int *classId_d;
	Int *id_d;
	Int row_d;
	bool hasRow_d;
	
public:

	inline explicit ExprObject();

	inline void clear();
	Instruction::Id load(Datum &buff);

	inline const String &name();
	inline Int classId();
	inline Int id();
	inline bool hasRow();
	inline Int ty();
	inline Int row();

	inline void name(const String &n);
	inline void classId(Int ci);
	inline void id(Int i);
	inline void row(Int r);
};

class ExprStack {

	Expr *expr;
	Datum &ptr;
	enum {
		process,
		noargs,
		finished,
	} status;
		
public:
	
	ExprStack(Expr *e, Datum &p);
	
	void push(const AnyValue &av);
	Int reference();
	String referenceName();
	AnyValue value();

//	bool getAddress(Int *&addrNo, String *&addrName);
//	bool haveArgs();
  	bool getAddress(ExprObject &obj);	
	bool haveArgs() const;

	Expr *expression();

	inline bool wasExecuted() const;
	inline bool noMoreArgs() const;
};

typedef void (*FPES)(ExprStack &);
typedef bool (*BFPES)(ExprStack &);
typedef bool (*BFPEOES)(ExprObject &, ExprStack &);
typedef void (*FPEOES)(ExprObject &, ExprStack &);
typedef bool (*EnumHandler)(const String &, const String &, Int *&);

struct ExprFunction {
	String name;
	FPES fp;	
	BFPES resolve;
	inline explicit ExprFunction(const String &fname, FPES f = NULL,
			BFPES r = NULL);
	inline ExprFunction(const ExprFunction &ef);
};

struct ExprMethod {
	String name;
	FPEOES fp;
	BFPEOES resolve;
	inline explicit ExprMethod(const String &fname, FPEOES f = NULL,
							   BFPEOES r = NULL);
	inline ExprMethod(const ExprMethod &ef);
};

struct Variable;
class gtype(PtrArray, Variable);

struct Variable {

	String name_d;
	AnyValue value_d;
	
	Variable(const String &n = NULL_STRING)
	:	name_d(n)
	{ }
	~Variable()
	{ }
};

class gtype(PtrArray, ExprFunction);
class gtype(PtrArray, ExprMethod);

class Expr {

	friend class ExprStack;
	
// private:

	// routines for damn strings
	void disconnectStrings();
	void connectStrings();

	// Parse handling -----------------------------------
	Int current,	// Source-code-string position
		errorLine;	// Parse-error-line
	char *lexeme;
	int in_nflds;
	int blockLevel;
	int loopLevel;
	inline char get();
	inline char fetch();
	inline void unfetch();
	Int getpunt();
	int lex(YYSTYPE_PTR);
	int _parse();
	int _error(char *);

	void begin();
	void end();

	static UnShort *buffer; // 'static' is just to optimize memory usage.
	static UnShort *endbuff_;
	static union Datum buff;
	static inline UnShort *endbuff();
	static inline Int bottom();
	static Expr * currexpr_;
	void resize(Int);

	Int align(Instruction::Id);

	Int code(const String &s);
	Int code(const Num &n);
	Int code(Instruction::Id rt);
	Int code(const Time &t);
	Int code(const Date &d);
	Int code(bool b);
	Int code(Int i);
	

	Int codeLocalObj(const char *, bool hasRow = false);
	Int codeLocalId(const char *, Instruction::Id, bool hasRow = false);
	Int codeGlobalDecl(const char *, bool);
	Int codeForeignId(const String &, Instruction::Id, bool hasRow = false);
	Int codeGetEnv(const char *);
	Int codeIn(Int);
	Int codeFunction(const char *);
	Int codeMethod(const char *, const String &);
	Int codeMsg(const char *, const char *, const char *);

	void whileAddrFill(Int, Int, Int);
	void ifAddrFill(Int, Int, Int, Int);
	void foreachAddrFill(Int, Int, Int);

protected:

	// stack operations
	virtual void getEnv();
	virtual void in();	
	virtual void not();
	virtual void and();
	virtual void or();
	virtual void between();
	virtual void equal();
	virtual void notEqual();
	virtual void conditionalIf();
	virtual void when();
	virtual void greater();
	virtual void lessThan();
	virtual void greaterOrEqual();
	virtual void lessOrEqual();
	virtual void add();
	virtual void diff();
	virtual void mult();
	virtual void div();
	virtual void umin();
	virtual void execToday();
	virtual void execHour();
	virtual void execFunction(Datum &ptr);
	virtual void execMethod(Datum &ptr);	
	virtual void execMsg(Datum &ptr);
	virtual void execUser();
	virtual void execIf(Datum &ptr);
	virtual void execWhile(Datum &ptr);
	virtual void execForeach(Datum &ptr);
	virtual void execReturn(Datum &ptr);
	virtual void execBreak(Datum &);
	virtual void execContinue(Datum &);
	virtual void execAssign();
	virtual void execAssignAdd();
	virtual void execAssignSub();
	virtual void execAssignMul();
	virtual void execAssignDiv();
	virtual void execLocalObjSet();
	virtual void execLocalObjGet();
	virtual void execLocalDecl();
	virtual void like();
	virtual void match();

	// Local objects handling
	gtype(PtrArray, Variable) *vars_;
	bool declared;
	PtrArray(Variable) *vars();
	AnyValue *address(Int);

	Properties *propty;
	
	gtype(PtrArray, ExprFunction) &funcs_;
	void addDefFunctions();

	gtype(PtrArray, ExprMethod) &meths_;

	// routine to check whether a foreign object exists or not
	virtual bool resolveForeignObj(const String &name, Int &objId, 
								   Int &classId);

	// routine to check whether a global object exists or not
	virtual Int resolveGlobalObj(const String &name);

	// routine to check whether a local object (variable) exists or not
	virtual Int resolveLocalObject(const String &name);
	virtual Int installLocalObject(const String &name);

	AnyValue *localObjectAddress(Int varId);
	// routine to check whether a field exists or not
//	virtual Int mapField(const String &);

	// routine to check whether a variable exists or not
//	virtual Int mapVariable(const String &);

	String		expr;		// source string
	UnShort		*vector;	// pointer to compiled expression 
	Int			len;		// vector length
	Int			stlen;		// store length
	ExprFlags	flags;		// expression flags
	AnyValue	*sval;		// if the expression is simple this value
	String		errorDescr_;// Error description (parse, resolve, ...)
	IFPCCP		msgChecker;	// function that tells me if the identifier is
							// or is not a msgtable
	EnumHandler	eHandler;	// function that resolves the enum values

	bool		returning;	// if the return statement was executed
	bool		breaking;	// if the break statement was executed
	bool		continuing;	// if the continue statement was executed

	// stack operations
	AnyValue *myStack;
	AnyValue pop();
	const AnyValue &top();
	void top(const AnyValue &av);
	void push(const AnyValue &av);
	void assignStack();
	void resetStack();

	virtual void resolveOthers(Instruction::Id op);
	virtual void execGlobalObjGet();
	virtual void execGlobalObjSet();

	virtual bool resolveForeach(Datum &);

	bool resolveFunction(Int funcNo, Datum &buff);
	bool resolveMethod(Int methNo, Datum &buff);
	bool resolveArgs(Datum &dbuff);
	Instruction::Id resolveObjects(Datum &buff);
//	Instruction resolveFields(Datum &buff);
	
	inline void operator=(const Expr &);

	void clear();
	void fullClear();
	void clearFunctions();
	void clearMethods();
	void clearLocalObjects();
	
	Instruction::Id execute(Datum &ptr);

	void skipUnusedArgs(Datum &ptr);

	virtual Int findKeyword(const String &);

//	void destroy();

	// low level left value routines
//	Instruction findFullAddress(Int *&addrNo, String *&addrName, Datum &ptr);

public:

	explicit Expr(const String &txt, IFPCCP mc = NULL, EnumHandler eh = NULL);
	Expr();
	Expr(const Expr &e);
	virtual ~Expr();

	inline void setProperties(Properties *p);
	void compile(const String &txt);
	void parse(const String &txt);
	Int parseError(String &);

	bool operator !();
	operator int();
	bool isNull();

	virtual inline String &errorDescr();

	Datum goTo(const Datum &addr);

	inline const String &getExpr() const;
	inline Int storeLength() const;

	inline bool isComplex()	const;
	inline bool haveLValue() const;
	inline bool haveEnvVars() const;
	inline bool canBeVoid()	const;
	inline bool haveMsgs() const;
	
	Int addFunction(const String &fname, FPES f = NULL, BFPES r = NULL);
	//(Note: For public (or subclasses) exec and resolve funcs are mandatory)
	Int addMethod(const String &mname, FPEOES f, BFPEOES r);

	// left value routines
	Int address();
	String symAddress();

	// localObject related methods
	virtual const AnyValue &localObjectValue(Int varId);
	void localObjectValue(Int varId, const AnyValue &av);
	
	// read and write methods from/to a buffer
	// funciones para grabarse y leerse de un buffer
	Int store(void *buff);
	void load(void *buff);

	// read and write methods from/to a	FILE
	void store(FILE *);
	void load(FILE *);

	AnyValue eval();

	bool resolveObjects();

	// methods to send and receive an expression from/to a connection
	friend Connection &operator<<(Connection &c, const Expr &e);
	friend Connection &operator>>(Connection &c, Expr &e);

	static Expr * currExpr() { return currexpr_; }
	virtual ReportExpr *getReportExpr();
	virtual FormExpr *getFormExpr();
	virtual FmExpr *getFmExpr();
	virtual RpExpr *getRpExpr();

	virtual String getMsg(const String &tab, const String &sec,
						  const String &msg);

	bool isMsgTab(const String &id);
	void codeEnumValue(const String &en, const String &eni);
	void codeInEnum(const String &en);
	void debugExpr() const;
};

#include <ifound/expr.icc>

#endif // EXPR_H
