//!Package InterSoft.found.types
/***********************************************************************
* Copyright (c) 1991 InterSoft Co.  All Rights Reserved
* THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF InterSoft Co.
* The copyright notice above does not evidence any
* actual or intended publication of such source code.
*
* $Id: itypes.h,v 1.34 1996/08/15 16:40:35 eduardoc Exp $
*
* DESCRIPTION
* Class definitions for IdeaFix basic types.
*********************************************************************/

#ifndef	ITYPES_H
#define	ITYPES_H

#include <ifound.h>

#ifdef __GNUC__
#pragma interface
#endif

#include <ifound/str.h>
#include <ifound/numdefs.h>

#ifdef ceil
#	undef ceil
#endif
#ifdef floor
#	undef floor
#endif

// To have the store and load prototypes without including istream.h

class OutputStream;
class InputStream;

const String &typeName(Int t);

enum {
	SIGN_IGNORE,	// Show always as positive.
	SIGN_LEADING,	// Show with leading cero if necesary.
	SIGN_TRAIL,		// Show with trailing minus if necesary.
	SIGN_PAREN,		// Show number with brakets if negative.
	SIGN_CREDIT		// Credit sign format.
};

#include <essentia/connect.h> // class Connection;

// Num class 

// This class was documented by Oscar (/usr2/acct/oscar/doc/itypes/itypes.doc)
class Num : public NUM
{

	friend class SvNumField;

// private:

	void trunc(Int ndec, short up = 0);
	char *_store(char *buff, Int tots, Int nsd) const;
	const char *_load(const char *buff, Int tots, Int nsd);

public:

	Num()				{ exp = -MAX_EXP; len = 0; }
    Num(const Num &n)   { *this = n; }
	Num(int j)			{ if (j) LToNum(j, this); else exp = len = 0; }
	Num(Int j)			{ if (j) LToNum(j, this); else exp = len = 0; }
	Num(double k)		{ if (k) FToNum(k, this); else exp = len = 0; }
	explicit Num(const char *cs) { StrToNum(cs, this); }
	explicit Num(const String &r) { StrToNum(toCharPtr(r), this); }

	Num(const char *&buff, Int l, Int nd = 0) { load(buff, l, nd); }


	// This has to be deprecated

	friend Num toNum(char c)
	{ Num a; ByteToNum(c, (NUM *) &a); return a; }
	friend Num toNum(short i)
	{ Num a; IToNum(i, (NUM *) &a); return a; }
	friend Num toNum(Int j)
	{ Num a; LToNum(j, (NUM *) &a); return a; }
	friend Num toNum(double k)
	{ Num a; FToNum(k, (NUM *) &a); return a; }
	friend Num toNum(const char *cs)
	{ Num a; StrToNum(cs, (NUM *) &a); return a; }
	friend Num toNum(const String &str)
	{ return Num(toCharPtr(str)); }

	friend short toShort(const Num &num);
	friend char toByte(const Num &num);
	friend Int toInt(const Num &num)
	{ return NumToL((const NUM *) &num); }
	friend double toDouble(const Num &num)
	{ return NumToF((const NUM *) &num); }

	friend const char *toCharPtr(const Num &num, Int n = 255, Int p = INT_MAX)
	{ return NumToStr((const NUM *) &num, n, p); }
	friend String toString(const Num &num, Int n = 255, Int p = INT_MAX);

	String format(Int size, Int ndec = 0, bool separ = true, 
				  Int sign = SIGN_LEADING, char filler = ' ') const;
	String format(const String &mask) const;

	Num& operator=(const Num &a);

	Num& operator+=(const Num &a)  { (void) NumAdd(this, &a); return *this; }
	Num& operator-=(const Num &a)  { (void) NumSub(this, &a); return *this; }
	Num& operator*=(const Num &a)  { (void) NumMul(this, &a); return *this; }
	Num& operator/=(const Num &a)  { (void) NumDiv(this, &a); return *this; }

	friend Num operator<<(const Num &a, int b)   { return a.shift(b); }
	friend Num operator>>(const Num &a, int b)   { return a.shift(-b); }
	Num &operator<<=(int b) { (void) NumShift(this, b); return *this;  }
	Num &operator>>=(int b) { (void) NumShift(this, -b); return *this; }

	friend Num operator++(Num &a)  { return a += 1L; }
	friend Num operator--(Num &a)  { return a -= 1L; }

	friend Num operator+ (Num a, const Num &b) { return a += b; }
	friend Num operator- (Num a, const Num &b) { return a -= b; }
	friend Num operator* (Num a, const Num &b) { return a *= b; }
	friend Num operator/ (Num a, const Num &b) { return a /= b; }

	Num  operator^(const Num &a) const;

	Num  operator-() const { Num ret = *this; ret.exp = -exp; return ret; }
	Int  operator! () const { return ! len; }

	friend Int  cmp(const Num &a, const Num &b)
	{ return NumCmp((const NUM *) &a, (const NUM *) &b); }
	friend bool operator< (const Num &a, const Num &b)
	{ return NumCmp((const NUM *) &a, (const NUM *) &b) < 0; }
	friend bool operator> (const Num &a, const Num &b)
	{ return NumCmp((const NUM *) &a, (const NUM *) &b) > 0; }
	friend bool operator<=(const Num &a, const Num &b)
	{ return NumCmp((const NUM *) &a, (const NUM *) &b) <= 0; }
	friend bool operator>=(const Num &a, const Num &b)
	{ return NumCmp((const NUM *) &a, (const NUM *) &b) >= 0; }
	friend bool operator==(const Num &a, const Num &b)
	{ return NumCmp((const NUM *) &a, (const NUM *) &b) == 0; }
	friend bool operator!=(const Num &a, const Num &b)
	{ return NumCmp((const NUM *) &a, (const NUM *) &b) != 0; }

	Num  ceil (Int ndec = 0) const;
	Num  floor(Int ndec = 0) const;
	Num  round(Int ndec = 0) const;

	Num sqrt() const;

	Int nthDig(Int n) const
	{ return NumNthDig(this, n); }

	Int totDigits(Int &nint, Int &ndec) const
	{ return NumTotDigits(this, &nint, &ndec); }

	Num shift(int n = 1) const;

	bool isNull() const { return NumIsNull(this); }
	bool isMax() const  { return NumIsMax(this) ; }
	bool isMin() const	{ return NumIsMin(this) ; }
	bool isZero() const	{ return NumIsZero(this); }

	void store(OutputStream &stream) const;
	void load(InputStream &stream);
	void store(char *&buff, Int l, Int nd = 0) const;
	void load(const char *&buff, Int l, Int nd = 0);
	void store(char *&buff) const;
	void load(const char *&buff);

	static bool sameRep(Int l1, Int d1, Int l2, Int d2);
	static Int storeLength();
	
	static void intLength(Int l, Int nd, Int &tots, Int &nsd);

	static Int conLength(Int l, Int nd);

	friend Connection &Connection::operator<<(const Num &n);
	friend Connection &Connection::operator>>(Num &n);
	friend void stNum(FILE *f, const Num &n);
	friend Num ldNum(FILE *f);

	void debug(const char *name);
};

// Date class

// This class was documented by Oscar (/usr2/acct/oscar/doc/itypes/itypes.doc)
class Date {

	friend class Time;

// private:

	Int idate;

	void recalc_values() const;
	const char *form(const char *mask, Int seconds = -1L) const;

public:

	Date() 							{ idate = LONG_MIN; }
	Date(const Date &d)             { *this = d; }
	Date(const Num &d, const Num &m, const Num &y);
	Date(const char *s);
	Date(Int d, Int m, Int y);
	explicit Date(short d);
	
	friend const char *toCharPtr(Date d, Int dfmt = DEFAULT_FMT);
	friend Date toDate(Int i);
	friend Date toDate(const String &s) { return Date(toCharPtr(s)); }
	friend Date toDate(Time t);
	friend Time toTime(Date d);
	friend String toString(Date d, Int dfmt = DEFAULT_FMT)
	{ return String(toCharPtr(d, dfmt)); }

	friend Int  toInt(const Date &d) { return d.idate; }

	const char *format(const char *mask);
	String format(const String &mask) const;

	Int  day() const;
	Int  month() const;
	Int  year() const;
	Int  dayOfWeek() const;
	Int  dayOfYear() const;
	Int  weekOfYear() const;
	Int  quarter() const;
	Date lastOfMonth() const;
	Date firstOfMonth() const;
	String dayOfWeekName() const;
	String monthName() const;
	Int monthsSince(const Date b) const; 
	Date addMonths(Int) const;
	String isHoliday() const;	
	static Date easter(Int);
    
	friend Int cmp(const Date &a, const Date &b)
		{ return a.idate == b.idate ? 0 : (a.idate < b.idate ? -1 : 1); }
	bool operator< (Date b) const	{ return idate <  b.idate; }
	bool operator> (Date b) const	{ return idate >  b.idate; }
	bool operator<=(Date b) const	{ return idate <= b.idate; }
	bool operator>=(Date b) const	{ return idate >= b.idate; }
	bool operator==(Date b) const	{ return idate == b.idate; }
	bool operator!=(Date b) const	{ return idate != b.idate; }

	Date &operator=(const Date &d);
	Date &operator+=(Int b)  { idate = add(idate,  b); 	return *this; }
	Date &operator-=(Int b)  { idate = add(idate, -b); 	return *this; }
	Date &operator*=(Int b)  { idate = mult(idate, b); 	return *this; }
	Date &operator/=(Int b)  { idate = div(idate,  b);  return *this; }
	Date &operator++()	     { idate = add(idate,  1); 	return *this; }
	Date &operator++(int)    { idate = add(idate,  1); 	return *this; }
	Date &operator--()	     { idate = add(idate, -1); 	return *this; }
	Date &operator--(int)    { idate = add(idate, -1); 	return *this; }

	Date operator+ (Int b) const  { return toDate(add(idate,  b));}
	Date operator- (Int b) const  { return toDate(add(idate, -b));}
	Int  operator- (Date b) const { return add(idate, -b.idate);  }

	Date operator+=(const Num & b)
		{ idate = add(idate,  toInt(b)); return *this; }
	Date operator-=(const Num & b)
		{ idate = add(idate, -toInt(b)); return *this; }
	Date operator+ (const Num & b) const
		{ return toDate(add(idate,  toInt(b))); }
	Date operator- (const Num & b) const
		{ return toDate(add(idate, -toInt(b))); }

	bool isNull() const { return bool(idate == LONG_MIN); }

	void store(OutputStream &stream) const;
	void load(InputStream &stream);

	static Int storeLength() { return sizeof(Int); }

	void store(char *&buff) const;
	void store(FILE *fp) const;
	static Date load(const char *&buff);
	static Date load(FILE *fp);
	void storeShort(char *&buff) const;
	static Date loadShort(const char *&buff);

	friend Date osDate(Int s);
	Int osDate();

	static void setFmt(Int fmt);

	friend Connection &Connection::operator<<(const Date &d);
	friend Connection &Connection::operator>>(Date &d);

};


// Time class

// This class was documented by Oscar (/usr2/acct/oscar/doc/itypes/itypes.doc)
class Time {

	friend class Date;
	friend Date toDate(Time t);

// private:

	Int itime;

	void recalc_values() const;

public:

	Time() 							 { itime = LONG_MIN; }                  
	Time(const Time &t)              { *this = t; }
	Time(const char *s);
	Time(Int h, Int m, Int s);
	Time(Int d, Int h, Int m, Int s);
	Time(const Num &h, const Num &m, const Num &s);
	Time(const Num &d, const Num &h, const Num &m, const Num &s);
	explicit Time(short t);

	friend Time toTime(Int secs);
	friend Time toTime(const Num &secs);

	friend Time toTime(const String &s);
	friend Time toTime(Date d);

	friend Int  toInt(const Time &t) { return t.itime; }

	friend const char *toCharPtr(Time t, Int fmt = DEFAULT_TFMT);
	const char *format(const char *mask);
	String format(const String &mask) const;
	friend String toString(Time t, Int fmt = DEFAULT_TFMT) 
		{ return String(toCharPtr(t, fmt)); }

	Int second()	const;
	Int minute()	const;
	Int hour()		const;

	Int seconds()	const;
	Int minutes()	const;
	Int hours()		const;
	Int days()		const;

	friend Int cmp(const Time &a, const Time &b)
		{ return a.itime == b.itime ? 0 : (a.itime < b.itime ? -1 : 1); }
	bool operator< (Time b) const	{ return itime <  b.itime; }
	bool operator> (Time b) const	{ return itime >  b.itime; }
	bool operator<=(Time b) const	{ return itime <= b.itime; }
	bool operator>=(Time b) const	{ return itime >= b.itime; }
	bool operator==(Time b) const	{ return itime == b.itime; }
	bool operator!=(Time b) const	{ return itime != b.itime; }

	Time &operator=(const Time &b);
	Time &operator+=(Int b)   { itime=add(itime, b); 	  	  return *this; }
	Time &operator-=(Int b)   { itime=add(itime,-b); 	  	  return *this; }
	Time &operator*=(Int b)   { itime=mult(itime, b); 	  	  return *this; }
	Time &operator/=(Int b)   { itime=div(itime, b); 	  	  return *this; }
	Time &operator+=(Time b)  { itime=add(itime, b.itime); 	  return *this; }
	Time &operator-=(Time b)  { itime=add(itime,-b.itime); 	  return *this; }
	Time &operator++()		  { itime = add(itime,  1);       return *this; }
	Time &operator++(int)	  { itime = add(itime,  1);       return *this; }
	Time &operator--()		  { itime = add(itime, -1); 	  return *this; }
	Time &operator--(int) 	  { itime = add(itime, -1); 	  return *this; }
	Time operator+ (Time b) const { return toTime(add(itime, b.itime));}
	Time operator- (Time b) const { return toTime(add(itime,-b.itime));}

	void store(OutputStream &stream) const;
	void load(InputStream &stream);

	bool isNull() const { return bool(itime == LONG_MIN); }

	void store(char *&buff) const;
	void store(FILE *fp) const;
	static Time load(const char *&buff);
	static Time load(FILE *fp);
	void storeShort(char *&buff) const;
	static Time loadShort(const char *&buff);

	Int osTime();

	static Int storeLength() { return sizeof(Int); }

	friend Time osTime(Int s);
	friend Connection &Connection::operator<<(const Time &t);
	friend Connection &Connection::operator>>(Time &t);
};

#include <ifound/Boolean.h>
#endif
