/********************************************************************
* Copyright (c) 1994 InterSoft Co.  All Rights Reserved
* THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF InterSoft Co.
* The copyright notice above does not evidence any
* actual or intended publication of such source code.
*
* $Id: macros.h,v 1.10 1996/08/21 15:37:26 eugen Exp $
*
* DESCRIPTION
*********************************************************************/

#ifndef IX_MACROS_H
#define IX_MACROS_H

#include <ifound/osdep.h>
#include <cfix/priv/ideaimp.h>

#ifdef DEBUG
#	ifndef ISDEBUG
#		define ISDEBUG	1
#	endif
#endif

#include <ifound/iddebug.h>
#include <ifound/trmalloc.h>

/* Assertion macro */
#ifndef ISDEBUG
#	define NOASSERTH
#endif

#ifdef __cplusplus
extern "C" {
#endif
void _assert(char const *, char const *, int);
void _precond(char const *, char const *, int);
void _postcond(char const *, char const *, int);
void _abnormal(char const *, int);
#ifdef __cplusplus
}
#endif

#ifdef NOASSERTH
#	define hAssert(EX)
#else
#	define hAssert(EX) \
		do { if (EX) ; else _assert(quote(EX), __FILE__, __LINE__); } while(0)
#endif

#ifdef NOPRECONDITIONS
#	define require(EX)
#else
#	define require(EX) \
		do { if (EX) ; else _precond(quote(EX), __FILE__, __LINE__); } while(0)
#endif

#ifdef NOPOSTCONDITIONS
#	define ensure(EX)
#else
#	define ensure(EX) \
		do { if (EX) ; else _postcond(quote(EX), __FILE__, __LINE__); } while(0)
#endif

#define abnormal	forever _abnormal(__FILE__, __LINE__)

#define assert(EX) \
		do { if (EX) ; else _assert(quote(EX), __FILE__, __LINE__); } while(0)

#define check(EX)		assert(EX)
#define implies(a, b)	(!(a) || (b))

#if defined(__cplusplus) && !defined(NOFTRACE)
#define FTRACE(msg)	\
		FunctionTrace paste(__FUNCTION__,ftrace)(__FUNCTION__, msg)
#else
#define FTRACE(msg)
#endif

#ifdef NOTPRINTF
#	define tprintf(n) if(true) ; else printf
#else
#	define tprintf(n) if(!IxVerboseLevel(n)) ; else printf
#endif

#define forever for (;;)

#ifndef NULL
#define NULL	(0)
#endif

#if !HAVE_EXPLICIT
#define explicit
#endif

#if !HAVE_MUTABLE
#define mutable
#endif

#undef HIGH_VAL_IMAGE
#undef HIGH_VALUE_IMAGE
#undef HIGH_VALUE
#undef LOW_VALUE
#undef NULL_VALUE
#define HIGH_VAL_IMAGE		"(high value)"
#define HIGH_VALUE_IMAGE	HIGH_VAL_IMAGE
#define HIGH_VALUE			((char *) 1)
#define LOW_VALUE			((char *) NULL)
#define NULL_VALUE			((char *) NULL)

#ifndef ERR
#define ERR		(-1)
#endif

#ifndef ERROR
#define ERROR	(-1)
#endif

#ifndef OK
#define OK		(0)
#endif

#define TRACE													\
	do {														\
		fprintf(stddbg, "%s (%d)\t :%s\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);		\
		fflush(stddbg);											\
	} while (0);

#define TRACEV(fmt, var)										\
	do {														\
		fprintf(stddbg, "%s (%d)\t :%s\n\t", __FILE__, __LINE__, __PRETTY_FUNCTION__);		\
		fprintf(stddbg, quote(var) " = %" quote(fmt) "\n", var);\
		fflush(stddbg);											\
	} while (0);

#ifdef __cplusplus

#undef paste
#undef name2
#undef name3
#undef name4
#undef declare
#undef implement
#undef declare2
#undef implement2
#undef declare3
#undef implement3

#ifdef HAVE_GENERIC_HPP
#	include <generic.hpp>
#	define paste(a,b)		name2(a,b)
#else
#	ifdef HAVE_PASTE
#		include <generic.h>
#		define paste(a,b)		_Paste(a,b)
#		define name2(a,b)		_Paste2(a,b)
#		define name3(a,b,c) 	_Paste3(a,b,c)
#		define name4(a,b,c,d) 	_Paste4(a,b,c,d)
#	else
#		define paste(a,b)		a##b
#		define name2(a,b)		a##b
#		define name3(a,b,c)		a##b##c
#		define name4(a,b,c,d)  	a##b##c##d
#	endif
#	define declare(a, t) name2(a, declare)(t)
#	define implement(a, t) name2(a, implement)(t)
#	define declare2(a, t1, t2) name2(a, declare2)(t1,t2)
#	define implement2(a, t1, t2) name2(a, implement2)(t1,t2)
#endif

#define declare3(a, t1, t2, t3) name2(a, declare3)(t1,t2,t3)
#define implement3(a, t1, t2, t3) name2(a, implement3)(t1,t2,t3)

/* Macros to avoid declaring generic types in .h's */
#define gtype(GTYPE, CONTENT)				name2(CONTENT, GTYPE)
#define gtype2(GTYPE, CONTENT1, CONTENT2)	name3(CONTENT1, CONTENT2, GTYPE)
#define gtype3(GTYPE, C1, C2, C3)	name4(C1, C2, C3, GTYPE)

#endif

/*------------------- Messages Functions Constants ------------------*/

#define MSG_PRESERVE 		0x01
#define MSG_WAIT			0x02
#define MSG_RELEASE			0x04
#define MSG_FATALERROR		0x08
#define MSG_ERROR			0x10
#define MSG_OTHER			0x20
#define MSG_DISPLAY			0x40
#define MSG_WARNING			0x80

/* 
 * Packages enumeration:
 *   0 - 7  Reserved for system software packages.
 *   8 - 15 Reserved for application products.
 */
#define NUM_PCKGS       8
#define PKG_RUNTIME		0
#define PKG_DEVELOP		1
#define PKG_IQL			2
#define PKG_SERVICES	3
#define PKG_ESSENTIA	4
#define PKG_CRACKER		5
/*
#define PKG_RESERVED2	6
#define PKG_RESERVED3	7
*/
#define APPL_AURUS		8
#define APPL_DENARIUS	9
#define APPL_IANUS		10
#define APPL_ISYS		11
/*
#define APPL_RESERVED1	12
#define APPL_RESERVED2	13
#define APPL_RESERVED3	14
#define APPL_RESERVED4	15
*/

/* aplications products modules	*/

/* Aurus						*/
#define	APPL_AURUS_BASICO		801
#define	APPL_AURUS_AUFIN		802
#define	APPL_AURUS_AURDGI		803
#define	APPL_AURUS_BANCOS		804
#define	APPL_AURUS_IBRUTOS		805
#define	APPL_AURUS_TESOR		806
#define	APPL_AURUS_COB			807
#define	APPL_AURUS_FAS			808
#define	APPL_AURUS_PRESUP		809
#define	APPL_AURUS_AUTGAN		810
#define	APPL_AURUS_CTASCOB		811

/* Denarius						*/
#define	APPL_DENARIUS_BASICO	901
#define	APPL_DENARIUS_EMBAR		902
#define	APPL_DENARIUS_ICONT		903
#define	APPL_DENARIUS_NOVE		904
#define	APPL_DENARIUS_AUDIT		905
#define	APPL_DENARIUS_RETRO		906
#define	APPL_DENARIUS_PLVAC		907
#define	APPL_DENARIUS_HSEXT		908
#define	APPL_DENARIUS_ASIST		909

/* Ianus						*/
#define APPL_IANUS_BASICO		1001
#define APPL_IANUS_SUMIN		1002
#define APPL_IANUS_COMPRAS		1003
#define APPL_IANUS_INTIA		1004

#endif
