/********************************************************************
* Copyright (c) 1991 InterSoft Co.  All Rights Reserved
* THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF InterSoft Co.
* The copyright notice above does not evidence any
* actual or intended publication of such source code.
*
* $Id: sarray.h,v 1.1 1996/07/04 14:18:06 hernan Exp $
*
* DESCRIPTION
* Interface to generic Simple Array
*********************************************************************/

#ifndef	SARRAY_H
#define	SARRAY_H

#include <ifound.h>

#ifdef __GNUC__
#pragma interface
#endif

#include <ifound/template.h>

#define SimpleArray(CONTENT)		name2(CONTENT, SimpleArray)
#define SimpleArrayRef(CONTENT)		name2(CONTENT, SimpleArrayRef)
#define SimpleArrayCursor(CONTENT)	name2(CONTENT, SimpleArrayCursor)

#define	SimpleArraydeclare(CONTENT)									\
																	\
class SimpleArray(CONTENT) {										\
																	\
	void operator=(const SimpleArray(CONTENT) &);					\
																	\
	CONTENT *cont;													\
	Int dim_d;														\
																	\
	enum { ChunkSize = 8 };											\
																	\
	static Int allocSize(Int n)										\
	{ return (n+ChunkSize-1)/ChunkSize*ChunkSize; }					\
																	\
public:																\
																	\
	explicit SimpleArray(CONTENT)()									\
	: cont(0), dim_d(0) {}											\
	explicit SimpleArray(CONTENT)(Int n)							\
	: cont(0), dim_d(0) { reAlloc(n); }								\
	explicit SimpleArray(CONTENT)(const SimpleArray(CONTENT) &c);	\
	~SimpleArray(CONTENT)() { delete [] cont; }						\
																	\
	Int dim() const { return dim_d; }								\
																	\
	bool isValid(Int n) const { return n>=0 && n<dim(); }			\
																	\
	CONTENT &operator[](Int n)										\
	{																\
		if (!isValid(n)) error(libMsg("OUTOFBOUNDS"));				\
		return cont[n];												\
	}																\
	const CONTENT &operator[](Int n) const							\
	{																\
		if (!isValid(n)) error(libMsg("OUTOFBOUNDS"));				\
		return cont[n];												\
	}																\
																	\
	CONTENT &elem(Int n) { return cont[n]; }						\
	const CONTENT &elem(Int n) const { return cont[n]; }			\
																	\
	Int add(const CONTENT &val)										\
	{ Int n = dim(); reAlloc(n+1); elem(n) = val; return n; }		\
																	\
	void reAlloc(Int n);											\
};

#if 0
#define SimpleArrayRefdeclare(CONTENT)								\
																	\
class SimpleArrayRef(CONTENT) {										\
	void operator=(const SimpleArrayRef(CONTENT) &);				\
	SimpleArray(CONTENT)	&a_d;									\
public:																\
	SimpleArrayRef(CONTENT)											\
		: arr_d(*tr_new SimpleArray(CONTENT)) {}					\
	SimpleArrayRef(CONTENT, Int size)								\
		: arr_d(*tr_new SimpleArray(CONTENT, size)) {}				\
	~SimpleArrayRef(CONTENT) { tr_delete(&arr_d); }					\
																	\
	operator SimpleArray(CONTENT)&() { return a_d; }				\
	operator const SimpleArray(CONTENT)&() const { return a_d; }	\
																	\
	Int dim() const { return a_d.dim(); }							\
	bool isValid(Int n) const { return a_d.valid(n); }				\
	CONTENT &operator[](Int n) { return a_d[n]; }					\
	const CONTENT &operator[](Int n) const { return a_d[n]; }		\
	CONTENT &elem(Int n) { return a_d.elem(n); }					\
	const CONTENT &elem(Int n) const { return a_d.elem(n); }		\
	Int add(const CONTENT &val)	{ return a_d.add(val); }			\
	void reAlloc(Int n) { a_d.reAlloc(n); }							\
};
#endif

/*
	In the reAlloc function, we can NOT use the memcpy function
	and we must assign the string to increment the reference
	count, so the strings are not destroyed when we delete cont,
	also when we new n_cont CONTENTS constructor is called, so if
	we memcpy something over it, and the constructor has malloced
	dynamic memory (Ex: Strings), it is forever lost in 
	hyper-space.
*/
#define	SimpleArrayimplement(CONTENT)								\
void SimpleArray(CONTENT)::reAlloc(Int n)							\
{																	\
	Int newSize = allocSize(n);										\
	if (newSize != allocSize(dim())) {								\
		CONTENT *n_cont = new CONTENT[newSize];						\
		Int copy = dim()<n? dim(): n;								\
		for (Int i = 0; i < copy; i++)								\
			n_cont[i] = cont[i];									\
		delete [] cont;												\
		cont = n_cont;												\
	}																\
	dim_d = n;														\
}																	\
SimpleArray(CONTENT)::SimpleArray(CONTENT)(const SimpleArray(CONTENT) &a)\
{																	\
	dim_d = a.dim_d;												\
	cont = new CONTENT[allocSize(dim())];							\
	for (Int i = 0; i<dim(); i++)									\
		cont[i] = a.cont[i];										\
}


#define	SimpleArrayCursordeclare(CONTENT)							\
																	\
class SimpleArrayCursor(CONTENT) {									\
																	\
	CONTENT *base;													\
	CONTENT *curr;													\
	CONTENT *last;													\
																	\
	void reset()													\
	{ base = curr = last = 0; }										\
	void assign(SimpleArray(CONTENT) &pa)							\
	{																\
		base = curr = &pa.elem(0);									\
		last = base + pa.dim();										\
	}																\
public:																\
	explicit SimpleArrayCursor(CONTENT)(SimpleArray(CONTENT) &pa)	\
	{ assign(pa); }													\
	SimpleArrayCursor(CONTENT)()									\
	{ reset(); }													\
	SimpleArrayCursor(CONTENT) &operator=(SimpleArray(CONTENT) &pa)	\
	{ assign(pa); return *this; }									\
	Int index() const												\
	{ return curr - base; }											\
	operator bool()	const											\
	{ return last > curr && curr >= base; }							\
	bool operator!() const											\
	{ return !bool(*this); }										\
	SimpleArrayCursor(CONTENT) &operator+=(Int offs)				\
	{ if (base) curr += offs; return *this; }						\
	SimpleArrayCursor(CONTENT) &operator-=(Int offs)				\
	{ if (base) curr -= offs; return *this; }						\
	SimpleArrayCursor(CONTENT) &operator++()						\
	{ if (base) curr++; return *this; }								\
	SimpleArrayCursor(CONTENT) &operator--()						\
	{ if (base) curr--; return *this; }								\
	void operator++(int)											\
	{ if (base) curr++; }											\
	void operator--(int)											\
	{ if (base) curr--; }											\
	CONTENT *operator->()											\
	{ return curr; }												\
	operator CONTENT*()												\
	{ return curr; }												\
	CONTENT &operator*()											\
	{ return *curr; }												\
};

#define	SimpleArrayCursorimplement(CONTENT)

#endif // SARRAY_H

