//!Package InterSoft.found.types
// -----------------------------------------------------------------------
// Copyright (c) 1995 InterSoft Co.  All Rights Reserved
// THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF InterSoft Co.
// The copyright notice above does not evidence any actual or intended 
// publication of such source code.
//
// $Id: str.h,v 1.24 1996/08/15 16:40:42 eduardoc Exp $
//
// Description:
// Class definitions for the String data type
// -----------------------------------------------------------------------

#ifndef	STR_H
#define	STR_H

#include <ifound.h>

#ifdef __GNUC__
#	pragma interface
#endif

// forward declarations
struct srep;
class StrCursor;
class RegExp;
class OutputStream;
class InputStream;
class String;

// SubString class

class SubString
{

	friend class String;

public:

	SubString& operator=(const String&);

private:

	String* str_x;		// String this is a SubString of
	UnShort pos_d;		// position within str_x
	UnShort len_d;		// length of substring

	inline SubString(const String* str, Int pos, Int len);
//	SubString(const SubString& s);
	SubString(const String* org, const String& sep, Int n);
	SubString(const String* org, const char* cs, Int n);
	void make(const String& org, const char* s2, UnShort s2_len, Int n);

};


/**
 * String representation class
 * This class is somewhat strange, because objects are of variable length.
 * Do not define constructors or destructors!
 * Use <em>build</em> to construct and <em>destroy</em> instead of ~srep()
 * to destroy the object.
 * @version $Revision: 1.24 $
 * @author $Author: eduardoc $
 */
class srep {

	useIdCheck(srep_id);

public:

	static inline srep* build(UnShort len);	
	inline void destroy();

	inline UnShort& n();
	inline UnShort& len();
	inline char* s();

	inline void start();
	inline bool notEmpty() const;

private:

	UnShort	n_d;
	UnShort	len_d;
	char s_d[1];

};

/**
 * <em>String</em> class provides string operations not allowed in standard C++.                                                         
 * An instance of String can be created, concatenated, copied, assigned or 
 * compared with any other Strings or with other kind of objects such as
 * chars or character strings.<p>
 * Some member functions (toCharPtr, toInt, etc.) are declared friends 
 * to allow calling them using functional notation.
 * @version $Revision: 1.24 $
 * @author $Author: eduardoc $
 */
class String
{

	friend class SubString;
	friend class StrCursor;
	friend class RegExp;

public:

	inline String();
	inline ~String();
	String(Int l, char c);
	String(const String& x);
	String(const String& x, Int len);
	String(const SubString& ss);
	String(const char* cs, Int len = ERR);
	String(Int len, const char* cs);

	friend char* toCharPtr(const String& s);
	char* disect(Int from, Int len);
	void restore(Int from, Int len);

	friend const char* debugString(Int s);


	friend Int toInt(const String& s, Int base = 0);
	friend String toString(Int v, Int length = 0, Int base = -10);
	friend String toStr(const char* fmt, va_list ap);
	friend String toStr(const String& fmt, va_list ap);

	friend String doFormat(String fmt, ...);
	friend String doFormat(const char* fmt, ...);

	inline Int length() const;

#ifdef HAVE_BOOL
	inline operator bool() const;
#else
	inline operator const void*() const;
#endif
	inline bool operator!() const;
	inline bool isNull() const;
	friend Int cmp(const String& csr, const char* cs); 
	friend Int icmp(const String& csr, const char* cs); 
	inline friend bool operator==(const String& csr, const char* cs);
	inline friend bool operator!=(const String& csr, const char* cs);
	inline friend bool operator<(const String& csr, const char* cs);
	inline friend bool operator>(const String& csr, const char* cs);
	inline friend bool operator<=(const String& csr, const char* cs);
	inline friend bool operator>=(const String& csr, const char* cs);
	friend Int cmp(const String& csr, const String& s);
	friend Int icmp(const String& csr, const String& s);
	inline friend bool operator==(const String& csr, const String& s);
	inline friend bool operator!=(const String& csr, const String& s);
	inline friend bool operator<(const String& csr, const String& s); 
	inline friend bool operator>(const String& csr, const String& s);
	inline friend bool operator<=(const String& csr, const String& s);
	inline friend bool operator>=(const String& csr, const String& s);


	inline bool startsWith(const String& s) const;
	String& operator=(const char* cs);
	String& operator=(const String& x);
	String& operator=(const char c);
	
	String operator+=(const String& s2);
	String operator+=(const char* s2);
	String operator+=(const char c);
	String& operator-=(const String& st);
	friend String operator+(const String& s1, const String& s2);
	friend String operator+(const String& s1, const char* cs);

	inline Int storeLength() const;
	inline static Int storeLength(Int l);

	inline 		 SubString operator()(Int pos, Int len);
	inline 		 SubString operator()(Int pos);

	inline const SubString operator()(Int pos, Int len) const;
	inline const SubString operator()(Int pos) const;

	inline 		 SubString field(Int n, const char* cs);
	inline 		 SubString field(Int n, const String& sep);
	inline const SubString field(Int n, const char* cs) const;
	inline const SubString field(Int n, const String& sep) const;

	inline unsigned char operator[](Int n) const;

	bool setChar(char c, Int n);
	bool delChar(Int n);
	bool insChar(char c, Int n);

	Int index(const String& s, Int pos = 0) const;
	Int index(char c, Int pos = 0) const;
	Int indexIgnoringQuotes(const String& s, const String& quotes, Int pos = 0) const;
	Int indexIgnoringQuotes(char c, const String& quotes, Int pos = 0) const;
	Int backIndex(char c, Int pos = INT_MAX) const;

	Int count(char c, Int pos = 0) const;

	String toLower() const;
	String toUpper() const;

	String toAscii() const;
	String fromAscii() const;

	String& encode();
	String& decode();
	String& decodeRegExp();

	String replaceArgs(const String args[]) const;
	String replace(const String& search, const String& repl) const;

	//+ eliminate whitespace
	String& rTrim();
	String& lTrim();
	String& fullTrim();
	String& prune();

	String& compileMask();
	String center(int width) const;
	String format(String omask) const;
	String& reverse();

	String getEnv() const;
	void putEnv() const;
 	String expandEnvVars() const;

	String extName() const;
	String baseName() const;
	String baseFullName() const;
	String pathName() const;
	String capitalized() const;
	friend Int hashVal(const String& s, Int modulo);

	void store(OutputStream& stream) const;
	void load(InputStream& stream);

	void store(char*& buff) const;
	void load(const char*& buff);

	void connect();
	void disconnect();

	char* getBuffer(Int len);

private:

	srep* p;

	void realloc(UnShort len);

};

//-- These typedefs are useful for instantiating macro-templates --//
typedef String& StringRef;
typedef String* StringPtr;


/**
 * StrCursor is used to retrieve a substring from a given String. Different
 * substrings are separated by a separator token String.<p>
 * When you create one StrCursor object, it is positioned at the first
 * SubString of the String used like reference.  This cursor can move up 
 * or down the String finding an retrieving substrings.  
 * @version $Revision: 1.24 $
 * @author $Author: eduardoc $
 */
class StrCursor
{

public:

	StrCursor(const String& s, const String& t);
	explicit StrCursor(const String& s);
	void reStart();
	operator void*();
	bool ok();
	operator String();
	String value();
	StrCursor& operator++();
	StrCursor& operator--();

private:

	String str;				// we take substrings from this string
	String tok;				// separator token
	Int len;
	Int pos;

};

// Needed for Symantec
#ifdef __cplusplus
extern "C" {
#endif

#ifndef NULL_STRING_DEF
#	ifdef NO_EXTERN_CONST
		extern String NULL_STRING;
#	else
		extern const String NULL_STRING;
#	endif
#endif

#ifdef __cplusplus
}
#endif

String libMsg(const String& msg);
String appMsg(const String& msg);
String libMsg(const char*);
String appMsg(const char*);

#include <ifound/str.icc>

#endif
