#ifndef wx_fieldh
#define wx_fieldh

#ifdef __GNUG__
#pragma interface
#endif

#include <wx_item.h>
#include <wx_txt.h>

class wxField: public wxText
{
	typedef wxText inherited;

#ifdef wx_xview
	typedef Panel_setting (*xv_TextNotifyProc)(Panel_item, Event *);
	typedef void (*xv_TextEventProc)(Panel_item , Event *);
	xv_TextNotifyProc	xv_prevNotifyProc;
	xv_TextEventProc	xv_prevEventProc;
	static Panel_setting xv_NotifyProc(Panel_item item, Event * x_event);
	static void xv_EventProc(Panel_item item, Event * x_event);

	virtual void xv_OnChange();
#endif
#ifdef wx_motif
	static void m_modifyVerify(Widget w,
				XtPointer clientData, XtPointer callData);
	static void m_activate(Widget w,
				XtPointer clientData, XtPointer callData);
#endif
#ifdef wx_msw
	FARPROC oldWndProc;
	static LONG APIENTRY wxFieldSubClassWindowProc(HWND hWnd,
			UINT message, WPARAM wParam, LPARAM lParam);
	Bool sendIxFunctionKey(int key);
	virtual BOOL MSWCommand(UINT param, WORD id);
#endif

protected:
#ifdef wx_motif
	typedef XmTextVerifyCallbackStruct ModifyStruct;
#else
	struct TextBlock
	{
		char	*ptr;
		int		length;
		Bool	allocated;

		TextBlock() : ptr(""), length(0), allocated(FALSE) {}
		TextBlock(char *str) { tb_AllocText(str); }
		~TextBlock() { tb_FreeText(); }
		void SetNewText(const char *str) { tb_FreeText(); tb_AllocText(str); }

	private:
		void tb_FreeText() { if (allocated) delete [] ptr; }
		void tb_AllocText(const char *str)
			{ length = strlen(str); ptr = new char[length+1];
			  allocated = TRUE; memcpy(ptr,str,length+1); }
	};

	struct ModifyStruct
	{
		Bool	doit;
		int		currInsert, newInsert;
		int		startPos, endPos;
		int		startSel, endSel;
		TextBlock *text;

		ModifyStruct(wxField *item, TextBlock &tb) : doit(TRUE), text(&tb)
		{
#ifdef wx_msw
			DWORD sel = SendMessage((HWND)item->ms_handle,EM_GETSEL,0,0);
			currInsert = newInsert = startPos = startSel = LOWORD(sel);
			endPos = endSel = HIWORD(sel);
#else
			currInsert = newInsert = startPos = startSel = endPos = endSel = 0;
#endif
		}
	};
#endif

	static void SetNewText(ModifyStruct &ms, const char *str);
	virtual int GetTextLength() const;

	virtual void OnModifyVerify(ModifyStruct &ms);

#ifdef wx_msw
	void OnKeyDown(WPARAM wParam);
	void OnChar(WPARAM wParam);
	void OnClear(Bool copy);
	void OnPaste();
	void OnDelChar(Bool backspace);
	void FlushModify(ModifyStruct &ms);
#endif

public:
	wxField(wxPanel *panel, wxFunction func, char *label, char *value = "",
		int x = -1, int y = -1, int width = -1, int height = -1,
		long style = 0, char *name = NULL);

	virtual void ProcessCommand(wxCommandEvent & event);

	void SetValue(const char *value);

    void SetSkip(Bool skip = TRUE);
    void SetReadOnly(Bool readOnly = TRUE);
	void SetLength(int len);
};

inline void wxField::SetValue(const char *value)
	{ inherited::SetValue(value?(char*)value:""); }

////////////////////////////////////////////////////////////////////////////
//
// specific fields: ALPHA, NUM, DATE etc.
//
////////////////////////////////////////////////////////////////////////////

class wxCharField : public wxField
{
	int length_;
	char *testMask_;
	char *outputMask_;
	
	char *value_;

protected:

	virtual void OnModifyVerify(ModifyStruct &ms);

public:
	wxCharField(wxPanel *panel, wxFunction func, char *label, char *value = "",
		int x = -1, int y = -1, int width = -1, int height = -1,
		long style = 0, char *name = NULL,
		int length = 1,
		const char *testMask = NULL, const char *outputMask = NULL);
	~wxCharField();
	void setMasks(const char *tstMsk, const char *outMsk);
};

class wxNumField : public wxField
{

	int digits_, decimal_;
	Bool signed_;
	Bool useComma_;

protected:

	virtual void OnModifyVerify(ModifyStruct &ms);

public:
	wxNumField(wxPanel *panel, wxFunction func, char *label, char *value = "",
		int x = -1, int y = -1, int width = -1, int height = -1,
		long style = 0, char *name = NULL,
		int digits = 1, int decimal = 0,
        Bool isSigned = FALSE, Bool useComma = FALSE);

};

class wxFloatField : public wxField
{
	int length_;

protected:

	virtual void OnModifyVerify(ModifyStruct &ms);

public:
	wxFloatField(wxPanel *panel, wxFunction func, char *label, char *value = "",
		int x = -1, int y = -1, int width = -1, int height = -1,
		long style = 0, char *name = NULL,
		int length = 6);

};

class wxBoolField : public wxField
{
	static char *yes_;
	static char *no_;
	static int length_;

protected:

	virtual void OnModifyVerify(ModifyStruct &ms);

public:
	wxBoolField(wxPanel *panel, wxFunction func, char *label, char *value = "",
		int x = -1, int y = -1, int width = -1, int height = -1,
		long style = 0, char *name = NULL,
		const char *yes = NULL, const char *no = NULL);

};

class wxDateTimeField : public wxField
{
	enum {
		MaxNum = 3,
		MaxLen = 10
	};
	char value_[MaxLen+1];

	int numbers_;
	int numPos_[MaxNum+1];
	int numLen_[MaxNum];

protected:

	int maxLength() const;

	virtual char delimiter() const = 0;
	virtual Bool validValue() = 0;

	int numbers() const;
	int number(int num) const;
	int minPossibleNumber(int num) const;
	int maxPossibleNumber(int num) const;

	Bool isDelimiterPos(int pos) const;
	Bool addNumber(int len);
	Bool isValidLength();

protected:

	virtual void OnModifyVerify(ModifyStruct &ms);

public:
	wxDateTimeField(wxPanel *panel, wxFunction func, char *label,
		char *value = "",
		int x = -1, int y = -1, int width = -1, int height = -1,
		long style = 0, char *name = NULL);
};

inline int wxDateTimeField::numbers() const		{ return numbers_; }


class wxTimeField : public wxDateTimeField
{
protected:
	virtual char delimiter() const;

	static int maxValidNumber(int index);

	virtual Bool validValue();
public:
	wxTimeField(wxPanel *panel, wxFunction func, char *label, char *value = "",
		int x = -1, int y = -1, int width = -1, int height = -1,
        long style = 0, char *name = NULL, Bool haveSecond = TRUE);

};

class wxDateField : public wxDateTimeField
{
	enum NumType {
		Day,
		Month,
		Year
	};
	NumType numType[3];
	int numIndex[Year];
	Bool fullYear;

protected:
	virtual char delimiter() const;

	int maxValidNumber(int index);
	int minValidNumber(int index);

	virtual Bool validValue();
public:
	wxDateField(wxPanel *panel, wxFunction func, char *label, char *value = "",
		int x = -1, int y = -1, int width = -1, int height = -1,
		long style = 0, char *name = NULL,
        Bool haveYear = TRUE, Bool haveCentury = FALSE);

}; 


/////////////////////////////////////////////////////////////////////////////
///////////                                                   ///////////////
/////////////////////////////////////////////////////////////////////////////


class Checker {

protected:	
	char *oldValueBuffer;
	char *newValueBuffer;
	char *value;

public:
	Checker() { oldValueBuffer = newValueBuffer = value = NULL; }
	Checker( int valueSize )
	{ 
		oldValueBuffer = new char[ valueSize ];
		newValueBuffer = new char[ valueSize ];
		value = new char[ valueSize ];
	}
	
	virtual ~Checker() 
	{
		delete( oldValueBuffer );
		delete( newValueBuffer );
		delete( value );
	}

protected:
	inline void saveOldValue( char *v, int oldLength );
	inline void setNewValue( char *v );
public:	
	inline char *getValue( bool _new = true );  
	
	virtual bool pickValue() const { return false; }
	virtual char *getResult() { return value; }
	virtual void load( char *oldVal, int oldLength, char *newVal, int newLengh,
						int startPos, int endPos );
    
    inline char *_value() { return value; }
    
protected:
	inline char *getNewValue() { return newValueBuffer; }
	inline char *getOldValue() { return oldValueBuffer; }
	virtual bool checkInst() { return true; }
};                              
                          
////////////////////										/////////////////                                         
class NumNode;                                         

class NumChecker : public Checker {     

	int maxLength;
	int decimals;
	bool signed_;
	bool hasComma;
	
	NumNode *node;                           

public:         

	struct Lim {
		int minNum;
		int maxNum;
	};


	NumChecker( int _maxLength, int _decimals = 0, bool _signed = false,
							bool _hasComma = false );
							
	virtual ~NumChecker();
	
	bool pickValue( char &_sign );                     
	inline char *getResult() { return Checker::_value(); }
	
	inline char *getValue() { return Checker::getValue( true ); }
	inline char *getValue( bool _new ) { return Checker::getValue( _new ); }

    inline bool _signed() 
    { 
    	if( signed_ ) return true; 
    	return false; 
   	}
   	inline bool _hasComma()
   	{
   		if( hasComma ) return true;
   		return false;
   	}
   	inline int _decim() { return decimals; }
   	inline int _maxLen() { return maxLength; }                                           
};


/////////////////											/////////////////  
class FloatNode;  
    
class FloatChecker : public Checker {

	int length;

	FloatNode *node;
	
public:
	FloatChecker( int l );
	~FloatChecker();
	
	bool pickValue( char &fSign, char &sSign, int &ePos );

	inline char* getResult() { return Checker::_value(); }
	inline char* getValue() { return Checker::getValue( true ); }
	inline char* getValue( bool _new ) { return Checker::getValue( _new ); }
	
	inline int _maxLen() { return length; }
};

/////////////////											////////////////

    
class NumNode {

protected:
    static int position;
    static bool passed;
    static int numbDot;
    static int dotPos;
    static char sign;
    static int numbSign;
    static int nS;
    static int numbComm;
    static char testchar;                  
    
    static int valLength;
    static NumChecker* val;
	
    NumNode *next;
public:
	NumNode( NumChecker *v, int leng );
	virtual ~NumNode() 
	{   
		delete next; 
	}
                          
	inline bool isPassed() { return passed; }
	inline char theSign() { return sign; }
protected:
	NumNode();
	void eval();
	inline bool hasVal();	
	virtual bool checkPossible() { return true; }
	virtual bool checkPosition() { return true; }
};                                         


class DigNode : public NumNode {
    
    int thePosition;
public:
	DigNode( int p );                             
	~DigNode() {}                                 
protected:
    inline bool checkPosition();       
};
                                          

class SignNode : public NumNode {
    
public:
	SignNode(): NumNode() {}
	~SignNode(){}
protected:             
    inline bool checkPossible() 
    { 
    	if( val->_signed() ) return true; 
    	return false;
   	}
};


class DotNode : public NumNode {    
    int thePosition;
public:
	DotNode( int p );                       
	~DotNode(){}                            
protected:           
    inline bool checkPossible()             
    { 
    	if( val->_decim() > 0 ) return true;
    	return false;
    }
    bool checkPosition();                   
};


class CommNode : public NumNode {
    
public:
	CommNode() {}                        
	~CommNode() {}                            
	
protected:           
    bool checkPossible()                          
    {
    	if( val->_hasComma() ) return true;
    	return false;
    }
};                              


//////////////////											////////////////


class FloatNode {

protected:
	static int position;
	static bool passed;
	static int numbDot;
	static int dotPos;
	static char firstSign;
	static int numbFirstSign;
	static char secondSign;
	static int numbSecondSign;
	static bool getExp;
	static int expPos;
	static char testchar;
	
	static int valLength;
	static FloatChecker* val;
	
	FloatNode *next;
	
public:
	FloatNode( FloatChecker *v, int leng );
	virtual ~FloatNode() { delete next; }
	                          
	inline bool _getExp() { return getExp; }                          
	inline int expPosition() { return expPos; }                  
	inline bool isPassed() { return passed; }
	inline char theFirstSign() { return firstSign; }
	inline char theSecondSign() { return secondSign; }
protected:
	FloatNode();
	void eval();
	inline bool hasVal();
	virtual bool checkPossible() { return true; }
	virtual bool checkPosition() { return true; }

	inline int digitsAfter() { return 4; }
};

                         
class FDigNode : public FloatNode {

	int thePosition;
public:
	FDigNode( int p );
	~FDigNode(){}
protected:
	inline bool checkPosition();
};


class FSignNode : public FloatNode {

public:
    FSignNode() : FloatNode() {}
    ~FSignNode(){}
};    


class FDotNode : public FloatNode {

public:
	FDotNode() : FloatNode() {}
	~FDotNode() {}
};


class ExpNode : public FloatNode {

public:
	ExpNode() : FloatNode() {}
	~ExpNode(){}
	
protected:      
	bool checkPosition();
};


/////////////////////////////////////////////////////////////////////////////
//						test masks definition
/////////////////////////////////////////////////////////////////////////////


#define TST_ANUM	0x01
#define TST_ALPHA	0x02
#define TST_NUM		0x04
#define TST_HEXA	0x08
#define TST_UPPER	0x10
#define TST_LOWER	0x20
#define TST_REQ		0x40
                                         
/*		For output mask		*/
#define TST_OCHK_DIG	0x01 /* Ctrl-A */

#endif // wx_fieldh

