create schema perms descr "Permission Schema" language "C";

table users	descr "Users Table"
{
	name		char(8)		descr "User name"
							not null,
	fullname	char(20)	descr "Full user name",
	gpid		num(5)		descr "Group id"
							not null
							in groups:name,
}
primary key(gpid, name),
unique index name(name);

table groups	descr "Group Table"
{
	id			num(5)		descr "Group id"
							not null,
	name		char(8)		descr "Group name"
							not null,
	fullname	char(20)	descr "Full group name",
	type		num(2)		descr "Group type"
							not null
							in (0:"Sin permisos por default",
								1:"Con permisos por default")
							default 0,
}
primary key(id),
unique index name(name);

table units descr "Unit Table"
{
	id			num(9)		descr "Unit id"
							not null,
	name		char(20)	descr "Unit name"
							not null,
	type		num(2)		descr "Unit type"
							not null
							in (0:"WCMD",
								1:"MENU",
								2:"SHELL",
								3:"PIPE",
								4:"DOFORM"),
	host	char(10)	descr "Host RWCMD",
}
primary key(id),
unique index name(name),
unique index type(type, id);

table insts descr "Instances Table"
{
	unitid	num(9)		descr "Unit id"
						not null
						in units:(name),
	instid	num(4)		descr "Unit instance id",
	descrip	char(30)	descr "Unit description",
	father	num(9)		descr "Menu where it's called"
							in units by type(1):(name),
	args	char(30)	descr "Unit args",
}
primary key(unitid, instid),
unique index father(father, unitid, instid);

table unitperm descr "Permission Table"
{
	unitid	num(9)		descr "Unit id"
						not null
						in units:(name),
	gpid	num(5)		descr "Group id"
						not null
						in groups:(name, fullname),
	perms	num(5)		descr "Permission mask"
						in (null:"Cualquier operacin",
						    0:"Sin permisos",
						    7:"Solo consulta",
						    6:"Solo alta",
						    5:"Solo baja",
						    3:"Solo modificacin",
						    1:"Modificacin y baja",
						    2:"Alta y modificacin",
						    4:"Alta y baja"),
}
primary key(unitid, gpid),
unique index gpid(gpid, unitid);

table modinst descr "Installed Modules Table" {

	id		num(4)		descr "Product Id"
						not null,
	descrip	char(40)	descr "Module description",
	wsch	char(12)	descr "Schema witness",
	version	char(6)		descr "Module version",
	inst	bool		descr "This module is installed ?"
						not null
						default false,
	cdate	date		descr "Creation date stamp",
	ctime	time		descr "Creation time stamp",
	cuid	num(5)		descr "Creation uid stamp",
	mdate	date		descr "Modification date stamp",
	mtime	time		descr "Modification time stamp",
	muid	num(5)		descr "Modification uid stamp",
}
primary key(id);
